@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWBTree.cpp,v 1.22 2005/03/21 06:36:47 vkurland Exp $
+  $Id: FWBTree.cpp,v 1.25 2005/12/07 07:26:31 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -38,6 +38,8 @@
 #include "fwbuilder/Host.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/IPv4.h"
+#include "fwbuilder/DNSName.h"
+#include "fwbuilder/AddressTable.h"
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/ObjectGroup.h"
 #include "fwbuilder/Interface.h"
@@ -49,6 +51,7 @@
 #include "fwbuilder/ServiceGroup.h"
 #include "fwbuilder/Interval.h"
 #include "fwbuilder/IntervalGroup.h"
+#include <fwbuilder/TagService.h>
 
 #include <iostream>
 #include <algorithm>
@@ -61,6 +64,8 @@ FWBTree *FWBTree::standardObjectTreeFormat=NULL;
 const char* systemObjects[] = {
     "Objects",
     "Objects/Addresses",
+    "Objects/DNS Names",
+    "Objects/Address Tables",
     "Objects/Address Ranges",
     "Objects/Groups",
     "Objects/Hosts",
@@ -73,6 +78,7 @@ const char* systemObjects[] = {
     "Services/ICMP",
     "Services/TCP",
     "Services/UDP",
+    "Services/TagServices",
 
     "Firewalls",
 
@@ -91,6 +97,8 @@ FWBTree::FWBTree()
     systemGroupPaths[Library::TYPENAME]       = "";
 
     systemGroupPaths[IPv4::TYPENAME]          = "Objects/Addresses";
+    systemGroupPaths[DNSName::TYPENAME]       = "Objects/DNS Names";
+    systemGroupPaths[AddressTable::TYPENAME]  = "Objects/Address Tables";
     systemGroupPaths[AddressRange::TYPENAME]  = "Objects/Address Ranges";
     systemGroupPaths[ObjectGroup::TYPENAME]   = "Objects/Groups";
     systemGroupPaths[Host::TYPENAME]          = "Objects/Hosts";
@@ -102,6 +110,7 @@ FWBTree::FWBTree()
     systemGroupPaths[ICMPService::TYPENAME]   = "Services/ICMP";
     systemGroupPaths[TCPService::TYPENAME]    = "Services/TCP";
     systemGroupPaths[UDPService::TYPENAME]    = "Services/UDP";
+    systemGroupPaths[TagService::TYPENAME]          = "Services/TagServices";
 
     systemGroupPaths[Firewall::TYPENAME]      = "Firewalls";
 
@@ -121,6 +130,12 @@ FWBTree::FWBTree()
     systemGroupTypes[IPv4::TYPENAME]=          ObjectGroup::TYPENAME;
     systemGroupNames[IPv4::TYPENAME]=          "Addresses"      ;
 
+    systemGroupTypes[DNSName::TYPENAME]=       ObjectGroup::TYPENAME;
+    systemGroupNames[DNSName::TYPENAME]=       "DNS Names"      ;
+
+    systemGroupTypes[AddressTable::TYPENAME]=  ObjectGroup::TYPENAME;
+    systemGroupNames[AddressTable::TYPENAME]=  "Address Tables"      ;
+
     systemGroupTypes[AddressRange::TYPENAME]=  ObjectGroup::TYPENAME;
     systemGroupNames[AddressRange::TYPENAME]=  "Address Ranges" ;
 
@@ -142,6 +157,9 @@ FWBTree::FWBTree()
     systemGroupTypes[UDPService::TYPENAME]=    ServiceGroup::TYPENAME;
     systemGroupNames[UDPService::TYPENAME]=    "UDP"           ;
 
+    systemGroupTypes[TagService::TYPENAME]=          ServiceGroup::TYPENAME;
+    systemGroupNames[TagService::TYPENAME]=          "TagService"           ;
+
     systemGroupTypes[ServiceGroup::TYPENAME]=  ServiceGroup::TYPENAME;
     systemGroupNames[ServiceGroup::TYPENAME]=  "Groups"        ;
 
@@ -194,11 +212,20 @@ FWBTree::FWBTree()
     standardIDs["stdid15_1"]=true;
     standardIDs["stdid16"]  =true;
     standardIDs["stdid16_1"]=true;
+    standardIDs["stdid17"]  =true;
+    standardIDs["stdid17_1"]=true;
+    standardIDs["stdid18"]  =true;
+    standardIDs["stdid18_1"]=true;
+    standardIDs["stdid19"]  =true;
+    standardIDs["stdid19_1"]=true;
+    
 
     copyMenuState[""] = false;
     copyMenuState["Firewalls"] = false;
     copyMenuState["Objects"] = false;
     copyMenuState["Objects/Addresses"] = false;
+    copyMenuState["Objects/DNS Names"] = false;
+    copyMenuState["Objects/Address Tables"] = false;
     copyMenuState["Objects/Address Ranges"] = false;
     copyMenuState["Objects/Groups"] = false;
     copyMenuState["Objects/Hosts"] = false;
@@ -210,12 +237,15 @@ FWBTree::FWBTree()
     copyMenuState["Services/IP"] = false;
     copyMenuState["Services/TCP"] = false;
     copyMenuState["Services/UDP"] = false;
+    copyMenuState["Services/TagServices"] = false;
     copyMenuState["Time"] = false;
 
     cutMenuState[""] = true;
     cutMenuState["Firewalls"] = false;
     cutMenuState["Objects"] = false;
     cutMenuState["Objects/Addresses"] = false;
+    cutMenuState["Objects/DNS Names"] = false;
+    cutMenuState["Objects/Address Tables"] = false;
     cutMenuState["Objects/Address Ranges"] = false;
     cutMenuState["Objects/Groups"] = false;
     cutMenuState["Objects/Hosts"] = false;
@@ -227,12 +257,15 @@ FWBTree::FWBTree()
     cutMenuState["Services/IP"] = false;
     cutMenuState["Services/TCP"] = false;
     cutMenuState["Services/UDP"] = false;
+    cutMenuState["Services/TagServices"] = false;
     cutMenuState["Time"] = false;
 
     pasteMenuState[""] = false;
     pasteMenuState["Firewalls"] = true;
     pasteMenuState["Objects"] = false;
     pasteMenuState["Objects/Addresses"] = true;
+    pasteMenuState["Objects/DNS Names"] = true;
+    pasteMenuState["Objects/Address Tables"] = true;
     pasteMenuState["Objects/Address Ranges"] = true;
     pasteMenuState["Objects/Groups"] = true;
     pasteMenuState["Objects/Hosts"] = true;
@@ -244,12 +277,15 @@ FWBTree::FWBTree()
     pasteMenuState["Services/IP"] = true;
     pasteMenuState["Services/TCP"] = true;
     pasteMenuState["Services/UDP"] = true;
+    pasteMenuState["Services/TagServices"] = true;
     pasteMenuState["Time"] = true;
 
     deleteMenuState[""] = true;
     deleteMenuState["Firewalls"] = false;
     deleteMenuState["Objects"] = false;
     deleteMenuState["Objects/Addresses"] = false;
+    deleteMenuState["Objects/DNS Names"] = false;
+    deleteMenuState["Objects/Address Tables"] = false;
     deleteMenuState["Objects/Address Ranges"] = false;
     deleteMenuState["Objects/Groups"] = false;
     deleteMenuState["Objects/Hosts"] = false;
@@ -261,6 +297,7 @@ FWBTree::FWBTree()
     deleteMenuState["Services/IP"] = false;
     deleteMenuState["Services/TCP"] = false;
     deleteMenuState["Services/UDP"] = false;
+    deleteMenuState["Services/TagServices"] = false;
     deleteMenuState["Time"] = false;
 
 }
@@ -368,6 +405,14 @@ FWObject* FWBTree::createNewLibrary(FWObjectDatabase *db)
     o2->setName("Addresses");
     o1->add(o2);
 
+    o2 = db->create(ObjectGroup::TYPENAME,true);
+    o2->setName("DNS Names");
+    o1->add(o2);
+    
+    o2 = db->create(ObjectGroup::TYPENAME,true);
+    o2->setName("Address Tables");
+    o1->add(o2);
+    
     o2 = db->create(ObjectGroup::TYPENAME,true);
     o2->setName("Groups");
     o1->add(o2);
@@ -412,6 +457,10 @@ FWObject* FWBTree::createNewLibrary(FWObjectDatabase *db)
     o2->setName("Custom");
     o1->add(o2);
 
+    o2 = db->create(ServiceGroup::TYPENAME,true);
+    o2->setName("TagServices");
+    o1->add(o2);
+
     o1 = db->create(ObjectGroup::TYPENAME,true);
     o1->setName("Firewalls");
     nlib->add(o1);
