@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.h,v 1.52.4.1 2005/09/04 20:49:14 vkurland Exp $
+  $Id: FWWindow.h,v 1.73 2006/07/14 06:06:42 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,16 +28,21 @@
 #define  __FWWINDOW_H_
 
 #include "FWBMainWindow_q.h"
+#include "ObjectEditor.h"
 #include "RCS.h"
+#include "FindObjectWidget.h"
 
 #include "fwbuilder/FWObject.h"
 
 #include <qstring.h>
 
 #include <vector>
+#include <list>
 
 namespace libfwbuilder {
     class FWObjectDatabase;
+    class Firewall;
+    class PolicyRule;
 };
 
 class QTabWidget;
@@ -53,19 +58,26 @@ class FWWindow : public FWBMainWindow_q {
     bool                                    systemFile;
     bool                                    editingStandardLib;
     bool                                    editingTemplateLib;
+    bool                                    changingTabs;
 
     std::vector<libfwbuilder::FWObject*>    firewalls;
     std::map<libfwbuilder::FWObject*, RuleSetView*> ruleSetViews;
+    int                                     ruleSetTabIndex;
 
     libfwbuilder::FWObject                 *visibleFirewall;
     libfwbuilder::FWObjectDatabase         *objdb;
+    libfwbuilder::FWObject                 *shownInInfo;
 
+    QWidget                                *editorOwner;
+    
     QTimer                                 *autosaveTimer;
     QString                                 noFirewalls;
     QPrinter                               *printer;
-    
+    libfwbuilder::FWObject                 *searchObject;
+    libfwbuilder::FWObject                 *replaceObject;
+    int                                    lastFirewallIdx;
     void clearFirewallTabs();
-
+    
  public slots:
 
      virtual void search();
@@ -74,6 +86,8 @@ class FWWindow : public FWBMainWindow_q {
      virtual void reopenFirewall();
      virtual void deleteFirewall(libfwbuilder::FWObject *fw);
      virtual void changeInfoStyle();
+     virtual void ruleSetTabChanged(QWidget* w);
+     virtual void restoreRuleSetTab();
 
      virtual void fileNew();
      virtual void fileOpen();
@@ -97,7 +111,9 @@ class FWWindow : public FWBMainWindow_q {
      virtual void helpAbout();
      virtual void debug();
      
+     virtual void compile(std::set<libfwbuilder::Firewall * > vf);
      virtual void compile();
+     virtual void install(std::set<libfwbuilder::Firewall * > vf);
      virtual void install();
 
      virtual void insertRule();
@@ -113,14 +129,28 @@ class FWWindow : public FWBMainWindow_q {
      virtual void pasteRuleBelow();
 
      virtual void newObject();
-     
+     virtual void lockObject();
+     virtual void unlockObject();
+     virtual void prepareObjectMenu();
+     virtual void toolsDiscoveryDruid();
+     virtual void closeAuxiliaryPanel();
+     virtual void closeEditorPanel();
+     virtual void openEditorPanel();
+
+     virtual void rollBackSelectionSameWidget();
+     virtual void rollBackSelectionDifferentWidget();
+
+ signals:
+     void restoreSelection_sign(bool same_widget);
+
  public:
+    FindObjectWidget *findObjectWidget;
     
     FWWindow();
     ~FWWindow();
 
     virtual void closeEvent( QCloseEvent * );
-    
+    RCS * getRCS();
         
     void load(QWidget *dialogs_parent,RCS *rcs);
     void load(QWidget *dialogs_parent);
@@ -135,7 +165,7 @@ class FWWindow : public FWBMainWindow_q {
 
     bool editingLibrary();
 
-    void ensureObjectVisible(libfwbuilder::FWReference *obj);
+    void ensureObjectVisibleInRules(libfwbuilder::FWReference *obj);
 
     QString chooseNewFileName(const QString &fname,
                               bool checkPresence, const QString &title);
@@ -160,11 +190,37 @@ class FWWindow : public FWBMainWindow_q {
     libfwbuilder::FWObjectDatabase* db() { return objdb; }
 
     libfwbuilder::FWObject* getVisibleFirewall() { return visibleFirewall; }
-
+    QString getCurrentFileName();
+    
     void info(libfwbuilder::FWObject *o);
-    int  doCompile(const QString &closeButtonText="");
 
     void setupAutoSave();
+    void findObject(libfwbuilder::FWObject *);
+    RuleSetView* getRuleSetViews(libfwbuilder::FWObject *o) 
+    {return ruleSetViews[o];};
+
+    void addPolicyBranchTab(libfwbuilder::PolicyRule *rule);
+    void removePolicyBranchTab(libfwbuilder::PolicyRule *rule);
+    void setPolicyBranchTabName(libfwbuilder::PolicyRule *rule);
+
+    /**
+     * panel that wants to open an object in the editor
+     * uses this method to request permission to do so and
+     * to register itself as an owner of the editor
+     */
+    bool requestEditorOwnership(QWidget *w,
+                                libfwbuilder::FWObject *o,
+                                ObjectEditor::OptType   otype,
+                                bool validate = true);
+    void releaseEditor();
+    void connectEditor(QWidget *w);
+    bool exportLibraryTest(std::list<libfwbuilder::FWObject*> &selectedLibs);
+    void exportLibraryTo(QString fname,std::list<libfwbuilder::FWObject*> &selectedLibs, bool rof);
+
+    void findExternalRefs(libfwbuilder::FWObject *lib,
+                       libfwbuilder::FWObject *root,
+                       std::list<libfwbuilder::FWReference*> &extRefs);
+    
     
  protected:
 
