@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FirewallDialog.cpp,v 1.29.4.1 2005/10/25 06:11:26 vkurland Exp $
+  $Id: FirewallDialog.cpp,v 1.39 2006/06/14 07:03:15 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -53,6 +53,8 @@
 #include <qpushbutton.h>
 #include <qiconview.h>
 #include <qtabwidget.h>
+#include <qlabel.h>
+#include <qtimer.h>
 
 #include <iostream>
 
@@ -67,9 +69,6 @@ void FirewallDialog::loadFWObject(FWObject *o)
 
     init=true;
 
-/* hide tab "Templates" */
-    dialogTabWidget->removePage( dialogTabWidget->page(1) );
-    
     fillLibraries(libs,obj);
 
 /* fill in platform */
@@ -88,13 +87,15 @@ void FirewallDialog::loadFWObject(FWObject *o)
 //    FWOptions  *opt =s->getOptionsObject();
 
     obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
-    snmpCommunity->setText( mgmt->getSNMPManagement()->getReadCommunity().c_str() );
+//    snmpCommunity->setText( mgmt->getSNMPManagement()->getReadCommunity().c_str() );
 
     comment->setTextFormat(QTextEdit::PlainText);
 
     comment->setText( QString::fromUtf8(s->getComment().c_str()) );
 
-    apply->setEnabled( false );
+    inactive->setChecked(s->getInactive());
+
+    //apply->setEnabled( false );
 
 
     obj_name->setEnabled(!o->isReadOnly());
@@ -121,12 +122,12 @@ void FirewallDialog::loadFWObject(FWObject *o)
     comment->setReadOnly(o->isReadOnly());
     setDisabledPalette(comment);
 
-    snmpCommunity->setEnabled(!o->isReadOnly());
-    setDisabledPalette(snmpCommunity);
-
-    fwTemplatesView->setEnabled(!o->isReadOnly());
-    setDisabledPalette(fwTemplatesView);
+//    snmpCommunity->setEnabled(!o->isReadOnly());
+//    setDisabledPalette(snmpCommunity);
 
+    inactive->setEnabled(!o->isReadOnly());
+    setDisabledPalette(inactive);
+    
 
     init=false;
 }
@@ -136,13 +137,16 @@ void FirewallDialog::fillVersion()
 {
     version->clear();
 
-    QMap<QString,QString> vl=getVersionsForPlatform( readPlatform(platform) );
-    QString                v=obj->getStr("version").c_str();
+    list<QStringPair> vl=getVersionsForPlatform( readPlatform(platform) );
+    QString            v=obj->getStr("version").c_str();
     int cp=0;
-    for (QMap<QString,QString>::iterator i1=vl.begin(); i1!=vl.end(); i1++,cp++)
+    for (list<QStringPair>::iterator i1=vl.begin(); i1!=vl.end(); i1++,cp++)
     {
-        version->insertItem( i1.data() );
-        if ( v == i1.key() ) version->setCurrentItem( cp );
+        if (fwbdebug)
+            qDebug(QString("Adding version %1").arg(i1->second));
+
+        version->insertItem( i1->second );
+        if ( v == i1->first ) version->setCurrentItem( cp );
     }
 }
 
@@ -150,10 +154,16 @@ void FirewallDialog::saveVersion()
 {
     QString pl = readPlatform(platform);
 
-    QMap<QString,QString> vl=getVersionsForPlatform( pl.latin1() );
-    QString               v = version->currentText();
+    list<QStringPair> vl=getVersionsForPlatform( pl.latin1() );
+    QString           v = version->currentText();
+    list<QStringPair>::iterator li =
+        std::find_if(vl.begin(),vl.end(),findSecondInQStringPair(v));
+    if (li!=vl.end())
+        obj->setStr("version", li->first.latin1() );
+
+#if 0
     int cp=0;
-    for (QMap<QString,QString>::iterator i1=vl.begin(); i1!=vl.end(); i1++,cp++)
+    for (list<QStringPair>::iterator i1=vl.begin(); i1!=vl.end(); i1++,cp++)
     {
         if ( v == i1.data() )
         {
@@ -161,7 +171,7 @@ void FirewallDialog::saveVersion()
             break;
         }
     }
-    
+#endif
 }
 
 void FirewallDialog::platformChanged()
@@ -188,7 +198,8 @@ void FirewallDialog::hostOSChanged()
 
 void FirewallDialog::changed()
 {
-    apply->setEnabled( true );
+    //apply->setEnabled( true );
+    emit changed_sign();
 }
 
 void FirewallDialog::validate(bool *res)
@@ -198,7 +209,9 @@ void FirewallDialog::validate(bool *res)
 
 void FirewallDialog::isChanged(bool *res)
 {
-    *res=(!init && apply->isEnabled());
+    if (fwbdebug)
+        qDebug("FirewallDialog::isChanged");
+    //*res=(!init && apply->isEnabled());
 }
 
 void FirewallDialog::libChanged()
@@ -227,20 +240,32 @@ void FirewallDialog::applyChanges()
     string newname=string(obj_name->text().utf8());
     string oldplatform=obj->getStr("platform");
 
+    string oldVer=obj->getStr("version");
+
     obj->setName( newname );
     obj->setComment( string(comment->text().utf8()) );
-    mgmt->getSNMPManagement()->setReadCommunity( snmpCommunity->text().latin1() );
+//    mgmt->getSNMPManagement()->setReadCommunity( snmpCommunity->text().latin1() );
 
     string pl = readPlatform(platform).latin1();
     obj->setStr("platform", pl );
 
     obj->setStr("host_OS", readHostOS(hostOS).latin1() );
-
+    
+    s->setInactive(inactive->isChecked());
+    
     saveVersion();
 
+    string newVer=obj->getStr("version");
+        
     om->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
 
-    if (oldplatform!=pl || oldname!=newname)  mw->reopenFirewall();
+    if (oldplatform!=pl || oldname!=newname || oldVer!=newVer)
+    {
+        if (fwbdebug)
+            qDebug("FirewallDialog::applyChanges() scheduling call to reopenFirewall()");
+        //mw->reopenFirewall();
+        QTimer::singleShot( 0, mw, SLOT(reopenFirewall()) );
+    }
 
     init=true;
 
@@ -250,7 +275,9 @@ void FirewallDialog::applyChanges()
 
     init=false;
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
+    om->updateLastModifiedTimestampForAllFirewalls(s);
+
 }
 
 void FirewallDialog::discardChanges()
@@ -260,7 +287,7 @@ void FirewallDialog::discardChanges()
 
 void FirewallDialog::openFWDialog()
 {
-    if (apply->isEnabled()) applyChanges();
+    /*if (apply->isEnabled())*/ applyChanges();
 
     if (obj->getStr("version").empty()) saveVersion();
 
@@ -287,7 +314,7 @@ void FirewallDialog::openFWDialog()
 
 void FirewallDialog::openOSDialog()
 {
-    if (apply->isEnabled()) applyChanges();
+    /*if (apply->isEnabled())*/ applyChanges();
 
     try
     {
