@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: GroupObjectDialog.cpp,v 1.43 2005/04/17 21:58:55 vkurland Exp $
+  $Id: GroupObjectDialog.cpp,v 1.52 2006/06/12 06:55:05 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,11 +28,13 @@
 #include "global.h"
 #include "utils.h"
 
+#include "fwbuilder/FWObjectDatabase.h"
+#include "FWWindow.h"
 #include "FWBTree.h"
 #include "FWBSettings.h"
 #include "FWObjectPropertiesFactory.h"
 #include "GroupObjectDialog.h"
-#include "ObjectTreeViewItem.h"
+#include "ObjectListViewItem.h"
 #include "ObjectIconViewItem.h"
 #include "ObjectManipulator.h"
 #include "FWObjectDrag.h"
@@ -111,7 +113,7 @@ void ListViewObjToolTip::maybeTip(const QPoint &pos)
     {
         QListViewItem      *itm   = view->itemAt( pos );
         if (itm==NULL) return;
-        ObjectTreeViewItem *otvi  = dynamic_cast<ObjectTreeViewItem*>(itm);
+        ObjectListViewItem *otvi  = dynamic_cast<ObjectListViewItem*>(itm);
         assert(otvi!=NULL);
 
         FWObject *obj     = otvi->getFWObject();
@@ -141,27 +143,37 @@ GroupObjectDialog::GroupObjectDialog(QWidget *parent) : GroupObjectDialog_q(pare
     listView->addColumn( tr( "Properties" ) );
     listView->setAcceptDrops( TRUE );
 
+
     new IconViewObjToolTip(iconView);
     new ListViewObjToolTip(listView);
 
 
-    QGridLayout *WStackPageLayout = new QGridLayout( WStackPage, 1, 1, 0, 0, "WStackPageLayout"); 
+    //QGridLayout *WStackPageLayout = new QGridLayout( WStackPage, 1, 1, 0, 0, "WStackPageLayout"); 
 
-    WStackPageLayout->addWidget( iconView, 0, 0 );
-    WStackPageLayout->addWidget( listView, 0, 0 );
-    objectViewsStack->addWidget( WStackPage, 0 );
+    //WStackPageLayout->addWidget( iconView, 0, 0 );
+    //WStackPageLayout->addWidget( listView, 0, 0 );
+    //objectViewsStack->addWidget( WStackPage, 0 );
 
+    objectViewsStack->addWidget(iconView);
+    objectViewsStack->addWidget(listView);
+    objectViewsStack->raiseWidget(iconView);
+
+    
     setTabOrder( obj_name, iconView );
     setTabOrder( iconView, listView );
     setTabOrder( listView, comment );
 
-    listView->hide();
-    iconView->show();
+    listView->setSelectionMode(QListView::Extended);
+    iconView->setSelectionMode(QIconView::Extended);
+    
+    //listView->hide();
+    //iconView->show();
+    
 
     iconViewBtn->setAutoRaise(false);
     listViewBtn->setAutoRaise(false);
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
 
     connect( iconView, SIGNAL( doubleClicked(QIconViewItem*) ),
              this,     SLOT( openObject(QIconViewItem*) ) );
@@ -172,6 +184,9 @@ GroupObjectDialog::GroupObjectDialog(QWidget *parent) : GroupObjectDialog_q(pare
     connect( iconView, SIGNAL( currentChanged(QIconViewItem*) ),
              this,     SLOT( iconViewCurrentChanged(QIconViewItem*) ) );
 
+    connect( iconView, SIGNAL (selectionChanged()),
+            this,      SLOT (iconViewSelectionChanged())); 
+
     connect( iconView, SIGNAL( dropped(QDropEvent*,const QValueList<QIconDragItem>&) ),
              this,     SLOT( dropped(QDropEvent*) ) );
 
@@ -189,6 +204,8 @@ GroupObjectDialog::GroupObjectDialog(QWidget *parent) : GroupObjectDialog_q(pare
              this,     SLOT( openObject(QListViewItem*) ) );
     connect( listView, SIGNAL( currentChanged(QListViewItem*) ),
              this,     SLOT( listViewCurrentChanged(QListViewItem*) ) );
+    connect( listView, SIGNAL (selectionChanged()),
+            this,      SLOT (listViewSelectionChanged())); 
 
     connect( listView, SIGNAL( dropped(QDropEvent*) ),
              this,     SLOT( dropped(QDropEvent*) ) );
@@ -210,6 +227,44 @@ GroupObjectDialog::GroupObjectDialog(QWidget *parent) : GroupObjectDialog_q(pare
 
 }
 
+void GroupObjectDialog::iconViewSelectionChanged()
+{
+    selectedObjects.clear();
+    QIconViewItem *item;
+    for ( item = iconView->firstItem(); item; item = item->nextItem() )
+    {
+        if(item->isSelected())
+        {
+            ObjectIconViewItem *oivi=dynamic_cast<ObjectIconViewItem*>(item);
+            assert(oivi!=NULL);
+            FWObject *o = oivi->getFWObject();
+            if (o!=NULL)
+                selectedObjects.push_back(o);
+        }
+        
+    }
+        
+}
+void GroupObjectDialog::listViewSelectionChanged()
+{
+    selectedObjects.clear();
+    QListViewItemIterator it(listView);
+    while(it.current())
+    {
+        if(it.current()->isSelected())
+        {
+            QListViewItem *itm= it.current();
+            ObjectListViewItem *otvi=dynamic_cast<ObjectListViewItem*>(itm);
+            assert(otvi!=NULL);
+
+            FWObject *o=otvi->getFWObject();
+            assert(o!=NULL);
+            selectedObjects.push_back(o);
+        }
+        ++it;
+    }
+}
+
 void GroupObjectDialog::iconViewCurrentChanged(QIconViewItem *itm)
 {
     if (itm==NULL)
@@ -236,7 +291,7 @@ void GroupObjectDialog::listViewCurrentChanged(QListViewItem *itm)
         return;
     }
 
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+    ObjectListViewItem *otvi=dynamic_cast<ObjectListViewItem*>(itm);
     assert(otvi!=NULL);
 
     FWObject *o = otvi->getFWObject();
@@ -250,6 +305,7 @@ void GroupObjectDialog::listViewCurrentChanged(QListViewItem *itm)
  */
 void GroupObjectDialog::insertObject(FWObject *o)
 {
+    assert(o!=NULL);
     Group *g = dynamic_cast<Group*>(obj);
     assert(g!=NULL);
 
@@ -262,16 +318,18 @@ void GroupObjectDialog::insertObject(FWObject *o)
 /* avoid duplicates */
     string cp_id=o->getId();
 
-    map<FWObject*, ObjectTreeViewItem*>::iterator i;
+    map<string, ObjectListViewItem*>::iterator i;
     for (i=allListViewItems.begin(); i!=allListViewItems.end(); ++i)
     {
-        FWObject *go=(*i).first;
-        if (FWReference::cast(go)!=NULL) go=FWReference::cast(go)->getPointer();
-        if (o==go || o->getId()==go->getId()) return;
+        string go=(*i).first;
+        //if (FWReference::cast(go)!=NULL) go=FWReference::cast(go)->getPointer();
+        //if (o==go || o->getId()==go->getId()) return;
+	if(go==cp_id) return;
     }
 
     addIcon(o, ! FWBTree::isSystem(obj) );
 
+    
     changed();
 }
 
@@ -295,7 +353,7 @@ void GroupObjectDialog::addIcon(FWObject *o,bool ref)
             o->getTypeName() + "/hidden") ) return;
 
     QString obj_name=QString::fromUtf8(o->getName().c_str());
-
+    
     QString icn_filename =
         Resources::global_res->getObjResourceStr(o, (ref)?"icon-ref":"icon").c_str();
 
@@ -306,11 +364,11 @@ void GroupObjectDialog::addIcon(FWObject *o,bool ref)
     ivitm->setProperty("type", o->getTypeName().c_str() );
     ivitm->setFWObject( o );
 
-    allIconViewItems[o]=ivitm;
+    allIconViewItems[o->getId()]=ivitm;
 
     icn_filename = Resources::global_res->getObjResourceStr(o, "icon-tree").c_str();
 
-    ObjectTreeViewItem *tvitm=new ObjectTreeViewItem( listView );
+    ObjectListViewItem *tvitm=new ObjectListViewItem( listView );
     tvitm->setText( 0, obj_name );
     tvitm->setText( 1, FWObjectPropertiesFactory::getObjectProperties(o) );
     tvitm->setPixmap( 0, QPixmap::fromMimeSource( icn_filename ) );
@@ -320,7 +378,7 @@ void GroupObjectDialog::addIcon(FWObject *o,bool ref)
     tvitm->setProperty("type", o->getTypeName().c_str() );
     tvitm->setFWObject( o );
 
-    allListViewItems[o]=tvitm;
+    allListViewItems[o->getId()]=tvitm;
 }
 
 void GroupObjectDialog::loadFWObject(FWObject *o)
@@ -356,14 +414,16 @@ void GroupObjectDialog::loadFWObject(FWObject *o)
     {
     case Icon:
         if ( ! iconViewBtn->isOn() ) iconViewBtn->toggle();
-        listView->hide();
-        iconView->show();
+        //listView->hide();
+        //iconView->show();
+        objectViewsStack->raiseWidget(iconView);
         break;
 
     case List:
         if ( ! listViewBtn->isOn() ) listViewBtn->toggle();
-        listView->show();
-        iconView->hide();
+        //listView->show();
+        //iconView->hide();
+        objectViewsStack->raiseWidget(listView);
         break;
     }
 
@@ -371,7 +431,7 @@ void GroupObjectDialog::loadFWObject(FWObject *o)
         addIcon( *i );
 
     
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
 
     obj_name->setEnabled(!o->isReadOnly() && !FWBTree::isSystem(o));
     setDisabledPalette(obj_name);
@@ -394,7 +454,9 @@ void GroupObjectDialog::loadFWObject(FWObject *o)
     
 void GroupObjectDialog::changed()
 {
-    if (!init) apply->setEnabled( true );
+    //if (!init) apply->setEnabled( true );
+
+    emit changed_sign();
 }
 
 void GroupObjectDialog::validate(bool *res)
@@ -404,7 +466,7 @@ void GroupObjectDialog::validate(bool *res)
 
 void GroupObjectDialog::isChanged(bool *res)
 {
-    *res=(!init && apply->isEnabled());
+    //*res=(!init && apply->isEnabled());
 }
 
 void GroupObjectDialog::libChanged()
@@ -414,6 +476,9 @@ void GroupObjectDialog::libChanged()
 
 void GroupObjectDialog::applyChanges()
 {
+    if (fwbdebug)
+        qDebug("GroupObjectDialog::applyChanges");
+
     if (!isTreeReadWrite(this,obj)) return;
     if (!validateName(this,obj,obj_name->text())) return;
 
@@ -425,10 +490,10 @@ void GroupObjectDialog::applyChanges()
 
     set<FWObject*> oldobj;
     set<FWObject*> newobj;
-    map<FWObject*, ObjectTreeViewItem*>::iterator i;
+    map<string, ObjectListViewItem*>::iterator i;
     for (i=allListViewItems.begin(); i!=allListViewItems.end(); ++i)
     {
-        newobj.insert( (*i).first );
+        newobj.insert( mw->db()->findInIndex((*i).first) );
     }
 
     for (FWObject::iterator j=obj->begin(); j!=obj->end(); ++j)
@@ -476,7 +541,11 @@ void GroupObjectDialog::applyChanges()
 
     init=false;
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
+    om->updateLastModifiedTimestampForAllFirewalls(obj);
+
+    if (fwbdebug)
+        qDebug("GroupObjectDialog::applyChanges done");
 }
 
 void GroupObjectDialog::discardChanges()
@@ -491,8 +560,9 @@ void GroupObjectDialog::switchToIconView()
 
     if ( ! iconViewBtn->isOn() ) iconViewBtn->toggle();
 
-    listView->hide();
-    iconView->show();
+    //listView->hide();
+    //iconView->show();
+    objectViewsStack->raiseWidget(iconView);
 
     st->setGroupViewMode(ICON_VIEW_MODE);
 }
@@ -504,8 +574,9 @@ void GroupObjectDialog::switchToListView()
 
     if ( ! listViewBtn->isOn() ) listViewBtn->toggle();
 
-    listView->show();
-    iconView->hide();
+    //listView->show();
+    //iconView->hide();
+    objectViewsStack->raiseWidget(listView);
 
     st->setGroupViewMode(LIST_VIEW_MODE);
 }
@@ -515,20 +586,20 @@ void GroupObjectDialog::openObject()
     if (selectedObject!=NULL)
     {
         om->openObject( selectedObject );
-        om->edit( selectedObject );
+        om->editObject( selectedObject );
     }
 }
 
 void GroupObjectDialog::openObject(QListViewItem *itm)
 {
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+    ObjectListViewItem *otvi=dynamic_cast<ObjectListViewItem*>(itm);
     assert(otvi!=NULL);
 
     FWObject *o = otvi->getFWObject();
     if (o!=NULL)
     {
         om->openObject( o );
-        om->edit( o );
+        om->editObject( o );
     }
 }
 
@@ -541,7 +612,7 @@ void GroupObjectDialog::openObject(QIconViewItem *itm)
     if (o!=NULL)
     {
         om->openObject( o );
-        om->edit( o );
+        om->editObject( o );
     }
 }
 
@@ -570,7 +641,7 @@ void GroupObjectDialog::iconContextMenu(QIconViewItem *itm,const QPoint &p)
 void GroupObjectDialog::listContextMenu(QListViewItem *itm,const QPoint &p,int col)
 {
     FWObject *o=NULL;
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+    ObjectListViewItem *otvi=dynamic_cast<ObjectListViewItem*>(itm);
     if (otvi!=NULL) o = otvi->getFWObject();
     selectedObject=o;
 
@@ -614,41 +685,39 @@ void GroupObjectDialog::setupPopupMenu(const QPoint &pos)
 
 void GroupObjectDialog::copyObj()
 {
-    if (selectedObject!=NULL && ! FWBTree::isSystem(selectedObject) ) 
+    FWObjectClipboard::obj_clipboard->clear();
+    for(vector<FWObject*>::iterator it=selectedObjects.begin();
+            it!=selectedObjects.end(); ++it)
     {
-        FWObject *o=selectedObject;
-        if (FWReference::cast(o)!=NULL)
-            o=FWReference::cast(o)->getPointer();
+        FWObject* selectedObject=*it;
+        
+        if (selectedObject!=NULL && ! FWBTree::isSystem(selectedObject) ) 
+        {
+            FWObject *o=selectedObject;
+            if (FWReference::cast(o)!=NULL)
+                o=FWReference::cast(o)->getPointer();
+            
 
-        FWObjectClipboard::obj_clipboard->clear();
-        FWObjectClipboard::obj_clipboard->add( o );
+            FWObjectClipboard::obj_clipboard->add( o );
+        }
+        
     }
 }
 
 void GroupObjectDialog::cutObj()
 {
-    if (selectedObject!=NULL &&  ! FWBTree::isSystem(obj) )
-    {
-        FWObject *o=selectedObject;
-        if (FWReference::cast(o)!=NULL)
-            o=FWReference::cast(o)->getPointer();
-
-        FWObjectClipboard::obj_clipboard->clear();
-        FWObjectClipboard::obj_clipboard->add( o );
-
-        deleteObj();
-    }
-
+    copyObj();
+    deleteObj();
 }
 
 void GroupObjectDialog::pasteObj()
 {
     vector<FWObject*>::iterator i;
+    
     for (i= FWObjectClipboard::obj_clipboard->begin();
          i!=FWObjectClipboard::obj_clipboard->end(); ++i)
     {
-        FWObject *co= *i;
-        insertObject( co );
+        insertObject( *i );
     }
 
 //    if (FWObjectClipboard::obj_clipboard->getObject()==NULL) return;
@@ -657,29 +726,40 @@ void GroupObjectDialog::pasteObj()
 
 void GroupObjectDialog::deleteObj()
 {
-    if (selectedObject!=NULL &&  ! FWBTree::isSystem(obj) )
+    vector<FWObject*> tv;
+    FWObject* selectedObject;
+    copy (selectedObjects.begin(),selectedObjects.end(),inserter(tv,tv.begin()));
+    
+    for(vector<FWObject*>::iterator it=tv.begin();
+            it!=tv.end(); ++it)
     {
-        Group *g = dynamic_cast<Group*>(obj);
-        assert(g!=NULL);
+        selectedObject=(*it);
+        
+        if (selectedObject!=NULL &&  ! FWBTree::isSystem(obj) )
+        {
+            Group *g = dynamic_cast<Group*>(obj);
+            assert(g!=NULL);
 
-        FWObject *o=selectedObject;
-        if (FWReference::cast(o)!=NULL)
-            o=FWReference::cast(o)->getPointer();
+            FWObject *o=selectedObject;
+            assert(o!=NULL);
+            if (FWReference::cast(o)!=NULL)
+                o=FWReference::cast(o)->getPointer();
 
-        selectedObject=NULL;
+            selectedObject=NULL;
 
-//        g->removeRef(o);
+    //        g->removeRef(o);
 
-        assert(allListViewItems[o]!=NULL);
-        delete allListViewItems[o];
-        allListViewItems.erase(o);
+            assert(allListViewItems[o->getId()]!=NULL);
+            delete allListViewItems[o->getId()];
+            allListViewItems.erase(o->getId());
 
-        assert(allIconViewItems[o]!=NULL);
-        delete allIconViewItems[o];
-        allIconViewItems.erase(o);
-
-        changed();
+            assert(allIconViewItems[o->getId()]!=NULL);
+            delete allIconViewItems[o->getId()];
+            allIconViewItems.erase(o->getId());
+            
+        }
     }
+    changed();
 }
 
 /* ObjectEditor class connects its slot to this signal and does all
@@ -699,3 +779,10 @@ void GroupObjectDialog::closeEvent(QCloseEvent *e)
 
 }
 
+void GroupObjectDialog::selectObject(FWObject *o)
+{
+//    ObjectListViewItem* list_item=allListViewItems[o->getId()];
+    ObjectIconViewItem* icon_item=allIconViewItems[o->getId()];
+
+    iconView->setSelected(icon_item,true,false);
+}
