@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectEditor.h,v 1.12 2004/12/11 07:54:30 vkurland Exp $
+  $Id: ObjectEditor.h,v 1.20 2006/06/16 04:33:13 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -41,6 +41,7 @@
 class ObjectTreeViewItem;
 class QComboBox;
 class QPopupMenu;
+class QWidgetStack;
 
 class ObjectEditor : public QObject {
 
@@ -51,27 +52,50 @@ class ObjectEditor : public QObject {
     
     libfwbuilder::FWObject          *opened;
     int                              visible;
-
+    QWidgetStack *parentWidget;
+    QPushButton *closeButton;
+    QPushButton *applyButton;
 
     void disconnectSignals();
     
-public:
+public: 
+    enum OptType{optAction,optComment,optMetric,optNone};
+private: 
+   OptType  openedOpt;
     
+
+public:
+   
     ObjectEditor( QWidget *parent );
     virtual ~ObjectEditor() {}
+    
 
+    QString getOptDialogName(OptType t);
     void open(libfwbuilder::FWObject *o);
+    void openOpt(libfwbuilder::FWObject *, OptType t);
     void show();
     void hide();
     bool isVisible();
-    
-    libfwbuilder::FWObject* getOpened() { return opened; }
+    bool isModified();
+
+    libfwbuilder::FWObject* getOpened() { return opened; };
+    OptType getOpenedOpt() {return openedOpt;};
 
     bool validateAndSave();
+    void setCloseButton(QPushButton * b);
+    void setApplyButton(QPushButton * b);
+    void selectObject(libfwbuilder::FWObject *o);
+    void selectionChanged(libfwbuilder::FWObject *o);
+    void actionChanged(libfwbuilder::FWObject *o);
 
+    
 public slots:
     void validateAndClose(QCloseEvent *e);
- 
+    void apply();
+    void discard();
+    void close();
+    void changed();
+    void blank();
 signals:
 
 /**
