@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectManipulator.h,v 1.36 2005/04/17 21:58:55 vkurland Exp $
+  $Id: ObjectManipulator.h,v 1.53 2006/06/12 06:55:05 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -38,13 +38,16 @@
 
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/FWObjectDatabase.h"
+#include "fwbuilder/ObjectGroup.h"
 
 #include <stack>
+#include <set>
 
 class ObjectTreeView;
 class ObjectTreeViewItem;
 class QComboBox;
 class QPopupMenu;
+class libfwbuilder::Firewall;
 
 class HistoryItem {
     ObjectTreeViewItem *itm;
@@ -74,6 +77,8 @@ class ObjectManipulator : public ObjectManipulator_q {
     std::vector<QListView*>                       idxToTrees;
     
     std::stack<HistoryItem>                       history;
+    int searchId;
+    int cacheHits;
 
     libfwbuilder::FWObject                        *currentObj;
     ObjectTreeView                                *currentTreeView;
@@ -111,17 +116,37 @@ class ObjectManipulator : public ObjectManipulator_q {
                                                  const QString &objName,
                                                  libfwbuilder::FWObject *copyFrom=NULL);
     void autorename(libfwbuilder::FWObject *obj,bool ask=true);
-    void autorename(libfwbuilder::FWObject *obj,
-                    const std::string &objtype,
-                    const std::string &namesuffix);
-    
- public slots:
+    void extractFirewallsFromGroup(
+            libfwbuilder::ObjectGroup *gr,
+            std::set<libfwbuilder::Firewall*> &fo);
+    void findObjectsInGroup(
+            libfwbuilder::Group *g,
+            std::set<libfwbuilder::FWObject *> &res);
+    void _findObjectsInGroup(
+            libfwbuilder::Group *g,
+            std::set<libfwbuilder::FWObject *> &res);
+
+public slots:
      virtual void libChanged(int l);
 
      void selectionChanged();
 
      void info();
-     void edit(libfwbuilder::FWObject *obj);
+
+     /**
+      * open object obj in the editor. Does not open editor panel
+      * if it is closed. Asks FWWindow permission to own editor.
+      */
+     bool switchObjectInEditor(libfwbuilder::FWObject *obj);
+
+     /**
+      * same as above but opens editor panel if it is closed. This is
+      * an entry point for menu items 'edit', all 'new object' as well
+      * as doubleclick
+      */
+     bool editObject(libfwbuilder::FWObject *obj);
+
+     void editSelectedObject();
      
      void contextMenu(QListViewItem *item, const QPoint &pos, int col);
 
@@ -145,17 +170,19 @@ class ObjectManipulator : public ObjectManipulator_q {
      void newPhysicalAddress();
      void newAddressRange();
      void newObjectGroup();
+     void newDNSName();
+     void newAddressTable();
      
      void newCustom();
      void newIP();
      void newICMP();
      void newTCP();
      void newUDP();
+     void newTagService();
      void newServiceGroup();
      
      void newInterval();
 
-     void editObj();
      void duplicateObj(int libid);
      void duplicateObjUnderSameParent();
      void moveObj(int libid);
@@ -171,12 +198,24 @@ class ObjectManipulator : public ObjectManipulator_q {
      
      void openObject(QListViewItem *otvi);
      void openObject(libfwbuilder::FWObject *obj);
+     void restoreSelection(bool same_widget);
 
      void find();
+     void findObject();
      
      virtual void back();
-    
+     virtual void lockObject();
+     virtual void unlockObject();
+     virtual void simulateInstall();
+
+     
  public:
+    void filterFirewallsFromSelection(
+            std::vector<libfwbuilder::FWObject*> &so,
+            std::set<libfwbuilder::Firewall*> &fo);
+    void autorename(libfwbuilder::FWObject *obj,
+                    const std::string &objtype,
+                    const std::string &namesuffix);
     
     ObjectManipulator( QWidget *parent );
     void loadObjects();
@@ -213,6 +252,7 @@ class ObjectManipulator : public ObjectManipulator_q {
                        bool  askForAutorename=true);
     
     ObjectTreeView* getCurrentObjectTree();
+    libfwbuilder::FWObject* getSelectedObject();
 
     /**
      *  this method opens given library in the tree
@@ -271,6 +311,25 @@ class ObjectManipulator : public ObjectManipulator_q {
                       bool &delMenuItem,
                       bool &newMenuItem,
                       bool &inDeletedObjects);
+    
+    void updateLastModifiedTimestampForOneFirewall(libfwbuilder::FWObject *o);
+    void updateLastModifiedTimestampForAllFirewalls(libfwbuilder::FWObject *o);   
+    void updateLastInstalledTimestamp(libfwbuilder::FWObject *o);
+    void updateLastCompiledTimestamp(libfwbuilder::FWObject *o);
+    
+    void findWhereUsed(
+            libfwbuilder::FWObject *o,
+            libfwbuilder::FWObject *p,
+            std::set<libfwbuilder::FWObject *> &resset);
+
+    bool _findWhereUsed(
+            libfwbuilder::FWObject *o,
+            libfwbuilder::FWObject *p,
+            std::set<libfwbuilder::FWObject *> &resset);
+    
+    std::list<libfwbuilder::Firewall * > findFirewallsForObject(libfwbuilder::FWObject *o);
+    void findFirewallsInCurrentLib (std::list<libfwbuilder::Firewall *> &fws);
+
  signals:
 /**
  * the dialog class should have a slot that can load object's data
