@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectTreeView.cpp,v 1.25 2005/04/17 22:34:07 vkurland Exp $
+  $Id: ObjectTreeView.cpp,v 1.34 2006/07/10 00:26:08 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,9 +32,12 @@
 #include "ObjectTreeView.h"
 #include "ObjectTreeViewItem.h"
 #include "ObjectManipulator.h"
+#include "ObjectEditor.h"
 #include "FWObjectDrag.h"
+#include "FWWindow.h"
 
 #include "fwbuilder/FWObject.h"
+#include "fwbuilder/Firewall.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Group.h"
 
@@ -69,6 +72,9 @@ ObjectTreeView::ObjectTreeView(QWidget* parent, const char * name, WFlags f) :
     lastSelected = NULL;
     second_click = false;
     selectionFrozen = false;
+    expandOrCollapse = false;
+    Lockable = false;
+    Unlockable = false;
 
     connect( this, SIGNAL(currentChanged(QListViewItem*)),
              this, SLOT(currentChanged(QListViewItem*)) );
@@ -76,9 +82,17 @@ ObjectTreeView::ObjectTreeView(QWidget* parent, const char * name, WFlags f) :
     connect( this, SIGNAL( selectionChanged() ),
              this, SLOT( selectionChanged() ) );
 
+    connect(this, SIGNAL( collapsed(QListViewItem*)),
+            this, SLOT( collapsed(QListViewItem*)) );
+
+    connect(this, SIGNAL( expanded(QListViewItem*)),
+            this, SLOT( expanded(QListViewItem*)) );
+
     connect( &singleClickTimer, SIGNAL( timeout() ),
              this, SLOT( resetSelection() ) );
 
+
+
     addColumn( tr( "Object" ) );
 
     header()->hide();
@@ -94,6 +108,7 @@ ObjectTreeView::ObjectTreeView(QWidget* parent, const char * name, WFlags f) :
     setItemMargin( 2 );
 
     setDragAutoScroll( TRUE );
+    setAllColumnsShowFocus( TRUE );
     setSelectionMode( QListView::Extended );
     setShowSortIndicator( FALSE );
     setRootIsDecorated( TRUE );
@@ -103,9 +118,28 @@ ObjectTreeView::ObjectTreeView(QWidget* parent, const char * name, WFlags f) :
     setSorting(0,true);
 }
 
-void ObjectTreeView::currentChanged(QListViewItem *ovi)
+void ObjectTreeView::currentChanged(QListViewItem *itm)
 {
-    lastSelected = ovi;
+    if (fwbdebug)
+        qDebug("ObjectTreeView::currentChanged  itm=%s",itm->text(0).ascii());
+    expandOrCollapse = false;
+
+//    lastSelected = ovi;
+//    lastSelected = currentItem();
+}
+
+void ObjectTreeView::collapsed(QListViewItem* itm)
+{
+    if (fwbdebug)
+        qDebug("ObjectTreeView::collapsed  itm=%s",itm->text(0).ascii());
+    expandOrCollapse = true;
+}
+
+void ObjectTreeView::expanded(QListViewItem* itm)
+{
+    if (fwbdebug)
+        qDebug("ObjectTreeView::expanded  itm=%s",itm->text(0).ascii());
+    expandOrCollapse = true;
 }
 
 /*
@@ -132,15 +166,102 @@ std::vector<libfwbuilder::FWObject*> ObjectTreeView::getSimplifiedSelection()
     return so;
 }
 
-
-QDragObject* ObjectTreeView::dragObject()
+FWObject* ObjectTreeView::getCurrentObject()
 {
     QListViewItem *ovi = currentItem();
-    item_before_drag_started = ovi;
     ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
     assert(otvi!=NULL);
+    return otvi->getFWObject();
+}
 
-    FWObject *obj = otvi->getFWObject();
+bool ObjectTreeView::isLockable()
+{
+    return Lockable;
+}
+
+bool ObjectTreeView::isUnlockable()
+{
+    return Unlockable;
+}
+
+void ObjectTreeView::focusInEvent(QFocusEvent* ev)
+{
+    QListView::focusInEvent(ev);
+    repaintItem(currentItem());
+}
+
+void ObjectTreeView::focusOutEvent(QFocusEvent* ev)
+{
+    QListView::focusOutEvent(ev);
+    repaintItem(currentItem());
+}
+
+void ObjectTreeView::updateTreeItems()
+{
+    QListViewItemIterator it(this);
+    QListViewItem *itm;
+    ObjectTreeViewItem *otvi;
+    FWObject *obj;
+    QPixmap pix(16,16);
+    string icn;
+    QImage img;
+    QImage img_lock=QImage::fromMimeSource( "lock.png" );
+    //img_lock=img_lock.scale(10,10);
+    
+    while ( it.current() ) 
+    {
+        itm= it.current();
+        otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+        obj=otvi->getFWObject();
+        
+        if (FWBTree::isSystem(obj))
+        {
+            icn="folder1.png";
+        }
+        else
+        {
+            icn=Resources::global_res->getObjResourceStr(obj, "icon-tree").c_str();
+        }
+        img=QImage::fromMimeSource( icn.c_str() );
+        
+        if (obj->getBool("ro"))
+        {
+            //QPainter painter(&img);
+            //painter.drawImage(6,6,img_lock);
+
+            img=img_lock;
+        }
+        else
+        {
+        }
+        
+        itm->setPixmap(0, img );
+        
+        Firewall *fw=Firewall::cast(obj);
+        if (fw!=NULL)
+        {
+            itm->setText(0,(fw->needsInstall())?
+                    QString::fromUtf8(fw->getName().c_str())+" *":
+                    QString::fromUtf8(fw->getName().c_str()));
+        }
+        
+        ++it;
+    }
+    triggerUpdate ();  
+}
+
+/*
+ * This is where drag starts
+ */
+QDragObject* ObjectTreeView::dragObject()
+{
+    if (fwbdebug)
+        qDebug("ObjectTreeView::dragObject");
+
+    QListViewItem *ovi = currentItem();
+    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
+
+    FWObject *obj = getCurrentObject();
 
 /* can't drag system folders 
 
@@ -191,8 +312,23 @@ QDragObject* ObjectTreeView::dragObject()
                          QPoint( pm.rect().width() / 2,
                                  pm.rect().height() / 2 ) );
 
+// lets try to always reset selection
+    setSelected(otvi,false);
     resetSelection();
-
+#if 0
+    /*
+     * need to reset selection if:
+     *
+     * object editor is opened, and
+     * we are dragging one object, and
+     * object opened in editor is not the same as the one we are dragging
+     */
+    if (oe->isVisible() && dragobj.size()==1 && oe->getOpened()!=obj)
+    {
+        setSelected(otvi,false);
+        resetSelection();
+    }
+#endif
     if (fwbdebug) qDebug("ObjectTreeView::dragObject()  returns !NULL");
 
     return drag;
@@ -276,11 +412,8 @@ void ObjectTreeView::dragMoveEvent( QDragMoveEvent *ev)
 void ObjectTreeView::dropEvent(QDropEvent *ev)
 {
     QListViewItem *ovi = itemAt( ev->pos() );
-    
     ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
     if (otvi==NULL) return;
-
-
     FWObject *trobj = otvi->getFWObject();
 
 /* the tree can accept drop only if it goes into a group and if that group
@@ -348,8 +481,17 @@ void ObjectTreeView::contentsMouseMoveEvent( QMouseEvent * e )
 
 void ObjectTreeView::contentsMousePressEvent( QMouseEvent *e )
 {
+    if (fwbdebug)
+        qDebug("ObjectTreeView::contentsMousePressEvent");
+
     second_click = false;
 
+    if (fwbdebug)
+        qDebug(QString("ObjectTreeView::contentsMousePressEvent :: currentItem=%1 lastSelected=%2")
+               .arg((currentItem())?currentItem()->text(0):"nil")
+               .arg((lastSelected)?lastSelected->text(0):"nil")
+        );
+
     lastSelected = currentItem();
 
     QListView::contentsMousePressEvent(e);
@@ -370,21 +512,59 @@ void ObjectTreeView::contentsMousePressEvent( QMouseEvent *e )
  */
 void ObjectTreeView::contentsMouseReleaseEvent( QMouseEvent *e )
 {
-//    if (!second_click) singleClickTimer.start( 200 , true );
+    if (fwbdebug)
+        qDebug("ObjectTreeView::contentsMouseReleaseEvent 1");
 
     QListView::contentsMouseReleaseEvent(e);
+
+    if (fwbdebug)
+        qDebug("ObjectTreeView::contentsMouseReleaseEvent 2 selectedObjects.size()=%d getCurrentObject()=%p current object %s",
+               selectedObjects.size(),
+               getCurrentObject(),
+               (getCurrentObject()!=NULL)?getCurrentObject()->getName().c_str():"nil");
+    
+    if (expandOrCollapse) return;  // user expanded or collapsed subtree,
+                                   // no need to change object in the editor
+
+    if (selectedObjects.size()==1)
+        emit switchObjectInEditor_sign( getCurrentObject() );
+    else
+    {
+        // user selected multiple objects
+        // do not let them if editor has unsaved changes
+        // 
+        if (oe->isVisible() && oe->isModified())
+            emit switchObjectInEditor_sign( getCurrentObject() );
+        else
+            oe->blank();
+    }
 }
 
+/*
+ * sends signal that should be connected to a slot in
+ * ObjectManipulator which opens editor panel if it is closed and then
+ * opens current object in it
+ */
+void ObjectTreeView::editCurrentObject()
+{
+    if (fwbdebug)
+        qDebug("ObjectTreeView::editCurrentObject");
+
+    emit editCurrentObject_sign();
+
+    if (fwbdebug)
+        qDebug("ObjectTreeView::editCurrentObject done");
+}
 
 void ObjectTreeView::contentsMouseDoubleClickEvent( QMouseEvent *e )
 {
+    if (fwbdebug)
+        qDebug("ObjectTreeView::contentsMouseDoubleClickEvent");
+
     second_click=true;
     singleClickTimer.stop();
 
-    QListViewItem *ovi = currentItem();
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
-    assert(otvi!=NULL);
-    FWObject *obj = otvi->getFWObject();
+    FWObject *obj = getCurrentObject();
 
 /* system folders open on doubleclick, while for regular objects it
  * opens an editor 
@@ -392,19 +572,16 @@ void ObjectTreeView::contentsMouseDoubleClickEvent( QMouseEvent *e )
     if (FWBTree::isSystem(obj))
         QListView::contentsMouseDoubleClickEvent(e);
     else
-        emit openObjectEditor_sign(obj);
+        editCurrentObject();
 }
 
 void ObjectTreeView::keyPressEvent( QKeyEvent* ev )
 {
-    QListViewItem *ovi = currentItem();
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
-    assert(otvi!=NULL);
-    FWObject *obj = otvi->getFWObject();
+    FWObject *obj = getCurrentObject();
 
     if (ev->key()==Qt::Key_Enter || ev->key()==Qt::Key_Return)
     {
-        emit openObjectEditor_sign(obj);
+        editCurrentObject();
         ev->accept();
         return;
     }
@@ -417,18 +594,43 @@ void ObjectTreeView::keyPressEvent( QKeyEvent* ev )
     QListView::keyPressEvent(ev);
 }
 
+void ObjectTreeView::keyReleaseEvent( QKeyEvent* ev )
+{
+    if (fwbdebug)
+        qDebug("ObjectTreeView::keyReleaseEvent");
+
+    QListView::keyReleaseEvent(ev);
+
+    if (selectedObjects.size()==1)
+        emit switchObjectInEditor_sign( getCurrentObject() );
+    else
+    {
+        // user selected multiple objects
+        // do not let them if editor has unsaved changes
+        // 
+        if (oe->isVisible() && oe->isModified())
+            emit switchObjectInEditor_sign( getCurrentObject() );
+        else
+            oe->blank();
+    }
+}
+
 void ObjectTreeView::returnPressed(QListViewItem *itm)
 {
-    QListViewItem *ovi = currentItem();
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
-    assert(otvi!=NULL);
-    FWObject *obj = otvi->getFWObject();
+    if (fwbdebug)
+        qDebug("ObjectTreeView::returnPressed");
 
-    emit openObjectEditor_sign(obj);
+    editCurrentObject();
+
+    QListView::returnPressed(itm);
 }
 
 void ObjectTreeView::resetSelection()
 {
+    if (fwbdebug)
+        qDebug(QString("ObjectTreeView::resetSelection :: lastSelected=%1").arg(lastSelected->text(0)));
+
+    setCurrentItem(lastSelected);
     setSelected( lastSelected , true);
 }
 
@@ -458,10 +660,35 @@ void ObjectTreeView::selectionChanged()
         }
         ++it;
     }
-
+    setLockFlags();
 /* now list  selectedObjects   holds all selected items */
 }
 
+void ObjectTreeView::setLockFlags()
+{
+    QListViewItemIterator it(this);
+    Lockable=false;
+    Unlockable=false;
+    while ( it.current() )
+    {
+        if (it.current()->isSelected())
+        {
+            QListViewItem *itm= it.current();
+            ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+            if (otvi->getFWObject()->getBool("ro")) 
+            {
+                Unlockable=true;
+            }
+            else
+            {
+                Lockable=true;
+            }
+        }
+        ++it;
+    }
+    
+}
+
 bool ObjectTreeView::isSelected(FWObject* obj)
 {
     for (vector<FWObject*>::iterator i=selectedObjects.begin();
