@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectTreeView.h,v 1.12 2005/04/17 21:58:55 vkurland Exp $
+  $Id: ObjectTreeView.h,v 1.18 2006/07/10 00:26:08 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -48,6 +48,9 @@ class ObjectTreeView : public QListView {
     bool             second_click;
 
     bool             selectionFrozen;
+    bool             expandOrCollapse;
+    bool             Lockable;
+    bool             Unlockable;
 
     std::vector<libfwbuilder::FWObject*>          selectedObjects;
         
@@ -64,7 +67,11 @@ class ObjectTreeView : public QListView {
     virtual void contentsMouseDoubleClickEvent( QMouseEvent *e );
     virtual void contentsMouseMoveEvent( QMouseEvent *e );
     virtual void keyPressEvent( QKeyEvent* ev );
+    virtual void keyReleaseEvent( QKeyEvent* ev );
 
+    virtual void focusInEvent(QFocusEvent*);
+    virtual void focusOutEvent(QFocusEvent*);
+    
  public:
 
     ObjectTreeView(QWidget* parent = 0, const char * name = 0, WFlags f = 0);
@@ -76,7 +83,14 @@ class ObjectTreeView : public QListView {
 
     bool isSelected(libfwbuilder::FWObject* obj);
     int  getNumSelected();
+    bool isLockable();
+    bool isUnlockable();
+    void setLockFlags();
 
+    libfwbuilder::FWObject* getCurrentObject();
+
+    void editCurrentObject();
+    
 /* Under some circumstances, user may select several host or fw
  * objects so that their children objects are selected as well
  * (e.g. when shift-click is used). "Delete objects" or "group
@@ -94,11 +108,15 @@ class ObjectTreeView : public QListView {
      void resetSelection();
      void returnPressed(QListViewItem *itm);
      void currentChanged(QListViewItem *itm);
+     void collapsed(QListViewItem *itm);
+     void expanded(QListViewItem *itm);
+     virtual void updateTreeItems();
 
  signals:
 
 //     void showObjectInfo_sign(libfwbuilder::FWObject *);
-     void openObjectEditor_sign(libfwbuilder::FWObject *);
+     void editCurrentObject_sign();
+     void switchObjectInEditor_sign(libfwbuilder::FWObject*);
      void objectDropped_sign(libfwbuilder::FWObject *);
      void deleteObject_sign(libfwbuilder::FWObject *);
 };
