@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland vadim@fwbuilder.org
 
-  $Id: RCS.cpp,v 1.57 2005/05/30 03:34:40 vkurland Exp $
+  $Id: RCS.cpp,v 1.59 2006/07/19 06:21:08 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -31,7 +31,7 @@
 #include "fwbuilder/libfwbuilder-config.h"
 #include "fwbuilder/Tools.h"
 
-#include "FWWindow.h"
+//#include "FWWindow.h"
 
 #include <RCS.h>
 #include <qdir.h>
@@ -279,11 +279,82 @@ RCS::RCS(const QString &file)
  * rlog is started with environment defined by RCSEnvFix, which does
  * not have env. var LANG so it always runs in english
  */
-        QStringList rcslog=rlog();
+        QStringList split_log = QStringList::split(QRegExp("------|======"),rlog());
+                                                   
+        QString head_section = split_log[0];
+
+        QRegExp head_rx("head:\\s+([0-9\\.]+)\\s*\\n");
+        int pos = head_rx.search( head_section );
+        if (pos>-1) head = head_rx.cap(1);
 
         QStringList::iterator i;
+        for (i=split_log.begin(),++i; i!=split_log.end(); ++i)
+        {
+            QString section = *i;
+            if (section.length()==0) continue;
 
-        for (i=rcslog.begin(); i!=rcslog.end(); ++i)
+            int match = -1;
+
+            Revision r(filename);
+            r.rev = "";
+            r.log = "";
+
+            QRegExp rev_rx("revision\\s+([0-9\\.]+)");
+            match = rev_rx.search( section );
+            if (match>-1)
+            {
+                r.rev = rev_rx.cap(1);
+            }
+
+            QRegExp lock_rx("revision\\s+([0-9\\.]+)\\s+locked by:\\s+(\\S+);");
+            lock_rx.setMinimal(true);
+            match = lock_rx.search( section );
+            if (match>-1)
+            {
+                r.locked_by = lock_rx.cap(2);
+                locked      = true;
+                locked_by   = lock_rx.cap(2);
+                locked_rev  = r.rev;
+            }
+
+// older implementation copied revision and "locked by" to r.log
+// we'll do the same here to maintain compatibility
+            QRegExp rev2_rx("(revision.+)\\n");
+            rev2_rx.setMinimal(true);
+            match = rev2_rx.search( section );
+            if (match>-1)
+            {
+                r.log += rev2_rx.cap(1) + "\n";
+            }
+
+
+            QRegExp date_rx("date:\\s+([^;]+);\\s+author:\\s+(\\S+);");
+            date_rx.setMinimal(true);
+            match = date_rx.search( section );
+            if (match>-1)
+            {
+                r.date   = date_rx.cap(1);
+                r.author = date_rx.cap(2);
+            }
+
+            QRegExp log_rx("date:.*\\n(.*)$");
+            log_rx.setMinimal(true);
+            match = log_rx.search( section );
+            if (match>-1)
+                r.log += log_rx.cap(1);
+
+            r.log.replace('\r',"");
+
+            if (r.rev != "")
+            {
+                revisions.push_back(r);
+                if (fwbdebug) qDebug("revision %s: '%s'",r.rev.ascii(),r.log.ascii());
+            }
+
+        }
+
+#if 0
+        for ()
         {
             if ( (*i).find("head: ")==0)
             {
@@ -311,7 +382,7 @@ RCS::RCS(const QString &file)
 
                 r.log="";
 
-                for ( ; (*i).find(QRegExp("^=========="))==-1 &&
+                for ( ;   (*i).find(QRegExp("^=========="))==-1 &&
                           (*i).find(QRegExp("^----------"))==-1 &&
                           i!=rcslog.end(); ++i )
                 {
@@ -332,6 +403,7 @@ RCS::RCS(const QString &file)
             }
         }
 //        qBubbleSort( revisions.begin() , revisions.end() );
+#endif
 
         inrcs         = true;
         tracking_file = true;
@@ -378,6 +450,8 @@ void RCS::readFromStderr()
 
 void RCS::abandon()
 {
+    if (!isInRCS()) return;
+
 /* check out head revision and unlock it */
     proc->clearArguments();
 
@@ -412,7 +486,7 @@ void RCS::abandon()
     checked_out=false;
     
     QString err = tr("Error checking file out: %1").arg(stderrBuffer);
-    QMessageBox::critical(mw, "Firewall Builder", err, tr("&Continue") );
+    QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
 
     throw(FWException(err.latin1()));
 }
@@ -601,7 +675,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
 		if (fd<0)
 		{
                     QString err = tr("Error creating temporary file ")+tname+QString(" :\n")+strerror(errno);
-                    QMessageBox::critical(mw, "Firewall Builder", err, tr("&Continue") );
+                    QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
                     throw(FWException(err.latin1()));
 		}
 #ifdef _WIN32
@@ -614,7 +688,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
 		    close(fd);
 #endif
                     QString err = tr("Error writing to temporary file ")+tname+QString(" :\n")+strerror(errno);
-                    QMessageBox::critical(mw, "Firewall Builder", err, tr("&Continue") );
+                    QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
                     throw(FWException(err.latin1()));
                 }
                 close(fd);
@@ -631,7 +705,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
         selectedRev = head;
 
         QString err = tr("Error checking file out: %1").arg(stderrBuffer);
-        QMessageBox::critical(mw, "Firewall Builder", err, tr("&Continue") );
+        QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
         throw(FWException(err.latin1()));
 
     } else
@@ -644,7 +718,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
             if (me!=locked_by)
             {
                 switch (QMessageBox::warning(
-                            mw,"Firewall Builder", 
+                            app->activeWindow(),"Firewall Builder", 
                             tr("File is opened and locked by %1.\nYou can only open it read-only.")
                             .arg(locked_by),
                             "Open &read-only", "&Cancel", QString::null,
@@ -657,7 +731,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
 
             if (force) goto checkout;
 
-            switch ( QMessageBox::warning(mw, "Firewall Builder",
+            switch ( QMessageBox::warning(app->activeWindow(), "Firewall Builder",
                                           tr("Revision %1 of this file has been checked out and locked by you earlier.\n\
 The file may be opened in another copy of Firewall Builder or was left opened\n\
 after the program crashed.").arg(locked_rev),
@@ -714,7 +788,7 @@ after the program crashed.").arg(locked_rev),
         selectedRev = head;
 
         QString err = tr("Error checking file out: %1").arg(stderrBuffer);
-        QMessageBox::critical(mw, "Firewall Builder", err, tr("&Continue") );
+        QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
 
         throw(FWException(err.latin1()));
     }
@@ -850,7 +924,7 @@ void RCS::completeCI()
  * "-z+09:00" works properly
  *
  */
-QStringList RCS::rlog() throw(libfwbuilder::FWException)
+QString RCS::rlog() throw(libfwbuilder::FWException)
 {
     proc->clearArguments();
 
@@ -902,7 +976,8 @@ QStringList RCS::rlog() throw(libfwbuilder::FWException)
     QString rlogTxt = QString::fromUtf8(stdoutBuffer.ascii());
 
     if (proc->normalExit() && proc->exitStatus()==0)
-        return QStringList::split("\n",rlogTxt );
+        return rlogTxt;
+
     QString msg=QObject::tr("Fatal error running rlog for %1").arg(filename);
     throw( FWException( msg.latin1() ) );
 }
