@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleOptionsDialog.cpp,v 1.7.4.3 2005/10/25 06:11:26 vkurland Exp $
+  $Id: RuleOptionsDialog.cpp,v 1.20 2006/05/24 04:36:54 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -48,13 +48,17 @@
 #include <qcheckbox.h>
 #include <qmessagebox.h>
 #include <qregexp.h>
+#include <qlabel.h>
 
 #include <iostream>
 
 using namespace libfwbuilder;
 using namespace std;
 
-RuleOptionsDialog::RuleOptionsDialog(QWidget *parent) : RuleOptionsDialog_q(parent) {}
+RuleOptionsDialog::RuleOptionsDialog(QWidget *parent) : RuleOptionsDialog_q(parent)
+{
+    init=false;
+}
 
 void RuleOptionsDialog::loadFWObject(FWObject *o)
 {
@@ -65,8 +69,14 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
     while ( !Firewall::isA(p) ) p=p->getParent();
     platform=p->getStr("platform").c_str();
 
+    
     Rule      *rule = dynamic_cast<Rule*>(o);
     FWOptions *ropt = rule->getOptionsObject();
+    
+    editorTitle->setText(QString("%1 / %2 / %3 ")
+            .arg(QString::fromUtf8(p->getName().c_str()))
+            .arg(rule->getTypeName().c_str())
+            .arg(rule->getPosition()));
 
     int wid=0;
     if (platform=="iptables") wid=0;
@@ -89,17 +99,6 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
     QStringList  logFacilities=getLogFacilities( obj->getStr("platform").c_str() );
     ipf_logFacility->clear();
     ipf_logFacility->insertStringList(getScreenNames(logFacilities));    
-
-    QStringList actionsOnReject=getActionsOnReject( obj->getStr("platform").c_str() );
-    ipt_actionOnReject->clear();
-    ipt_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
-    ipf_actionOnReject->clear();
-    ipf_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
-    pf_actionOnReject->clear();
-    pf_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
-    ipfw_actionOnReject->clear();
-    ipfw_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
-
     QStringList limitSuffixes=getLimitSuffixes( obj->getStr("platform").c_str() );
     ipt_limitSuffix->clear();
     ipt_limitSuffix->insertStringList(getScreenNames(limitSuffixes));
@@ -112,18 +111,21 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
         data.registerOption( ipt_logPrefix            , ropt,  "log_prefix" );
         data.registerOption( ipt_logLevel             , ropt,  "log_level", logLevels );
         data.registerOption( ipt_nlgroup              , ropt,  "ulog_nlgroup" );
-        data.registerOption( ipt_rule_name_accounting , ropt,  "rule_name_accounting" );
-        data.registerOption( ipt_actionOnReject       , ropt,  "action_on_reject" ,actionsOnReject);
         data.registerOption( ipt_limit                , ropt,  "limit_value" );
         data.registerOption( ipt_limitSuffix          , ropt,  "limit_suffix", limitSuffixes);
         data.registerOption( ipt_burst                , ropt,  "limit_burst" );
+
+        data.registerOption( ipt_connlimit            , ropt,  "connlimit_value" );
+        data.registerOption( ipt_connlimit_masklen    , ropt,  "connlimit_masklen" );
+
+        data.registerOption( ipt_hashlimit            , ropt,  "hashlimit_value" );
+        data.registerOption( ipt_hashlimit_suffix     , ropt,  "hashlimit_suffix" );
+        data.registerOption( ipt_hashlimit_burst      , ropt,  "hashlimit_burst" );
+        data.registerOption( ipt_hashlimit_mode       , ropt,  "hashlimit_mode" );
+        data.registerOption( ipt_hashlimit_dstlimit   , ropt,  "hashlimit_dstlimit");
+
         data.registerOption( ipt_assumeFwIsPartOfAny  , ropt,  "firewall_is_part_of_any_and_networks" );
         data.registerOption( ipt_stateless            , ropt,  "stateless" );
-
-        bool f=false;
-        if (PolicyRule::isA(rule))
-            f = (PolicyRule::cast(rule)->getAction()==PolicyRule::Accounting);
-        ipt_rule_name_accounting->setEnabled(f);
     }
 
 
@@ -131,7 +133,6 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
     {
         data.registerOption( ipf_logFacility     , ropt,  "ipf_log_facility", logFacilities);
         data.registerOption( ipf_logLevel        , ropt,  "log_level" , logLevels);
-        data.registerOption( ipf_actionOnReject  , ropt,  "action_on_reject", actionsOnReject);
         data.registerOption( ipf_masq_icmp       , ropt,  "ipf_return_icmp_as_dest");
         data.registerOption( ipf_stateless       , ropt,  "stateless" );
         data.registerOption( ipf_keep_frags      , ropt,  "ipf_keep_frags" );
@@ -140,7 +141,6 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
     if (platform=="pf")
     {
         data.registerOption( pf_logPrefix        , ropt,  "log_prefix" );
-        data.registerOption( pf_actionOnReject   , ropt,  "action_on_reject",actionsOnReject);
         data.registerOption( pf_stateless        , ropt,  "stateless" );
         data.registerOption( pf_rule_max_state   , ropt,  "pf_rule_max_state" );
         data.registerOption( pf_source_tracking  , ropt,  "pf_source_tracking" );
@@ -150,7 +150,6 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
 
     if (platform=="ipfw")
     {
-        data.registerOption( ipfw_actionOnReject , ropt,"action_on_reject",actionsOnReject);
         data.registerOption( ipfw_stateless      , ropt,"stateless" );
     }
 
@@ -182,16 +181,18 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
     pf_max_src_nodes->setEnabled( pf_source_tracking->isChecked() );
     pf_max_src_states->setEnabled( pf_source_tracking->isChecked() );
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
     init=false;
 }
     
 void RuleOptionsDialog::changed()
 {
-    apply->setEnabled( true );
+    //apply->setEnabled( true );
 
     pf_max_src_nodes->setEnabled( pf_source_tracking->isChecked() );
     pf_max_src_states->setEnabled( pf_source_tracking->isChecked() );
+
+    emit changed_sign();
 }
 
 void RuleOptionsDialog::validate(bool *res)
@@ -201,7 +202,7 @@ void RuleOptionsDialog::validate(bool *res)
 
 void RuleOptionsDialog::isChanged(bool *res)
 {
-    *res=(!init && apply->isEnabled());
+    //*res=(!init && apply->isEnabled());
 }
 
 void RuleOptionsDialog::libChanged()
@@ -213,31 +214,20 @@ void RuleOptionsDialog::applyChanges()
 {
     if (!isTreeReadWrite(this,obj)) return;
 
-    if (platform=="iptables")
-    {
-        QString rn = ipt_rule_name_accounting->text();
-/* rule name for accounting may contain only alphanumeric characters
- * and no white spaces or spec. characters
- */
-        if (rn.contains(QRegExp("[^a-zA-Z0-9_]"))!=0)
-        {
-            QMessageBox::information(
-                this,"Firewall Builder", 
-                tr("Rule name for accounting is converted to the iptables\nchain name and therefore may not contain white space\nand special characters."),
-                tr("&Continue"), QString::null,QString::null,
-                0, 1 );
-
-            return;
-        }
-    }
-
     init=true;
     data.saveAll();
     init=false;
 
     mw->updateRuleOptions();
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
+    om->updateLastModifiedTimestampForAllFirewalls(obj);
+}
+
+void RuleOptionsDialog::cancelChanges()
+{
+    //apply->setEnabled( false );
+    close();
 }
 
 void RuleOptionsDialog::discardChanges()
