@@ -1,4 +1,6 @@
 
+#include "fwbuilder/Rule.h"
+
 #include <qtable.h>
 #include <qpixmap.h>
 #include <qevent.h>
@@ -11,16 +13,17 @@ class QPopupMenu;
 
 namespace libfwbuilder {
     class FWObject;
+    class Firewall;
     class FWReference;
     class RuleElement;
     class RuleSet;
     class Policy;
     class InterfacePolicy;
     class NAT;
+    class Routing;
     class RuleElement;
-    class Rule;
-    class PolicyRule;
     class NATRule;
+    class RoutingRule;
 };
 
 class RuleSetView;
@@ -63,27 +66,35 @@ class RuleSetView : public QTable
     void horizontalHeaderClicked(int col);
     void verticalHeaderClicked(int row);
     void selectionChanged();
+    void restoreSelection(bool same_widget);
 
     void contextMenu(int row, int col, const QPoint &pos);
     void doubleClicked(int row,int col,int btn,const QPoint &pos);
-        
-    void editSelectedObject();
+
+    void editSelected();
     void copySelectedObject();
     void cutSelectedObject();
     void deleteSelectedObject();
     void pasteObject();
+    void revealObjectInTree();
 
     void negateRE();
     void editRE();
-    void editComment();
+
     void changeActionToAccept();
     void changeActionToDeny();
     void changeActionToReject();
     void changeActionToAccounting();
+    void changeActionToPipe();
+    void changeActionToTag();
+    void changeActionToClassify();
+    void changeActionToCustom();
+    void changeActionToBranch();
+    void changeActionToRoute();
+
     void changeDirectionToIn();
     void changeDirectionToOut();
     void changeDirectionToBoth();
-    void editRuleOptions();
     void changeLogToOn();
     void changeLogToOff();
 
@@ -99,13 +110,14 @@ class RuleSetView : public QTable
     void pasteRuleAbove();
     void pasteRuleBelow();
 
+
     void enableRule();
     void disableRule();
 
     void setRuleColor(const QString &c);
     void setRuleNumber(int row, libfwbuilder::Rule *rule);
 
- protected:
+ public:
 
     enum REType          { RuleOp,
                            Object,
@@ -113,7 +125,10 @@ class RuleSetView : public QTable
                            Direction,
                            Options,
                            Time,
-                           Comment };
+                           Comment,
+                           Metric };
+
+ protected:
 
     enum PixmapAttr      { Normal,  Neg,     Ref,       Tree };
     enum PopupMenuAction { None,    EditObj, EditRE,    NegateRE };
@@ -148,10 +163,18 @@ class RuleSetView : public QTable
     std::map<int,REType>                  colTypes;
 
     libfwbuilder::FWObject               *selectedObject;
+    int                                   selectedObjectRow;
+    int                                   selectedObjectCol;
+    
+    libfwbuilder::FWObject               *prevSelectedObject;
+    int                                   prevSelectedObjectRow;
+    int                                   prevSelectedObjectCol;
+
     bool                                  kbdGoingUp;
     bool                                  changingSelection;
     bool                                  dragging;
-    
+    bool                                  changingRules;
+ 
     PopupMenuAction                       lastPopupMenuAction;
 
     void    iinit();
@@ -164,6 +187,7 @@ class RuleSetView : public QTable
     virtual QDragObject* dragObject();
     virtual void contentsMousePressEvent( QMouseEvent* ev );
     virtual void contentsMouseMoveEvent( QMouseEvent* ev );
+    virtual void contentsMouseReleaseEvent( QMouseEvent* ev );
     virtual void keyPressEvent( QKeyEvent* ev );
     virtual void dragMoveEvent( QDragMoveEvent *ev);
     virtual void dragEnterEvent( QDragEnterEvent *ev);
@@ -171,8 +195,18 @@ class RuleSetView : public QTable
     virtual void dropEvent( QDropEvent *ev);
     virtual void hideEvent(QHideEvent *ev);
     
+    virtual void focusInEvent(QFocusEvent*);
+    virtual void focusOutEvent(QFocusEvent*);
+    
     virtual void adjustRow( int row );
     virtual void adjustColumn( int col );
+
+    bool isDirection (int col);
+    bool isAction    (int col);
+    bool isOptions   (int col);
+    
+    QString getPlatform();
+    
     QRect calculateCellSize( int row, int col );
     
     virtual void resizeData( int ) {}
@@ -199,16 +233,19 @@ class RuleSetView : public QTable
 
 //    RuleSetOptions* getOpt() const { return opt; }
 
-    REType  getColType(int col) const;
-
     bool    insertObject(int row, int col, libfwbuilder::FWObject *obj);
     void    deleteObject(int row, int col, libfwbuilder::FWObject *obj);
 
     void    copyRuleContent(libfwbuilder::Rule *dst, libfwbuilder::Rule *src);
 
     void    headerMousePressEvent(const QPoint &pos);
+    void    changeDitection(libfwbuilder::PolicyRule::Direction dir);
+    void    changeAction(libfwbuilder::PolicyRule::Action act);
+
+    void    setSelectedObject(libfwbuilder::FWObject* obj);
 
  public:
+
     RuleSetView( int r, int c, QWidget *parent);
     virtual ~RuleSetView();
     virtual void init();
@@ -217,25 +254,38 @@ class RuleSetView : public QTable
     
     void unselect();
     bool isSelected();
+    libfwbuilder::FWObject* getSelectedObject();
+    libfwbuilder::Firewall* getFirewall();
 
     libfwbuilder::Rule* insertRule(int pos, libfwbuilder::FWObject *r);
 
     libfwbuilder::FWObject* getObj(int row, int col,
                                    int mouse_y_pos, QRect *cr=NULL);
+    libfwbuilder::PolicyRule* getRule(int row);
 
     void selectRE( int row, int col);
 
-/**
- * selects rule element a reference 'ref' belongs to
- */
+    /**
+     * selects rule element a reference 'ref' belongs to
+     */
     void selectRE(libfwbuilder::FWReference *ref);
 
     void updateCurrentCell();
     void updateAll();
 
-    void info();
-    
- private:
+    void editCurrentObject();
+    bool switchObjectInEditor(int col,bool validate=true);
+
+    /**
+     * shows and selects object obj in the tree, or, if obj is a rule,
+     * shows corresponding firewall object. Makes sure keyboard focus
+     * returns back to the ruleset.
+     */
+    void openObjectInTree(libfwbuilder::FWObject *obj);
+
+    REType  getColType(int col) const;
+
+private:
 
     QIntDict<QTableItem> items;
     QIntDict<QWidget> widgets;
@@ -247,6 +297,7 @@ class RuleSetView : public QTable
     
     QPixmap getPixmap(libfwbuilder::FWObject *obj, PixmapAttr pmattr = Normal) const;
     QString objectText(libfwbuilder::RuleElement *re,libfwbuilder::FWObject *obj);
+
 };
 
 
@@ -295,3 +346,18 @@ class NATView : public RuleSetView
 
 };
 
+class RoutingView : public RuleSetView
+{
+    virtual libfwbuilder::RuleElement* getRE( int row, int col ) ;
+    virtual libfwbuilder::RuleElement* getRE( libfwbuilder::Rule* r, int col ) ;
+
+ public:
+
+    RoutingView(libfwbuilder::Routing *p, QWidget *parent);
+    virtual ~RoutingView() {}
+
+    virtual void init();
+    
+
+};
+
