@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHSession.cpp,v 1.14.4.9 2006/04/13 07:08:48 vkurland Exp $
+  $Id: SSHSession.cpp,v 1.20 2006/05/15 03:13:48 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -75,6 +75,7 @@ SSHSession::SSHSession(const QString &_h,
     endOfCopy = false;
 
     proc = NULL;
+    retcode = 0;
     heartBeatTimer = new QTimer(this);
     connect(heartBeatTimer, SIGNAL(timeout()), this, SLOT(heartBeat()) );
 
@@ -101,6 +102,7 @@ SSHSession::SSHSession(const QString &_h,
 void SSHSession::startSession()
 {
     proc = new QProcess();
+    retcode = -1;
     proc->setCommunication(
         QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
 
@@ -212,18 +214,25 @@ void SSHSession::terminate()
         if (fwbdebug)
             qDebug("SSHSession::terminate   terminating child process pid=%ld",
                    proc->processIdentifier());
-
-        QString s=QString(proc->readStdout());
-        if (!quiet)
+#ifdef _WIN32
+        if (proc->processIdentifier() != NULL)
+#else
+        if (proc->processIdentifier() != -1)
+#endif
         {
-            s.replace('\r',"");    
-            emit printStdout_sign(s);
-        }
-
-        proc->kill();
-        delete proc;
-        proc=NULL;
-//        QTimer::singleShot( 5000, proc, SLOT(kill()) );
+            // process is stll alive, killing
+            QString s=QString(proc->readStdout());
+            if (!quiet)
+            {
+                s.replace('\r',"");    
+                emit printStdout_sign(s);
+            }
+            proc->kill();
+            delete proc;
+            proc=NULL;
+            retcode=-1;
+            processExited();
+        };
     }
 }
 
@@ -357,7 +366,8 @@ void SSHSession::readFromStderr()
 
 void SSHSession::processExited()
 {
-    int retcode=proc->exitStatus();
+    if (proc) retcode=proc->exitStatus();
+
     QString exitStatus = (retcode)?QObject::tr("ERROR"):QObject::tr("OK");
     
     emit printStdout_sign(tr("SSH session terminated, exit status: %1").arg(retcode));
