@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHSession.h,v 1.9.4.4 2006/04/12 07:34:47 vkurland Exp $
+  $Id: SSHSession.h,v 1.14 2006/05/08 01:26:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -50,11 +50,11 @@ class SSHSession : public QObject {
  protected:
 
     QProcess   *proc;
+    int         retcode;
     QTimer     *heartBeatTimer;
     QString     stdoutBuffer;
     QString     stderrBuffer;
     QString     ssh;
-    QString     scp;
     QString     cmd;
     QStringList args;
 
@@ -115,7 +115,6 @@ class SSHSession : public QObject {
     QString     enable_prompt;
     QString     pwd_prompt;
     QString     putty_pwd_prompt;
-    QString     scp_pwd_prompt;
     QString     ssh_pwd_prompt;
     QString     ssoft_prompt1;
     QString     ssoft_prompt2;
