@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHUnx.cpp,v 1.12.4.4 2006/04/12 07:34:47 vkurland Exp $
+  $Id: SSHUnx.cpp,v 1.15 2006/05/08 01:26:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -52,7 +52,6 @@ SSHUnx::SSHUnx(const QString &_h,
     enable_prompt="# ";
     pwd_prompt="'s password: ";
     epwd_prompt="Password: ";
-    scp_pwd_prompt="Password: ";
     ssh_pwd_prompt="'s password: ";
     ssoft_config_prompt="> ";
     sudo_pwd_prompt="Password:";
@@ -102,8 +101,7 @@ void SSHUnx::stateMachine()
     {
     case NONE:
     {
-        if ( cmpPrompt(stdoutBuffer,scp_pwd_prompt) ||
-             cmpPrompt(stdoutBuffer,ssh_pwd_prompt) ||
+        if ( cmpPrompt(stdoutBuffer,ssh_pwd_prompt) ||
              cmpPrompt(stdoutBuffer,putty_pwd_prompt) ||
              stdoutBuffer.findRev(passphrase_prompt,-1)!=-1 ||
 
