@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ipfwAdvancedDialog.cpp,v 1.9 2005/04/10 07:16:39 vkurland Exp $
+  $Id: ipfwAdvancedDialog.cpp,v 1.11 2006/03/16 05:38:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,6 +29,7 @@
 
 #include "ipfwAdvancedDialog.h"
 #include "SimpleTextEditor.h"
+#include "ObjectManipulator.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
@@ -123,6 +124,7 @@ void ipfwAdvancedDialog::accept()
     pis->setCommand( installScript->text().latin1() );
     pis->setArguments( installScriptArgs->text().latin1() );
 
+    om->updateLastModifiedTimestampForAllFirewalls(obj);
     ipfwAdvancedDialog_q::accept();
 }
 
