@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: main.cpp,v 1.89.4.5 2006/04/13 07:08:48 vkurland Exp $
+  $Id: main.cpp,v 1.111 2006/07/20 23:11:10 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -54,9 +54,9 @@
 #include "ObjectEditor.h"
 #include "FWObjectClipboard.h"
 #include "FWBTree.h"
-#include "StartWizard.h"
 #include "findDialog.h"
 #include "listOfLibraries.h"
+#include "platforms.h"
 
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/Tools.h"
@@ -127,9 +127,11 @@ findDialog        *fd         = NULL;
 int                fwbdebug   = 0;
 int                safemode   = 0;
 bool               registered = false;
-    
+bool               gui_experiment1 = false;
+
 listOfLibraries  *addOnLibs;
 
+
 #ifndef _WIN32
 
 #ifndef HAVE_CFMAKERAW
@@ -296,13 +298,13 @@ ssize_t writen(int fd,const void *vptr, size_t n)
 
     ptr = (const char*)(vptr);
     nleft = n;
-    if (fwbdebug) qDebug("need to write %d bytes",nleft);
+    if (fwbdebug) qDebug("need to write %d bytes",int(nleft));
     while (nleft > 0)
     {
         if ( (nwritten = write(fd,ptr,nleft )) <= 0)
             return nwritten;
 
-        if (fwbdebug) qDebug("%d bytes written",nwritten);
+        if (fwbdebug) qDebug("%d bytes written",int(nwritten));
 
         nleft -= nwritten;
         ptr += nwritten;
@@ -331,7 +333,6 @@ void usage()
 int main( int argc, char ** argv )
 {
     bool        ssh_wrapper=false;
-    bool        scp_wrapper=false;
     const char *arg[64];
     int         i, j;
 
@@ -342,10 +343,6 @@ int main( int argc, char ** argv )
 
     bool desktopaware = true;
 
-#if defined(Q_OS_MACX)
-    desktopaware = false;
-#endif
-
 /*
  * I am using njamd a lot, but gtkmm and probably some other libs
  * generate trap in their global static initialization code. Therefore
@@ -370,16 +367,14 @@ int main( int argc, char ** argv )
     {
         if (strncmp(argv[i], "-X", 2)==0) { ssh_wrapper=true; continue; }
         else
-            if (strncmp(argv[i], "-Y", 2)==0) { scp_wrapper=true; continue; }
+            if (strncmp(argv[i], "-d", 2)==0) { fwbdebug++; continue; }
             else
-                if (strncmp(argv[i], "-d", 2)==0) { fwbdebug++; continue; }
-                else
-                    arg[j]=strdup(argv[i]);
+                arg[j]=strdup(argv[i]);
         j++;
     }
     arg[j]=NULL;
 
-    if (ssh_wrapper || scp_wrapper)
+    if (ssh_wrapper)
     {
 
 /* need to create and initialize settings to be able to use ssh/scp
@@ -405,13 +400,10 @@ int main( int argc, char ** argv )
         st->init();
 
         QString sshcmd=st->getSSHPath();
-        QString scpcmd=st->getSCPPath();
 
         if (sshcmd.isEmpty()) sshcmd="ssh";
-        if (scpcmd.isEmpty()) scpcmd="scp";
         
-        if (ssh_wrapper)  arg[0]=strdup( sshcmd.latin1() );
-        else              arg[0]=strdup( scpcmd.latin1() );
+        arg[0]=strdup( sshcmd.latin1() );
 
         if (fwbdebug)
             qDebug("cmd: %s",arg[0]);
@@ -542,7 +534,7 @@ int main( int argc, char ** argv )
 
 
     int c;
-    while ((c = getopt (argc , argv , "hvf:o:p:dxs")) != EOF )
+    while ((c = getopt (argc , argv , "hvf:o:p:dxsg")) != EOF )
 	switch (c) {
 	case 'h':
 	    usage();
@@ -571,6 +563,10 @@ int main( int argc, char ** argv )
         case 's':
             safemode++;
             break;
+            
+        case 'g':
+            gui_experiment1 = true;
+            break;
 	}
 
     if ( (argc-1)==optind)
@@ -584,6 +580,7 @@ int main( int argc, char ** argv )
     
 /* need to initialize in order to be able to use FWBSettings */
         init(argv);
+        init_platforms();
 
         if (fwbdebug) qDebug("creating app ...");
     
@@ -600,7 +597,7 @@ int main( int argc, char ** argv )
         if (fwbdebug) qDebug("done");
     
 #ifdef ELC
-        registered=init2(argv0, "Firewall Builder","fwb_gui","FirewallBuilder",true,true);
+        registered=init2(argv0, "Firewall Builder","fwb_gui","FirewallBuilder/2.1",true,true);
 #endif
 
         if (fwbdebug) qDebug("reading resources ...");
@@ -666,12 +663,15 @@ int main( int argc, char ** argv )
         addOnLibs = new listOfLibraries();
 
         mw  = new FWWindow();
-        oe  = new ObjectEditor(mw);
-        oe->open(FWObjectDatabase::db);
+        oe  = new ObjectEditor((QWidget*)mw->objectEditorStack);
+//        oe->open(mw->db());
+        oe->setCloseButton(mw->closeObjectEditorButton);
+        oe->setApplyButton(mw->applyObjectEditorButton);
         oe->hide();
         fd  = new findDialog(mw);
         fd->hide();
 
+        
         if (fwbdebug) qDebug("loading data file ...");
 
         if (safemode)  mw->load(NULL);
@@ -718,12 +718,9 @@ int main( int argc, char ** argv )
                         mw->load(NULL);
                     break;
                 }
-                default:  // ask user
+                default:  //  load standard objects too
                 {
-/* no file specified on command line and autoload option is off */
-                    StartWizard sw;
-                    if ( sw.exec() != QDialog::Accepted )
-                        mw->load(NULL); // load standard objects
+                    mw->load(NULL); // load standard objects
                 }
                 }
             }
