@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newFirewallDialog.cpp,v 1.23 2005/04/30 18:22:27 vkurland Exp $
+  $Id: newFirewallDialog.cpp,v 1.27 2006/06/16 04:33:13 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -69,9 +69,10 @@ using namespace std;
 
 newFirewallDialog::newFirewallDialog() : newFirewallDialog_q()
 {
-    nfw=NULL; 
-    snmpPollCompleted=false;
-    q=NULL;
+    nfw = NULL; 
+    tmpldb = NULL;
+    snmpPollCompleted = false;
+    q = NULL;
     unloadTemplatesLib = false;
     getInterfacesBusy = false;
 
@@ -136,7 +137,9 @@ void newFirewallDialog::changed()
 
     if (p==2)
     {
-        if (iface_dyn->isChecked() || iface_unnum->isChecked())
+        if (iface_dyn->isChecked() ||
+            iface_unnum->isChecked() ||
+            iface_bridgeport->isChecked())
         {
             iface_addr->clear();
             iface_addr->setEnabled(false);
@@ -180,6 +183,7 @@ void  newFirewallDialog::monitor()
             QString dn;
             if (idata.isDyn)        dn+="dyn";
             if (idata.isUnnumbered) dn+="unn";
+            if (idata.isBridgePort) dn+="bridge";
 
             new QListViewItem(iface_list, 
                               idata.name.c_str(),
@@ -338,6 +342,18 @@ void newFirewallDialog::selected(const QString &title)
     {
         setFinishEnabled( QWizard::page(4), true );
 /* load templates if not loaded */
+        if (tmpldb==NULL)
+        {
+
+            MessageBoxUpgradePredicate upgrade_predicate(this);
+
+            tmpldb = new FWObjectDatabase();
+            tmpldb->setReadOnly( false );
+            tmpldb->load( tempfname, &upgrade_predicate, librespath);
+        }
+        FWObject *tlib = tmpldb->getById(TEMPLATE_LIB);
+
+#if 0
         FWObject *tlib = mw->db()->getById(TEMPLATE_LIB);
         if (tlib==NULL)
         {
@@ -349,6 +365,8 @@ void newFirewallDialog::selected(const QString &title)
 /* restore library that was opened prior loading templates */
             om->openLib(cl);
         }
+#endif
+
         list<FWObject*> fl;
         findFirewalls(tlib, fl);
 
@@ -386,8 +404,9 @@ void newFirewallDialog::fillInterfaceSLList()
         idata.label        =  itm->text(1).latin1();
         idata.isDyn        =  itm->text(4).find("Dyn")!=-1;
         idata.isUnnumbered =  itm->text(4).find("Unn")!=-1;
+        idata.isBridgePort =  itm->text(4).find("Bridge")!=-1;
 
-        if (!idata.isDyn && !idata.isUnnumbered)
+        if (!idata.isDyn && !idata.isUnnumbered && !idata.isBridgePort)
             idata.address  =  itm->text(2).latin1();
         else
             idata.address  =  QObject::tr("dynamic").latin1();
@@ -483,9 +502,11 @@ void newFirewallDialog::fillInterfaceData(Interface *intf, QTextBrowser *qte)
     else
         if (intf->isUnnumbered()) s +=  tr("Unnumbered interface");
         else
-            s += QString("%1/%2")
-                .arg(intf->getAddress().toString().c_str())
-                .arg( intf->getNetmask().toString().c_str());
+            if (intf->isBridgePort()) s +=  tr("Bridge port");
+            else
+                s += QString("%1/%2")
+                    .arg(intf->getAddress().toString().c_str())
+                    .arg( intf->getNetmask().toString().c_str());
     s += "</td>";
     s += "</tr>";
     s += "</table>";
@@ -495,13 +516,16 @@ void newFirewallDialog::fillInterfaceData(Interface *intf, QTextBrowser *qte)
 void newFirewallDialog::addInterface()
 {
     QString dn = "";
-    if (iface_dyn->isChecked())   dn+="Dyn";
-    if (iface_unnum->isChecked()) dn+="Unn";
+    if (iface_dyn->isChecked())        dn+="Dyn";
+    if (iface_unnum->isChecked())      dn+="Unn";
+    if (iface_bridgeport->isChecked()) dn+="Bridge";
 
     QString addr;
     QString netm;
 
-    if (!iface_dyn->isChecked() && !iface_unnum->isChecked())
+    if (!iface_dyn->isChecked() &&
+        !iface_unnum->isChecked() &&
+        !iface_bridgeport->isChecked())
     {
         addr = iface_addr->text();
         netm = iface_netmask->text();
@@ -538,8 +562,10 @@ void newFirewallDialog::selectedInterface(QListViewItem *itm)
     iface_label->setText( itm->text(1) );
     iface_addr->setText( itm->text(2) );
     iface_netmask->setText( itm->text(3) );
+    iface_reg->setChecked( itm->text(4).isEmpty() );
     iface_dyn->setChecked( itm->text(4).find("Dyn")!=-1 );
     iface_unnum->setChecked( itm->text(4).find("Unn")!=-1 );
+    iface_bridgeport->setChecked( itm->text(4).find("Bridge")!=-1 );
     iface_physaddr->setText( itm->text(5) );
 }
 
@@ -548,6 +574,7 @@ void newFirewallDialog::updateInterface()
     QString dn = "";
     if (iface_dyn->isChecked())   dn+="Dyn";
     if (iface_unnum->isChecked()) dn+="Unn";
+    if (iface_bridgeport->isChecked()) dn+="Bridge";
 
     QListViewItem *itm = iface_list->selectedItem();
     if (itm==NULL) return;
@@ -681,6 +708,7 @@ void newFirewallDialog::accept()
             QString netmask =  itm->text(3);
             bool    dyn     =  itm->text(4).find("Dyn")!=-1;
             bool    unnum   =  itm->text(4).find("Unn")!=-1;
+            bool    bridgeport =  itm->text(4).find("Bridge")!=-1;
             QString physaddr=  itm->text(5);
 
             QListViewItem *itm2 = iface_sl_list->findItem( name , 0 );
@@ -690,14 +718,17 @@ void newFirewallDialog::accept()
 
             Interface *oi = Interface::cast(om->createObject(nfw,Interface::TYPENAME,
                                                              name));
+#ifdef USE_INTERFACE_POLICY
             oi->add(new InterfacePolicy());
+#endif
             oi->setLabel( label.latin1() );
 
-            if (dyn)   oi->setDyn(true);
-            if (unnum) oi->setUnnumbered(true);
+            oi->setDyn(dyn);
+            oi->setUnnumbered(unnum);
+            oi->setBridgePort(bridgeport);
             oi->setSecurityLevel(sl);
 
-            if (!dyn && !unnum)
+            if (!dyn && !unnum && !bridgeport)
             {
                 QString addrname=QString("%1:%2:ip").arg(obj_name->text()).arg(name);
                 IPv4 *oa = IPv4::cast(om->createObject(oi, IPv4::TYPENAME,addrname));
@@ -712,10 +743,12 @@ void newFirewallDialog::accept()
     }
     if (unloadTemplatesLib)
     {
+#if 0
         FWObject *tlib = mw->db()->getById(TEMPLATE_LIB);
         assert (tlib!=NULL);
         string tlibID = tlib->getId();
-        if (fwbdebug) qDebug("  Delete library of templates");
+        if (fwbdebug)
+            qDebug("newFirewallDialog::accept  Delete template library");
         om->delObj(tlib,false);
 
 /*
@@ -728,9 +761,13 @@ void newFirewallDialog::accept()
         FWObject *delObjLib = mw->db()->getById( DELETED_LIB );
         if (delObjLib!=NULL && delObjLib->getById(tlibID)!=NULL)
         {
-            if (fwbdebug) qDebug("  Delete library of templates from 'Deleted objects'");
+            if (fwbdebug) qDebug("newFirewallDialog::accept  Delete library of templates from 'Deleted objects'");
             om->delObj(tlib,false);  // this time from deleted objects lib
         }
+#endif
+
+        delete tmpldb;
+        tmpldb = NULL;
         unloadTemplatesLib=false;
     }
     QDialog::accept();
