@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newHostDialog.cpp,v 1.7 2005/04/30 18:22:27 vkurland Exp $
+  $Id: newHostDialog.cpp,v 1.10 2006/06/16 04:33:13 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -76,6 +76,7 @@ using namespace std;
 newHostDialog::newHostDialog() : newHostDialog_q()
 {
     nhst=NULL; 
+    tmpldb = NULL;
     snmpPollCompleted=false;
     q=NULL;
     unloadTemplatesLib = false;
@@ -315,6 +316,19 @@ void newHostDialog::selected(const QString &title)
     {
         setFinishEnabled( QWizard::page(TEMPLATES_PAGE), true );
 /* load templates if not loaded */
+
+        if (tmpldb==NULL)
+        {
+
+            MessageBoxUpgradePredicate upgrade_predicate(this);
+
+            tmpldb = new FWObjectDatabase();
+            tmpldb->setReadOnly( false );
+            tmpldb->load( tempfname, &upgrade_predicate, librespath);
+        }
+        FWObject *tlib = tmpldb->getById(TEMPLATE_LIB);
+
+#if 0
         FWObject *tlib = mw->db()->getById(TEMPLATE_LIB);
         if (tlib==NULL)
         {
@@ -326,6 +340,8 @@ void newHostDialog::selected(const QString &title)
 /* restore library that was opened prior loading templates */
             om->openLib(cl);
         }
+#endif
+
         list<FWObject*> fl;
         findHosts(tlib, fl);
 
@@ -515,11 +531,21 @@ void newHostDialog::accept()
                                            o,
                                            obj_name->text(),
                                            false );  // do not ask to autorename
+        if (no==NULL)
+        {
+          QDialog::accept();
+          return;
+        }
         nhst=Host::cast(no);
     } else
     {
         FWObject *o;
         o=om->createObject(Host::TYPENAME, obj_name->text() );
+        if (o==NULL)
+        {
+          QDialog::accept();
+          return;
+        }
 
         nhst=Host::cast(o);
 
@@ -540,7 +566,9 @@ void newHostDialog::accept()
             Interface *oi = Interface::cast(
                 om->createObject(nhst,Interface::TYPENAME, name)
             );
+#ifdef USE_INTERFACE_POLICY
             oi->add(new InterfacePolicy());
+#endif
             oi->setLabel( label.latin1() );
 
             if (dyn)   oi->setDyn(true);
@@ -565,6 +593,7 @@ void newHostDialog::accept()
     }
     if (unloadTemplatesLib)
     {
+#if 0
         FWObject *tlib = mw->db()->getById(TEMPLATE_LIB);
         assert (tlib!=NULL);
         string tlibID = tlib->getId();
@@ -584,6 +613,11 @@ void newHostDialog::accept()
             if (fwbdebug) qDebug("  Delete library of templates from 'Deleted objects'");
             om->delObj(tlib,false);  // this time from deleted objects lib
         }
+#endif
+
+        delete tmpldb;
+        tmpldb = NULL;
+
         unloadTemplatesLib=false;
     }
     QDialog::accept();
