@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pfAdvancedDialog.cpp,v 1.16.4.1 2005/09/04 20:49:14 vkurland Exp $
+  $Id: pfAdvancedDialog.cpp,v 1.19 2006/03/16 05:38:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,6 +29,7 @@
 
 #include "pfAdvancedDialog.h"
 #include "SimpleTextEditor.h"
+#include "ObjectManipulator.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
@@ -242,6 +243,7 @@ void pfAdvancedDialog::accept()
     pis->setCommand( installScript->text().latin1() );
     pis->setArguments( installScriptArgs->text().latin1() );
 
+    om->updateLastModifiedTimestampForAllFirewalls(obj);
     pfAdvancedDialog_q::accept();
 }
 
