@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pixAdvancedDialog.cpp,v 1.18.4.1 2005/09/13 04:55:41 vkurland Exp $
+  $Id: pixAdvancedDialog.cpp,v 1.26 2006/05/15 04:26:33 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -31,11 +31,15 @@
 
 #include "pixAdvancedDialog.h"
 #include "SimpleTextEditor.h"
+#include "ObjectManipulator.h"
+#include "FWWindow.h"
+#include "FWBSettings.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Interface.h"
+#include "fwbuilder/XMLTools.h"
 
 #include <qcheckbox.h>
 #include <qspinbox.h>
@@ -48,10 +52,13 @@
 #include <qtabwidget.h>
 #include <qlistbox.h>
 #include <qlabel.h>
+#include <qprocess.h>
 
 #include <iostream>
 #include <sstream>
 
+#include <libxml/xmlmemory.h>
+
 using namespace std;
 using namespace libfwbuilder;
 
@@ -61,6 +68,52 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
 {
     obj=o;
 
+    Firewall  *fw=Firewall::cast(obj);
+    FWOptions *fwopt=fw->getOptionsObject();
+    string compiler=fwopt->getStr("compiler");
+    if (compiler=="") 
+    {
+        compiler=Resources::platform_res[fw->getStr("platform")]->getCompiler();
+    }
+/*
+ * On Unix compilers are installed in the standard place and are
+ * accessible via PATH. On Windows and Mac they get installed in
+ * unpredictable directories and need to be found
+ *
+ * first, check if user specified an absolute path for the compiler,
+ * then check  if compiler is registsred in preferences, and if not,
+ * look for it in appRootDir and if it is not there, rely on PATH
+ */
+#if defined(Q_OS_WIN32) ||  defined(Q_OS_MACX)
+
+    if ( ! QFile::exists( compiler ) )
+    {
+        QString cmppath = st->getStr( QString("Compilers/")+compiler );
+        if (!cmppath.isEmpty()) compiler=cmppath.latin1();
+        else
+        {
+            /* try to find compiler in appRootDir. */
+
+            if ( QFile::exists( appRootDir + FS_SEPARATOR + compiler ) )
+                compiler = appRootDir + FS_SEPARATOR + compiler;
+        }
+    }
+#endif
+
+    fwb_pix_proc = new QProcess();
+    fwb_pix_proc->setCommunication( QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
+    connect(fwb_pix_proc, SIGNAL(readyReadStdout()), this,  SLOT(readFromStdout() ) );
+    connect(fwb_pix_proc, SIGNAL(readyReadStderr()), this,  SLOT(readFromStderr() ) );
+    connect(fwb_pix_proc, SIGNAL(processExited()),   this,  SLOT(fwb_pix_Finished() ) );
+    connect(fwb_pix_proc, SIGNAL(wroteToStdin()),   this,  SLOT(allXMLSent() ) );
+
+    fwb_pix_proc->addArgument( compiler.c_str() );
+    fwb_pix_proc->addArgument( "-f" );
+    fwb_pix_proc->addArgument( "-" );
+    fwb_pix_proc->addArgument( "-i" );
+    fwb_pix_proc->addArgument( fw->getName().c_str() );
+
+
     string vers="version_"+obj->getStr("version");
     string platform = obj->getStr("platform");   // could be 'pix' or 'fwsm'
 
@@ -181,6 +234,11 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     assert(mgmt!=NULL);
 
 /* Page "Compiler Options" */
+
+    bool outboundACLSupported= (Resources::platform_res[platform]->getResourceBool(
+                  "/FWBuilderResources/Target/options/"+vers+"/pix_outbound_acl_supported") );
+    pix_emulate_out_acl->setEnabled(!outboundACLSupported);
+
     data.registerOption( outputFileName       , fwoptions,
                          "output_file"  );
 
@@ -477,7 +535,7 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
 
     data.loadAll();
     loadFixups();
-    showFixups();
+    updateFixupCommandsDisplay();
     scriptACLModeChanged();
 }
 
@@ -522,7 +580,7 @@ void pixAdvancedDialog::changeAllFixups(int state)
         fi->switch_widget->setCurrentItem(
             translateFixupSwitchFromOptionToWidget(state) );            
     }
-    showFixups();
+    updateFixupCommandsDisplay();
 }
 
 void pixAdvancedDialog::loadFixups()
@@ -590,101 +648,74 @@ void pixAdvancedDialog::saveFixups()
     }
 }
 
-void pixAdvancedDialog::showFixups()
+void pixAdvancedDialog::displayCommands()
 {
-    string str;
-
     pix_generated_fixup->setText("");
 
-    for (list<fixupControl>::iterator fi=allFixups.begin(); fi!=allFixups.end(); fi++)
+/*
+ * need to copy information from widgets that control fixups into
+ * firewall object's options, so that when we dump the database into
+ * memory buffer, we get updated info
+ */
+    saveFixups();
+
+    xmlChar  *buffer;
+    int       bufsize;
+    mw->db()->saveToBuffer(&buffer,&bufsize);
+    proc_buffer = (char*)buffer;
+    FREEXMLBUFF(buffer);
+
+    if (!fwb_pix_proc->start() )
     {
-        if (!fi->active) continue;
-
-        string name=fi->fwoption.latin1();
-        int    sw=translateFixupSwitchFromWidgetToOption(
-                    fi->switch_widget->currentItem());
-        int    p1=(fi->arg1)?fi->arg1->value():0;
-        int    p2=(fi->arg2)?fi->arg2->value():0;
-        bool   op=(fi->arg3)?fi->arg3->isChecked():false;
-
-        pix_generated_fixup->append( _printFixupCommand(fi->fixup_cmd,sw,p1,p2,op) );
+        pix_generated_fixup->append(
+            tr("Error: Policy compiler for PIX is not installed") );
     }
+
+    fwb_pix_proc->writeToStdin(proc_buffer);
 }
 
-/*
- * This is a copy of the method  OSConfigurator_pix_os::_printFixupCommand
- */
-QString pixAdvancedDialog::_printFixupCommand(const QString &fixup_name,
-                                              int   sw,
-                                              int   arg1, 
-                                              int   arg2,
-                                              bool  ov)
+void pixAdvancedDialog::allXMLSent()
 {
-    ostringstream res;
+    fwb_pix_proc->closeStdin();
+}
 
-    if (sw==0)
-    {
-        if (fixup_name=="dns")
-        {
-            if (arg1)
-            {
-                res << "fixup protocol "  << fixup_name;
-                res << " maximum-length " << arg1;
-                res << endl;
-            }
-        } else
-        {
-            if (fixup_name=="ftp")
-            {
-                if (arg1)
-                {
-                    res << "fixup protocol " << fixup_name << " ";
-                    if (ov) res << "strict ";
-                    res << arg1;
-                    res << endl;
-                }
-            } else
-            {
-                if (fixup_name=="mgcp")
-                {
-                    if (arg1)
-                    {
-                        res << "fixup protocol " << fixup_name << " ";
-                        res << arg1;
-                        res << endl;
-                    }
-                    if (arg2) 
-                    {
-                        res << "fixup protocol " << fixup_name << " ";
-                        res << arg2;
-                        res << endl;
-                    }
-                } else 
-                {
-                    res << "fixup protocol " << fixup_name << " ";
-                    if (arg1) res << arg1;
-                    if (arg2 && arg1!=arg2) res << "-" << arg2;
-                    res << endl;
-                }
-            }
-        }
-    }
-    if (sw==1)
-    {
-        res << "no fixup protocol " << fixup_name;
-        res << endl;
-    }
+void pixAdvancedDialog::readFromStdout()
+{
+    pix_generated_fixup->append( QString(fwb_pix_proc->readStdout() ) );
+}
 
-    return res.str().c_str();
+void pixAdvancedDialog::readFromStderr()
+{
+    pix_generated_fixup->append( QString(fwb_pix_proc->readStderr() ) );
+}
+
+void pixAdvancedDialog::fwb_pix_Finished()
+{
+    if (fwb_pix_proc->normalExit() && fwb_pix_proc->exitStatus()!=0)
+        pix_generated_fixup->append( tr("Compiler error") );
+}
+
+
+void pixAdvancedDialog::updateFixupCommandsDisplay()
+{
+
+    pix_generated_fixup->setText("");
+    return;
+
+    QString stdoutBuffer;
+
+    if (!fwb_pix_proc->start() )
+    {
+        pix_generated_fixup->append(  tr("Error: Policy compiler for PIX is not installed") );
+    }
+    return;
 }
 
 void pixAdvancedDialog::fixupCmdChanged()
 {  
-    showFixups();
+    updateFixupCommandsDisplay();
 }
 
-
-
 /*
  * store all data in the object
  */
@@ -731,6 +762,7 @@ void pixAdvancedDialog::accept()
     pis->setCommand( installScript->text().latin1() );
     pis->setArguments( installScriptArgs->text().latin1() );
 
+    om->updateLastModifiedTimestampForAllFirewalls(obj);
     pixAdvancedDialog_q::accept();
 }
 
