@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: platforms.h,v 1.4.4.1 2005/09/04 20:49:14 vkurland Exp $
+  $Id: platforms.h,v 1.10 2006/06/17 21:26:22 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,49 +28,83 @@
 
 #include "config.h"
 
+// among other things, utils.h defines list<QStringPair> which we need here
+#include "utils.h"
+
 #include <qstring.h>
 #include <qstringlist.h>
-#include <qmap.h>
+
+#include <list>
+#include <utility>
+
+#include <fwbuilder/Rule.h>
 
 namespace libfwbuilder {
     class FWOptions;
     class Firewall;
 };
 
+void init_platforms();
+
 bool isUsingNetZone(libfwbuilder::Firewall *fw);
 
-bool isDefaultOptions(libfwbuilder::FWOptions *opt);
+bool isDefaultPolicyRuleOptions(libfwbuilder::FWOptions *opt);
+bool isDefaultNATRuleOptions(libfwbuilder::FWOptions *opt);
+bool isDefaultRoutingRuleOptions(libfwbuilder::FWOptions *opt);
 
-QMap<QString,QString> getVersionsForPlatform(const QString &platform);
+// using list of pairs instead of a map or QMap because maps are dictionaries
+// and do not preserve order of elements
+std::list<QStringPair> getVersionsForPlatform(const QString &platform);
+
+QString getVersionString(const QString &platform,const QString &version);
 
 /**
  * !!! returns a list of log levels that can be used to populate qcombobox
  * !!! widget.  I do not see how log levels can be different for various
  * !!! fw platforms, but who knows.
  */
-QStringList getLogLevels(const QString &platform);
+const QStringList& getLogLevels(const QString &platform);
 
 /**
  * like the above, except returns a list of log facilities.
  */
-QStringList getLogFacilities(const QString &platform);
+const QStringList& getLogFacilities(const QString &platform);
+
 /**
  * returns a list of Actions on reject (mapping list) 
  * 
  */
-QStringList getActionsOnReject(const QString &platform);
+const QStringList& getActionsOnReject(const QString &platform);
+
+/**
+ * returns a list of options for Route action
+ * 
+ */
+const QStringList& getRouteOptions(const QString &platform);
+
 /**
  * returns a list of Prolog places (mapping list)
  */
-QStringList getPrologPlaces(const QString &platform);
+const QStringList& getPrologPlaces(const QString &platform);
+
 /**
  * returns a list of Limit Suffixes (mapping list)
  */
-QStringList getLimitSuffixes(const QString &platform);
+const QStringList& getLimitSuffixes(const QString &platform);
+
 /**
  * returns a list of screen names from the mapping list that can be 
  * used to populate qcombobox.
  */
-QStringList getScreenNames(QStringList sl);
+QStringList getScreenNames(const QStringList &sl);
+
+/**
+ * finds screen name (i.e. string that can be localized) for the
+ * internal item name s in the mapping list sl
+ */
+QString getScreenName(QString s,const QStringList &sl);
+
+QString getActionNameForPlatform(libfwbuilder::PolicyRule::Action action,const QString &platform);
+
 #endif
 
