@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: utils.cpp,v 1.43 2005/03/21 06:36:47 vkurland Exp $
+  $Id: utils.cpp,v 1.45 2006/06/23 16:46:24 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -74,23 +74,25 @@ class mapValEQPredicate {
 
 
 
-void addPopupMenuItem(QObject *res,
-                      QPopupMenu* menu,
-                      const QString &resourceIconPath,
-                      const QString itemName,
-                      const char* member,
-                      const QKeySequence &accel)
+int addPopupMenuItem(QObject *res,
+                     QPopupMenu* menu,
+                     const QString &resourceIconPath,
+                     const QString itemName,
+                     const char* member,
+                     const QKeySequence &accel)
 {
     string icn;
     QPixmap pm;
+    int    itmID = -1;
 
     icn = Resources::global_res->getResourceStr(resourceIconPath.latin1());
     if(icn!="")
     {
         pm = QPixmap::fromMimeSource( icn.c_str() );
-        menu->insertItem( pm, itemName , res , member, accel );
+        itmID = menu->insertItem( pm, itemName , res , member, accel );
     } else
-        menu->insertItem( itemName , res , member, accel);
+        itmID = menu->insertItem( itemName , res , member, accel);
+    return itmID;
 }
 
 void fillLibraries(QComboBox *libs, libfwbuilder::FWObject *obj, bool rw)
@@ -396,3 +398,20 @@ QString getAddrByName(const QString &name)
     if (alist.empty()) return "";
     return alist.front().toString();
 }
+
+QString getNameByAddr(const QString &addr)
+{
+    QHostAddress ha;
+    ha.setAddress(addr);
+    QDns qry( ha, QDns::Ptr);
+    QStringList nlist = qry.hostNames();
+
+    while (qry.isWorking())
+    {
+        app->processEvents(200);
+    }
+    nlist = qry.hostNames();
+    if (nlist.empty()) return "";
+    return nlist.front();
+}
+
