@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: utils.h,v 1.24 2004/12/11 07:54:30 vkurland Exp $
+  $Id: utils.h,v 1.27 2006/06/23 16:46:24 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -41,6 +41,7 @@ class QListBox;
 #include <qkeysequence.h>
 #include <qstring.h>
 #include <qmap.h>
+#include <utility>
 
 #include "fwbuilder/FWObject.h"
 
@@ -61,13 +62,32 @@ class findFWObjectIDPredicate: public std::unary_function<libfwbuilder::FWObject
     bool operator()(const libfwbuilder::FWObject *o) const {return o->getId()==_id;}
 };
 
+typedef std::pair<const QString,QString> QStringPair;
 
-extern void addPopupMenuItem(QObject *res,
-                             QPopupMenu* menu,
-                             const QString &resourceIconPath,
-                             const QString itemName,
-                             const char* member,
-                             const QKeySequence &accel = 0);
+// a predicate used to compare first string in pair<QString,QString>
+// use with find_if
+class findFirstInQStringPair {
+    QString str;
+    public:
+    findFirstInQStringPair(const QString &d) { str=d; }
+    bool operator()(std::pair<const QString,QString> &_d) { return (str == _d.first); }
+};
+
+class findSecondInQStringPair {
+    QString str;
+    public:
+    findSecondInQStringPair(const QString &d) { str=d; }
+    bool operator()(std::pair<const QString,QString> &_d) { return (str == _d.second); }
+};
+
+
+
+extern int addPopupMenuItem(QObject *res,
+                            QPopupMenu* menu,
+                            const QString &resourceIconPath,
+                            const QString itemName,
+                            const char* member,
+                            const QKeySequence &accel = 0);
 
 extern void fillLibraries(QComboBox *libs, libfwbuilder::FWObject *obj, bool rw=false);
 extern void fillLibraries(QListBox *libs, libfwbuilder::FWObject *obj, bool rw=false);
@@ -128,5 +148,6 @@ extern void findByObjectType(libfwbuilder::FWObject *o,
 extern void setDisabledPalette(QWidget *w);
 
 extern QString getAddrByName(const QString &name);
+extern QString getNameByAddr(const QString &addr);
 
 #endif
