@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipf.cpp,v 1.19.4.1 2006/03/19 06:08:18 vkurland Exp $
+  $Id: ipf.cpp,v 1.25 2006/03/20 16:58:28 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -24,7 +24,7 @@
 */
 
 #include "config.h"
-
+ 
 #include <qsettings.h>
 
 #ifdef HAVE_LOCALE_H
@@ -59,6 +59,8 @@
 #include "OSConfigurator_freebsd.h"
 #include "OSConfigurator_solaris.h"
 
+#include "fwcompiler/Preprocessor.h"
+
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/XMLTools.h"
@@ -93,6 +95,8 @@ static int              drp            = -1;
 static int              drn            = -1;
 static int              verbose        = 0;
 
+FWObjectDatabase       *objdb = NULL;
+
 class UpgradePredicate: public XMLTools::UpgradePredicate
 {
     public:
@@ -238,30 +242,32 @@ int main(int argc, char * const *argv)
         new Resources(respath+FS_SEPARATOR+"resources.xml");
 
 	/* create database */
-	new FWObjectDatabase();
+	objdb = new FWObjectDatabase();
 
 	/* load the data file */
 	UpgradePredicate upgrade_predicate; 
 
 	if (verbose) cout << _(" *** Loading data ...");
 
-        FWObjectDatabase::db->setReadOnly( false );
-        FWObjectDatabase::db->load( sysfname, &upgrade_predicate, librespath);
-        FWObjectDatabase::db->setFileName("");
+        objdb->setReadOnly( false );
+        objdb->load( sysfname, &upgrade_predicate, librespath);
+        objdb->setFileName("");
         FWObjectDatabase *ndb = new FWObjectDatabase();
         ndb->load(filename, &upgrade_predicate,  librespath);
-        FWObjectDatabase::db->merge(ndb, NULL);
+        objdb->merge(ndb, NULL);
         delete ndb;
-        FWObjectDatabase::db->setFileName(filename);
+        objdb->setFileName(filename);
 
-//	FWObjectDatabase::db->load(filename,  &upgrade_predicate, librespath);
+//	objdb->load(filename,  &upgrade_predicate, librespath);
 	if (verbose) cout << _(" done\n");
 
-        FWObject *slib = FWObjectDatabase::db->getById("syslib000");
+        FWObject *slib = objdb->getById("syslib000");
         if ( slib->isReadOnly()) slib->setReadOnly(false);
 
 	/* Review firewall and OS options and generate commands */
-	Firewall*  fw=FWObjectDatabase::db->findFirewallByName(fwobjectname);
+	Firewall*  fw=objdb->findFirewallByName(fwobjectname);
+ 
+	if (verbose) cout << _(" *** Data checks ...");
 
         /* some initial sanity checks */
         list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
@@ -277,7 +283,7 @@ int main(int argc, char * const *argv)
                 {
                     char errstr[256];
                     for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
-                        if ( FWObjectDatabase::db->findAllReferences(*j).size()!=0 )
+                        if ( objdb->findAllReferences(*j).size()!=0 )
                         {
                             sprintf(errstr,
                                     _("Dynamic interface %s has an IP address that is used in the firewall policy rule.\n"),
@@ -296,8 +302,7 @@ int main(int argc, char * const *argv)
             {
 
                 list<FWObject*> la=iface->getByType(IPv4::TYPENAME);
-                if ( !iface->isDyn() && !iface->isUnnumbered() &&
-                     la.empty() )
+                if ( iface->isRegular() && la.empty() )
                 {
                     char errstr[256];
                     sprintf(errstr,_("Missing IP address for interface %s\n"),
@@ -321,6 +326,7 @@ int main(int argc, char * const *argv)
 
         }
 
+	if (verbose) cout << _(" done\n");
 
 	FWOptions* options=fw->getOptionsObject();
 	string s;
@@ -332,6 +338,8 @@ int main(int argc, char * const *argv)
 	string shell_dbg=(debug)?"-x":"" ;
 	string ipf_dbg=(debug)?"-v":"";
 
+        Preprocessor* prep=new Preprocessor(objdb , fwobjectname);
+        prep->compile();
 
 /*
  * Process firewall options, build OS network configuration script
@@ -339,13 +347,13 @@ int main(int argc, char * const *argv)
 	OSConfigurator *oscnf=NULL;
         string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
         if ( family=="solaris" )
-            oscnf=new OSConfigurator_solaris(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_solaris(objdb , fwobjectname);
 
 	if ( family=="openbsd")
-            oscnf=new OSConfigurator_openbsd(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_openbsd(objdb , fwobjectname);
 
 	if ( family=="freebsd")
-            oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_freebsd(objdb , fwobjectname);
 
 	if (oscnf==NULL)
 	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
@@ -355,25 +363,21 @@ int main(int argc, char * const *argv)
 /*
  * create compilers and run the whole thing 
  */
-
-	PolicyCompiler_ipf c( FWObjectDatabase::db , fwobjectname , oscnf );
+	PolicyCompiler_ipf c( objdb , fwobjectname , oscnf );
 
 	c.setDebugLevel( dl );
 	c.setDebugRule(  drp );
 	c.setVerbose( verbose );
 
-
 	bool     have_ipf=false;
 	if ( c.prolog() > 0 ) {
-
-	    have_ipf=true;
-
-	    c.compile();
-	    c.epilog();
+          have_ipf=true;
+          c.compile();
+          c.epilog();
 	}
 
 
-	NATCompiler_ipf n( FWObjectDatabase::db , fwobjectname , oscnf );
+	NATCompiler_ipf n( objdb , fwobjectname , oscnf );
 
 	n.setDebugLevel( dl );
 	n.setDebugRule(  drn );
@@ -381,17 +385,13 @@ int main(int argc, char * const *argv)
 
 	bool     have_nat=false;
 	if ( n.prolog() > 0 ) {
-
-	    have_nat=true;
-
-	    n.compile();
-	    n.epilog();
+          have_nat=true;
+          n.compile();
+          n.epilog();
 	}
 /*
  * now write generated scripts to files
  */
-
-
         char          *timestr;
         time_t         tm;
         struct tm     *stm;
@@ -576,6 +576,8 @@ int main(int argc, char * const *argv)
         chmod(fw_file_name.c_str(),S_IXUSR|S_IRUSR|S_IWUSR|S_IRGRP);
 #endif
         
+        cout << _(" Compiled successfully") << endl << flush;
+        
         return 0;
 
     } catch(const FWException &ex)  {
