@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipfw.cpp,v 1.18 2005/05/30 22:09:19 vkurland Exp $
+  $Id: ipfw.cpp,v 1.23 2006/03/20 16:58:29 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -24,7 +24,7 @@
 */
 
 #include "config.h"
-
+ 
 #include <qsettings.h>
 
 #ifdef HAVE_LOCALE_H
@@ -61,6 +61,8 @@
 #include "OSConfigurator_freebsd.h"
 #include "OSConfigurator_solaris.h"
 
+#include "fwcompiler/Preprocessor.h"
+
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/XMLTools.h"
@@ -93,6 +95,8 @@ static int              drp            = -1;
 static int              drn            = -1;
 static int              verbose        = 0;
 
+FWObjectDatabase       *objdb = NULL;
+
 class UpgradePredicate: public XMLTools::UpgradePredicate
 {
     public:
@@ -207,30 +211,30 @@ int main(int argc, char * const *argv)
         new Resources(respath+FS_SEPARATOR+"resources.xml");
 
 	/* create database */
-	new FWObjectDatabase();
+	objdb = new FWObjectDatabase();
 
 	/* load the data file */
 	UpgradePredicate upgrade_predicate; 
 
 	if (verbose) cout << _(" *** Loading data ...");
 
-        FWObjectDatabase::db->setReadOnly( false );
-        FWObjectDatabase::db->load( sysfname, &upgrade_predicate, librespath);
-        FWObjectDatabase::db->setFileName("");
+        objdb->setReadOnly( false );
+        objdb->load( sysfname, &upgrade_predicate, librespath);
+        objdb->setFileName("");
         FWObjectDatabase *ndb = new FWObjectDatabase();
         ndb->load(filename, &upgrade_predicate,  librespath);
-        FWObjectDatabase::db->merge(ndb, NULL);
+        objdb->merge(ndb, NULL);
         delete ndb;
-        FWObjectDatabase::db->setFileName(filename);
+        objdb->setFileName(filename);
 
-//	FWObjectDatabase::db->load(filename,  &upgrade_predicate, librespath);
+//	objdb->load(filename,  &upgrade_predicate, librespath);
 	if (verbose) cout << _(" done\n");
 
-        FWObject *slib = FWObjectDatabase::db->getById("syslib000");
+        FWObject *slib = objdb->getById("syslib000");
         if ( slib->isReadOnly()) slib->setReadOnly(false);
 
 	/* Review firewall and OS options and generate commands */
-	Firewall*  fw=FWObjectDatabase::db->findFirewallByName(fwobjectname);
+	Firewall*  fw=objdb->findFirewallByName(fwobjectname);
 
         /* some initial sanity checks */
         list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
@@ -246,7 +250,7 @@ int main(int argc, char * const *argv)
                 {
                     char errstr[256];
                     for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
-                        if ( FWObjectDatabase::db->findAllReferences(*j).size()!=0 )
+                        if ( objdb->findAllReferences(*j).size()!=0 )
                         {
                             sprintf(errstr,
                                     _("Dynamic interface %s has an IP address that is used in the firewall policy rule.\n"),
@@ -265,8 +269,7 @@ int main(int argc, char * const *argv)
             {
 
                 list<FWObject*> la=iface->getByType(IPv4::TYPENAME);
-                if ( !iface->isDyn() && !iface->isUnnumbered() &&
-                     la.empty() )
+                if ( iface->isRegular() && la.empty() )
                 {
                     char errstr[256];
                     sprintf(errstr,_("Missing IP address for interface %s\n"),
@@ -300,16 +303,19 @@ int main(int argc, char * const *argv)
 	bool debug=options->getBool("debug");
 	string shell_dbg=(debug)?"-x":"" ;
 
+        Preprocessor* prep=new Preprocessor(objdb , fwobjectname);
+        prep->compile();
+
 /*
  * Process firewall options, build OS network configuration script
  */
 	OSConfigurator *oscnf=NULL;
         string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
 	if ( family=="macosx")
-            oscnf=new OSConfigurator_macosx(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_macosx(objdb , fwobjectname);
 
 	if ( family=="freebsd")
-            oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_freebsd(objdb , fwobjectname);
 
 	if (oscnf==NULL)
 	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
@@ -318,7 +324,7 @@ int main(int argc, char * const *argv)
 /*
  * create compilers and run the whole thing 
  */
-	PolicyCompiler_ipfw c( FWObjectDatabase::db , fwobjectname , oscnf );
+	PolicyCompiler_ipfw c( objdb , fwobjectname , oscnf );
 
 	c.setDebugLevel( dl );
 	c.setDebugRule(  drp );
@@ -459,6 +465,8 @@ int main(int argc, char * const *argv)
         chmod(fw_file_name.c_str(),S_IXUSR|S_IRUSR|S_IWUSR|S_IRGRP);
 #endif
         
+        cout << _(" Compiled successfully") << endl << flush;
+        
         return 0;
 
     } catch(const FWException &ex)  {
