@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipt.cpp,v 1.11.4.1 2005/11/02 06:16:50 vkurland Exp $
+  $Id: NATCompiler_ipt.cpp,v 1.23 2006/06/05 04:53:21 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -24,8 +24,9 @@
 */
 
 #include "NATCompiler_ipt.h"
-#include "combinedAddress.h"
+#include "OSConfigurator_linux24.h"
 
+#include "combinedAddress.h"
 
 #include "fwcompiler/OSConfigurator.h"
 
@@ -36,11 +37,15 @@
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
+#include "fwbuilder/CustomService.h"
+#include "fwbuilder/TagService.h"
 #include "fwbuilder/Host.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/AddressTable.h"
+#include "fwbuilder/DNSName.h"
 
 #include "config.h"
 
@@ -84,6 +89,17 @@ string NATCompiler_ipt::getInterfaceVarName(FWObject *iface)
     return ostr.str();
 }
 
+string NATCompiler_ipt::getAddressTableVarName(FWObject *at)
+{
+    ostringstream  ostr;
+    string name=at->getName();
+    string::size_type p1;
+    while ( (p1=name.find(" "))!=string::npos)
+        name=name.replace(p1,1,"_");
+    ostr << "at_" << name;
+    return ostr.str();
+}
+
 string NATCompiler_ipt::getNewTmpChainName(NATRule *rule)
 {
     std::ostringstream str;
@@ -206,7 +222,7 @@ bool NATCompiler_ipt::ConvertLoadBalancingRules::processNext()
         {
             FWObject *o= *i;
             FWObject *obj = NULL;
-            if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
             Address *a=Address::cast(obj);
 
             al.push_back( a->getAddress() );
@@ -547,7 +563,7 @@ bool NATCompiler_ipt::splitOnODst::processNext()
 	for(list<FWObject*>::iterator i=odst->begin(); i!=odst->end(); ++i) 
         {
             FWObject *o= *i;
-            if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    Address *a=Address::cast( o );
 	    assert(a);
 
@@ -578,7 +594,7 @@ bool NATCompiler_ipt::splitOnOSrv::processNext()
 	for(list<FWObject*>::iterator i=osrv->begin(); i!=osrv->end(); ++i) 
         {
             FWObject *o= *i;
-            if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    Service *s=Service::cast( o );
 	    assert(s);
 
@@ -673,9 +689,9 @@ bool NATCompiler_ipt::splitRuleIfRuleElementIsDynamicInterface::processNext()
     {
         FWObject *o= *i;
 	FWObject *obj = NULL;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Interface *iface=Interface::cast(obj);
-        if (iface!=NULL && (iface->isDyn() || iface->isUnnumbered()) )
+        if (iface!=NULL && !iface->isRegular())
         {
 	    cl.push_back(o);   // can not remove right now because remove invalidates iterator
             nre--;
@@ -710,10 +726,12 @@ bool NATCompiler_ipt::specialCaseWithUnnumberedInterface::dropUnnumberedInterfac
     {
         FWObject *o   = *i1;
         FWObject *obj = o;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Interface  *ifs   =Interface::cast( obj );
 
-        if (ifs!=NULL && ifs->isUnnumbered() ) cl.push_back(obj);
+        if (ifs!=NULL &&
+            (ifs->isUnnumbered() || ifs->isBridgePort())
+           ) cl.push_back(obj);
     }
     if (!cl.empty()) 
     {
@@ -865,7 +883,10 @@ bool NATCompiler_ipt::ReplaceFirewallObjectsTSrc::processNext()
                 for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
                 {
                     Interface *iface=Interface::cast(*i);
-                    if (! iface->isLoopback() && ! iface->isUnnumbered() ) 
+                    if (! iface->isLoopback() &&
+                        ! iface->isUnnumbered() &&
+                        ! iface->isBridgePort()
+                    )
                         rel->addRef( *i );
                 }
                 for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1)
@@ -989,8 +1010,7 @@ bool NATCompiler_ipt::dynamicInterfaceInTSrc::processNext()
     Address  *tsrc=compiler->getFirstTSrc(rule);
 
     if (rule->getRuleType()==NATRule::SNAT && 
-        Interface::cast(tsrc)!=NULL &&
-        (Interface::cast(tsrc)->isDyn() || Interface::cast(tsrc)->isUnnumbered()))
+        Interface::cast(tsrc)!=NULL && !Interface::cast(tsrc)->isRegular())
     {
 
             rule->setRuleType(NATRule::Masq);
@@ -1032,7 +1052,7 @@ void NATCompiler_ipt::checkForDynamicInterfacesOfOtherObjects::findDynamicInterf
     {
         FWObject *o   = *i1;
         FWObject *obj = o;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Interface  *ifs   =Interface::cast( obj );
 
         if (ifs!=NULL && ifs->isDyn() && ! ifs->isChildOf(compiler->fw))        
@@ -1081,7 +1101,7 @@ bool NATCompiler_ipt::splitServices::processNext()
     for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
 	Service *s=Service::cast( o );
 	assert(s);
@@ -1125,7 +1145,7 @@ bool NATCompiler_ipt::separatePortRanges::processNext()
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
 	    
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast(o);
 	assert(s!=NULL);
 
@@ -1172,13 +1192,13 @@ bool NATCompiler_ipt::separateSourcePorts::processNext()
     }
 
     NATRule         *rule_4_src_ports=NULL;
-    RuleElementOSrv *nsrv;
+    RuleElementOSrv *nsrv = NULL;
 
     list<Service*> services;
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast(o);
 	assert(s!=NULL);
 
@@ -1200,6 +1220,7 @@ bool NATCompiler_ipt::separateSourcePorts::processNext()
                     nsrv->clearChildren();
                     tmp_queue.push_back(rule_4_src_ports);
                 }
+                assert(nsrv!=NULL);
                 nsrv->addRef( s );
                 services.push_back(s);
             }
@@ -1226,13 +1247,13 @@ bool NATCompiler_ipt::separateSourceAndDestinationPorts::processNext()
     }
 
     NATRule         *nrule=NULL;
-    RuleElementOSrv *nsrv;
+    RuleElementOSrv *nsrv = NULL;
 
     list<Service*> services;
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast(o);
 	assert(s!=NULL);
 
@@ -1257,6 +1278,7 @@ bool NATCompiler_ipt::separateSourceAndDestinationPorts::processNext()
                     nsrv->clearChildren();
                     tmp_queue.push_back(nrule);
                 }
+                assert(nsrv!=NULL);
                 nsrv->addRef( s );
                 services.push_back(s);
             } 
@@ -1296,11 +1318,11 @@ bool NATCompiler_ipt::prepareForMultiport::processNext()
         {
 	    int n=0;
 	    NATRule         *r;
-	    RuleElementOSrv *nsrv;
+	    RuleElementOSrv *nsrv=NULL;
 	    for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
             {
 		FWObject *o= *i;
-		if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+		if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    
 		Service *s=Service::cast( o );
 		assert(s);
@@ -1315,6 +1337,7 @@ bool NATCompiler_ipt::prepareForMultiport::processNext()
 		    nsrv->clearChildren();
 		    tmp_queue.push_back(r);
 		}
+                assert(nsrv!=NULL);
 		nsrv->addRef( s );
 		if (++n>=15) n=0;
 	    }
@@ -1347,7 +1370,7 @@ bool NATCompiler_ipt::splitMultipleICMP::processNext()
         for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
         {
             FWObject *o= *i;
-            if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    
             Service *s=Service::cast( o );
             assert(s);
@@ -1644,13 +1667,14 @@ bool NATCompiler_ipt::localNATRule::processNext()
 
 //    if ( rule->getStr("ipt_chain").empty())
 //    {
-        Address *osrc=compiler->getFirstOSrc(rule);
-        bool osrcfw= compiler->complexMatch(osrc,compiler->fw);
 
-        switch( rule->getRuleType())
-        {
-        case NATRule::DNAT:
-        case NATRule::DNetnat:
+    Address          *osrc=compiler->getFirstOSrc(rule);
+    bool osrcfw= compiler->complexMatch(osrc,compiler->fw);
+
+    switch( rule->getRuleType())
+    {
+    case NATRule::DNAT:
+    case NATRule::DNetnat:
 
 /* it should not be necessary to do anything if rule type is NONAT
  * since splitNONATRule takes care of NONAT rules
@@ -1660,18 +1684,18 @@ bool NATCompiler_ipt::localNATRule::processNext()
  *
  * Can use OUTPUT chain only for DNAT rules and a like
  */
-            if (osrcfw) rule->setStr("ipt_chain","OUTPUT");
-            if (osrcfw && osrc->getId()==compiler->fw->getId())
-            {
-                RuleElementOSrc *src;
-                src=rule->getOSrc();
-                src->clearChildren();
-                src->setAnyElement();
-            }
-            break;
-        default:
-            break;
+        if (osrcfw) rule->setStr("ipt_chain","OUTPUT");
+        if (osrcfw && osrc->getId()==compiler->fw->getId())
+        {
+            RuleElementOSrc *src;
+            src=rule->getOSrc();
+            src->clearChildren();
+            src->setAnyElement();
         }
+        break;
+    default:
+        break;
+    }
 //    }
 
     tmp_queue.push_back(rule);
@@ -1832,7 +1856,10 @@ bool NATCompiler_ipt::AssignInterface::processNext()
             Interface *iface=Interface::cast(*i);
             assert(iface);
 
-            if (iface->isLoopback() || iface->isUnnumbered()) continue;
+            if (iface->isLoopback() ||
+                iface->isUnnumbered() ||
+                iface->isBridgePort()
+            ) continue;
 
             char *in=strdup( iface->getName().c_str() );
             char *cptr=in;
@@ -1923,7 +1950,7 @@ bool NATCompiler_ipt::verifyRuleWithMAC::processNext()
         {
             FWObject *o= *i;
             FWObject *o1= o;
-            if (FWReference::cast(o)!=NULL) o1=compiler->getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) o1=FWReference::cast(o)->getPointer();
 
             if (physAddress::isA(o1))
             {
@@ -1974,18 +2001,87 @@ _("SNAT rule can not match MAC address. Object %s removed from the rule %s"),
     return true;
 }
 
+bool NATCompiler_ipt::processMultiAddressObjectsInRE::processNext()
+{
+    NATRule *rule=getNext(); if (rule==NULL) return false;
+
+    OSConfigurator_linux24 *osconf = 
+        dynamic_cast<OSConfigurator_linux24*>(compiler->osconfigurator);
+
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+    bool neg = re->getNeg();
+
+    if (re->size()==1) 
+    {
+        FWObject *o = re->front();
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+        if (atrt!=NULL)
+        {
+            // we have just one object in RE and this object is MutiAddressRunTime
+            if (atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+            {
+                rule->setStr("address_table_file",atrt->getSourceName());
+                osconf->registerMultiAddressObject(atrt);
+            }
+            if (atrt->getSubstitutionTypeName()==DNSName::TYPENAME)
+            {
+                // this is DNSName converted to its run-time counterpart,
+                // we do not need to touch it at all
+            }
+            tmp_queue.push_back(rule);
+            return true;
+        }
+    }
+
+    list<MultiAddressRunTime*> cl;
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+        if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+            cl.push_back(atrt);
+    }
+
+    if (!cl.empty())
+    {
+        RuleElement *nre;
+        RuleElement *ore=RuleElement::cast( rule->getFirstByType(re_type) );
+        NATRule *r;
+        for (list<MultiAddressRunTime*>::iterator i=cl.begin(); i!=cl.end(); i++)
+        {
+            MultiAddressRunTime *atrt = *i;
+            r = NATRule::cast(compiler->dbcopy->create(NATRule::TYPENAME,false) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            nre=RuleElement::cast( r->getFirstByType(re_type) );
+            nre->clearChildren();
+            nre->addRef( atrt );
+            r->setStr("address_table_file",atrt->getSourceName());
+            osconf->registerMultiAddressObject(atrt);
+            tmp_queue.push_back(r);
+
+            ore->removeRef( *i );
+        }
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
 
 void NATCompiler_ipt::compile()
 {
 //    FWOptions* options=fw->getOptionsObject();
 
-    cout << _(" Compiling NAT rules for ") << fw->getName() << " ..." << endl << flush;
+    cout << _(" Compiling rules for 'nat' table ...") << endl << flush;
 
     try {
 
 	Compiler::compile();
 
-        add( new NATCompiler::Begin(             "Begin processing"        ));
+        add( new NATCompiler::Begin());
         add( new printTotalNumberOfRules(                                  ));
 
         add( new recursiveGroupsInOSrc("check for recursive groups in OSRC"));
@@ -2011,7 +2107,11 @@ void NATCompiler_ipt::compile()
         add( new eliminateDuplicatesInODST("eliminate duplicates in ODST"));
         add( new eliminateDuplicatesInOSRV("eliminate duplicates in OSRV"));
 
+        add( new swapMultiAddressObjectsInOSrc(" swap MultiAddress -> MultiAddressRunTime in OSrc") );
+        add( new swapMultiAddressObjectsInODst(" swap MultiAddress -> MultiAddressRunTime in ODst") );
 
+        add( new processMultiAddressObjectsInOSrc("process MultiAddress objects in OSrc") );
+        add( new processMultiAddressObjectsInODst("process MultiAddress objects in ODst") );
 
         add( new doOSrvNegation(   "process negation in OSrv"            ));
 
@@ -2108,7 +2208,6 @@ void NATCompiler_ipt::epilog()
     {
         output << "#" << endl;
     }
-    cout << _(" NAT rules compiled successfully \n") << flush;
 }
 
 string NATCompiler_ipt::flushAndSetDefaultPolicy()
@@ -2118,6 +2217,7 @@ string NATCompiler_ipt::flushAndSetDefaultPolicy()
 /* printRule may be null if there are no NAT rules and we never ran compile() */
     if(printRule!=NULL)
     {
+        res += printRule->_declareTable();
         res += printRule->_flushAndSetDefaultPolicy();
 //    res += printRule->_printOptionalGlobalRules();
     }
