@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.h,v 1.8 2004/12/29 07:56:23 vkurland Exp $
+  $Id: PolicyCompiler_ipt.h,v 1.22 2006/06/15 05:06:08 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -59,12 +59,14 @@ namespace fwcompiler {
         class PrintRule;
 
         PolicyCompiler_ipt::PrintRule *printRule;
-        bool   have_dynamic_interfaces;
-
+        bool                           have_dynamic_interfaces;
+        bool                           have_connmark;
+        std::string                    my_table;
         
-        PolicyCompiler_ipt::PrintRule* addPrintRuleProcessor();
+        PolicyCompiler_ipt::PrintRule* createPrintRuleProcessor();
 
         std::string getInterfaceVarName(libfwbuilder::FWObject *iface);
+        std::string getAddressTableVarName(libfwbuilder::FWObject *iface);
         
 	/**
 	 * Add some predefined rules controlled by checkboxes in
@@ -90,7 +92,12 @@ namespace fwcompiler {
          * chain and target at the end of the printed line
          */
         virtual std::string debugPrintRule(libfwbuilder::Rule *rule);
-        
+
+        /**
+         * this processor drops all rules that require mangle table
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(dropMangleTableRules);
+
         /**
          * adds few predefined (or "builtin") rules on top of the policy
          */
@@ -111,6 +118,16 @@ namespace fwcompiler {
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(storeAction);
 
+	/**
+	 * set target and chain in case of branching
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(Branching);
+
+	/**
+	 * set target and chain in case of route rules
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(Route);
+
 	/**
 	 * turns logging on if global logging override is used
 	 */
@@ -123,6 +140,17 @@ namespace fwcompiler {
         DECLARE_POLICY_RULE_PROCESSOR(Logging2);
 
 
+	/**
+	 * this processor checks if the rule is associated with an
+	 * interface and uses setInterfaceId to record its id. If the
+	 * rule is associated with multiple interfaces, this processor
+	 * splits the rule accordingly. Unlike basic processor
+	 * PolicyCompiler::InterfacePolicyrules, this processor tries
+	 * to optimize rules applied to multiple interfaces using
+	 * user-defined chains
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(InterfacePolicyRulesWithOptimization);
+
 	/**
 	 * if option "firewall is part of any" is OFF, replace all *
 	 * "Any" with "!fw" before checking for rule shadowing (if fw is
@@ -196,6 +224,11 @@ namespace fwcompiler {
         };
 
 
+	/**
+	 * split rule if action is Tag and connmark option is activated
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(splitIfTagAndConnmark);
+
 	/**
 	 * split rule if Src==any 
 	 * 
@@ -271,6 +304,34 @@ namespace fwcompiler {
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(checkSrcAndDst2);
 
+        /**
+         * Split rule if MultiAddress object is used in RE to make
+         * sure it is single object.
+         */
+        class processMultiAddressObjectsInRE : public PolicyRuleProcessor
+        {
+            std::string re_type;
+            public:
+            processMultiAddressObjectsInRE(const std::string &name,
+                      const std::string &t) : PolicyRuleProcessor(name) { re_type=t; }
+            virtual bool processNext();
+        };
+
+
+        class processMultiAddressObjectsInSrc : public processMultiAddressObjectsInRE
+        {
+            public:
+            processMultiAddressObjectsInSrc(const std::string &n) :
+                processMultiAddressObjectsInRE(n,libfwbuilder::RuleElementSrc::TYPENAME) {}
+        };
+
+        class processMultiAddressObjectsInDst : public processMultiAddressObjectsInRE
+        {
+            public:
+            processMultiAddressObjectsInDst(const std::string &n) :
+                processMultiAddressObjectsInRE(n,libfwbuilder::RuleElementDst::TYPENAME) {}
+        };
+
 	/**
 	 * splits rule if firewall is in src and dst
 	 */
@@ -338,7 +399,8 @@ namespace fwcompiler {
          * that we can drop it later in removeFW, but only after
          * decideOnChain has determined that chain is INPUT or OUTPUT.
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(expandMultipleAddressesIfNotFW);
+        DECLARE_POLICY_RULE_PROCESSOR(expandMultipleAddressesIfNotFWinSrc);
+        DECLARE_POLICY_RULE_PROCESSOR(expandMultipleAddressesIfNotFWinDst);
 
         /**
          * Compiler::_expandAddr skips loopback interface, so we need
@@ -394,6 +456,11 @@ namespace fwcompiler {
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(decideOnChainIfLoopback);
 
+        /**
+         * define chain for rules with action Classify
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(decideOnChainForClassify);
+
 	/**
 	 * decides what chain this rule should go to if it has not
 	 * been decided in decideOnChainIfFW
@@ -410,7 +477,9 @@ namespace fwcompiler {
 	 * remove firewall object from dst or src (resp.) NB: we can
 	 * remove only reference to the whole firewall. We DO NOT
 	 * remove reference to its interface or (in the future)
-	 * address objects under interfaces
+	 * address objects under interfaces. We do this only if we do
+	 * not add any virtual addresses for NAT and if original rule
+	 * did not have negation.
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(removeFW);
 
@@ -592,8 +661,6 @@ namespace fwcompiler {
             std::map<const std::string,bool> chains;
 
             virtual std::string _createChain(const std::string &chain);
-            virtual std::string _startRuleLine();
-            virtual std::string _endRuleLine();
             virtual std::string _printRuleLabel(libfwbuilder::PolicyRule *r);
 
             virtual std::string _printSrcService(libfwbuilder::RuleElementSrv  *o);
@@ -613,13 +680,21 @@ namespace fwcompiler {
 
             virtual std::string _printChain(libfwbuilder::PolicyRule *r);
             virtual std::string _printTarget(libfwbuilder::PolicyRule *r);
+            virtual std::string _printModules(libfwbuilder::PolicyRule *r);
             virtual std::string _printDirectionAndInterface(libfwbuilder::PolicyRule *r);
             virtual std::string _printMultiport(libfwbuilder::PolicyRule *r);
 
             virtual std::string _printTimeInterval(libfwbuilder::PolicyRule *r);
 
             virtual std::string _printLogParameters(libfwbuilder::PolicyRule *r);
-            virtual std::string _printLogPrefix(libfwbuilder::PolicyRule *r,const std::string &prefix);
+            virtual std::string _printLogPrefix(int rule_num,
+						const std::string &action,
+						const std::string &interf,
+						const std::string &chain,
+						const std::string &rule_label,
+						const std::string &prefix);
+            virtual std::string _printLogPrefix(libfwbuilder::PolicyRule *r,
+						const std::string &prefix);
             virtual std::string _printActionOnReject(libfwbuilder::PolicyRule *r);
             virtual std::string _printLimit(libfwbuilder::PolicyRule *r);
 
@@ -628,9 +703,13 @@ namespace fwcompiler {
             PrintRule(const std::string &name);
             virtual std::string _printGlobalLogParameters();
             virtual std::string _printOptionalGlobalRules();
+            virtual std::string _declareTable();
             virtual std::string _flushAndSetDefaultPolicy();
             virtual std::string _commit();
             virtual std::string _quote(const std::string &s);
+
+            virtual std::string _startRuleLine();
+            virtual std::string _endRuleLine();
             
             virtual bool processNext();
 
@@ -648,6 +727,7 @@ namespace fwcompiler {
 
             public:
             PrintRuleIptRst(const std::string &name) : PrintRule(name) {};
+            virtual std::string _declareTable();
             virtual std::string _flushAndSetDefaultPolicy();
             virtual std::string _commit();
             virtual std::string _quote(const std::string &s);
@@ -664,6 +744,7 @@ namespace fwcompiler {
 
             public:
             PrintRuleIptRstEcho(const std::string &name) : PrintRuleIptRst(name) {};
+            virtual std::string _declareTable();
             virtual std::string _flushAndSetDefaultPolicy();
             virtual std::string _commit();
             virtual std::string _quote(const std::string &s);
@@ -689,16 +770,27 @@ namespace fwcompiler {
 	PolicyCompiler_ipt(libfwbuilder::FWObjectDatabase *_db,
 			  const std::string &fwname,
                 fwcompiler::OSConfigurator *_oscnf) : PolicyCompiler(_db,fwname,_oscnf)
-            {have_dynamic_interfaces=false; printRule=NULL;}
+        {
+            have_dynamic_interfaces = false;
+            have_connmark = false;
+            printRule = NULL;
+            my_table = "filter";
+        }
 
 
 	virtual int  prolog();
 	virtual void compile();
 	virtual void epilog();
 
+        /**
+         * addRuleFilter() is a hook where we can add a rule processor to filter
+         * some of the rules out before we begin actual processing
+         */
+        virtual void addRuleFilter();
+        
         void setHaveDynamicInterfaces(bool f) { have_dynamic_interfaces=f; }
         
-        std::string flushAndSetDefaultPolicy();
+        virtual std::string flushAndSetDefaultPolicy();
         std::string commit();
 
 	static std::string getNewTmpChainName(libfwbuilder::PolicyRule *rule);
