@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt_optimizer.cpp,v 1.6 2004/11/11 06:15:41 vkurland Exp $
+  $Id: PolicyCompiler_ipt_optimizer.cpp,v 1.8 2006/07/21 05:25:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -100,7 +100,7 @@ void PolicyCompiler_ipt::optimize1::optimizeForRuleElement(PolicyRule    *rule,
                    if (TCPService::isA(srv))
                    {
                        re->clearChildren();
-                       re->addRef(compiler->getCachedObj(ANY_TCP_OBJ_ID));
+                       re->addRef(compiler->dbcopy->findInIndex(ANY_TCP_OBJ_ID));
 /* also leave a flag indicating that further optimization by service
  * is not needed */
                        rule->setBool("do_not_optimize_by_srv",true);
@@ -151,6 +151,25 @@ bool PolicyCompiler_ipt::optimize1::processNext()
     int  intn=1;
     if (intrel!=NULL) intn=intrel->size();
 
+    bool all_tcp_or_udp = true;
+    for (FWObject::iterator i=srvrel->begin(); i!=srvrel->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+	    
+        Service *s=Service::cast( o );
+        assert(s);
+
+// tcp and udp will be collapsed because we can use multiport module
+        if ( !TCPService::isA(s) && !UDPService::isA(s))
+        {
+            all_tcp_or_udp = false;
+            break;
+        }
+    }
+    
+    if (all_tcp_or_udp) srvn = 1;
+
 // Golden rule - try to introduce minimum forward rules ....
 // we can't optimize 1 src, 1 dstn, 1 service and 1 time interval
 // we can't optimize if we've got three 'anys' ..
