@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipt.cpp,v 1.41 2005/05/29 03:13:02 vkurland Exp $
+  $Id: ipt.cpp,v 1.55 2006/06/05 04:53:22 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -53,9 +53,13 @@
 #include <assert.h>
 
 #include "PolicyCompiler_ipt.h"
+#include "MangleTableCompiler_ipt.h"
 #include "NATCompiler_ipt.h"
+#include "RoutingCompiler_ipt.h"
 #include "OSConfigurator_linux24.h"
 
+#include "fwcompiler/Preprocessor.h"
+
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/XMLTools.h"
@@ -89,6 +93,8 @@ static int              drn            = -1;
 static int              verbose        = 0;
 static bool             have_dynamic_interfaces = false;
 
+FWObjectDatabase       *objdb = NULL;
+
 class UpgradePredicate: public XMLTools::UpgradePredicate
 {
     public:
@@ -125,7 +131,7 @@ string addPrologScript(bool nocomment,const string &script)
 
 void usage(const char *name)
 {
-    cout << _("Firewall Builder:  policy compiler for Linux 2.4.xx iptables") << endl;
+    cout << _("Firewall Builder:  policy compiler for Linux 2.4.x and 2.6.x iptables") << endl;
     cout << _("Version ") << VERSION << "-" << RELEASE_NUM << endl;
     cout << _("Usage: ") << name << _(" [-x level] [-v] [-V] [-f filename.xml] [-d destdir] [-m] firewall_object_name") << endl;
 }
@@ -226,30 +232,31 @@ int main(int argc, char * const *argv)
         new Resources(respath+FS_SEPARATOR+"resources.xml");
 
 	/* create database */
-	new FWObjectDatabase();
+	objdb = new FWObjectDatabase();
 
 	/* load the data file */
 	UpgradePredicate upgrade_predicate; 
 
 	if (verbose) cout << _(" *** Loading data ...");
 
-        FWObjectDatabase::db->setReadOnly( false );
-        FWObjectDatabase::db->load( sysfname, &upgrade_predicate, librespath);
-        FWObjectDatabase::db->setFileName("");
+        objdb->setReadOnly( false );
+        objdb->load( sysfname, &upgrade_predicate, librespath);
+        objdb->setFileName("");
         FWObjectDatabase *ndb = new FWObjectDatabase();
         ndb->load(filename, &upgrade_predicate,  librespath);
-        FWObjectDatabase::db->merge(ndb, NULL);
-        delete ndb;
-        FWObjectDatabase::db->setFileName(filename);
 
-//	FWObjectDatabase::db->load(filename,  &upgrade_predicate, librespath);
+        objdb->merge(ndb, NULL);
+        delete ndb;
+        objdb->setFileName(filename);
+
+//	objdb->load(filename,  &upgrade_predicate, librespath);
 	if (verbose) cout << _(" done\n");
 
-        FWObject *slib = FWObjectDatabase::db->getById("syslib000");
+        FWObject *slib = objdb->findInIndex("syslib000");
         if ( slib->isReadOnly()) slib->setReadOnly(false);
 
 	/* Review firewall and OS options and generate commands */
-	Firewall*  fw=FWObjectDatabase::db->findFirewallByName(fwobjectname);
+	Firewall*  fw=objdb->findFirewallByName(fwobjectname);
 	FWOptions* options=fw->getOptionsObject();
 	string s;
 
@@ -313,7 +320,7 @@ _("Wildcard interface '%s' should not have a physcal address object attached to
                 {
                     char errstr[256];
                     for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
-                        if ( FWObjectDatabase::db->findAllReferences(*j).size()!=0 )
+                        if ( objdb->findAllReferences(*j).size()!=0 )
                         {
                             sprintf(errstr,
 _("Dynamic interface %s has an IP address that is used in the firewall policy rule.\n"),
@@ -332,8 +339,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
             {
 
                 list<FWObject*> la=iface->getByType(IPv4::TYPENAME);
-                if ( !iface->isDyn() && !iface->isUnnumbered() &&
-                     la.empty() )
+                if ( iface->isRegular() && la.empty() )
                 {
                     char errstr[256];
                     sprintf(errstr,_("Missing IP address for interface %s\n"),
@@ -363,48 +369,89 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 	string shell_dbg=(debug)?"set -x":"" ;
 	string pfctl_dbg=(debug)?"-v":"";
 
+        Preprocessor* prep=new Preprocessor(objdb , fwobjectname);
+        prep->compile();
+
 	OSConfigurator_linux24 *oscnf=NULL;
         string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
         if ( family=="linux24" )
-            oscnf=new OSConfigurator_linux24(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_linux24(objdb , fwobjectname);
 
 	if (oscnf==NULL)
 	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
 
         oscnf->prolog();
 
-	PolicyCompiler_ipt c( FWObjectDatabase::db , fwobjectname , oscnf );
+        int policy_rules_count  = 0;
+        int mangle_rules_count  = 0;
+        int nat_rules_count     = 0;
+        int routing_rules_count = 0;
 
-	c.setDebugLevel( dl );
-	c.setDebugRule(  drp );
-	c.setVerbose( (bool)(verbose) );
-        c.setHaveDynamicInterfaces(have_dynamic_interfaces);
+	MangleTableCompiler_ipt m( objdb , fwobjectname , oscnf );
 
-	if ( c.prolog() > 0 )
+	m.setDebugLevel( dl );
+	m.setDebugRule(  drp );
+	m.setVerbose( (bool)(verbose) );
+        m.setHaveDynamicInterfaces(have_dynamic_interfaces);
+
+	if ( (mangle_rules_count=m.prolog()) > 0 )
         {
-	    c.compile();
-	    c.epilog();
+	    m.compile();
+	    m.epilog();
 	}
 
-	NATCompiler_ipt n( FWObjectDatabase::db , fwobjectname , oscnf );
+// compile NAT rules before policy rules because policy compiler
+// needs to know the number of virtual addresses being created for NAT
+
+	NATCompiler_ipt n( objdb , fwobjectname , oscnf );
 
 	n.setDebugLevel( dl );
 	n.setDebugRule(  drn );
 	n.setVerbose( (bool)(verbose) );
         n.setHaveDynamicInterfaces(have_dynamic_interfaces);
 
-	if ( n.prolog() > 0 ) {
+	if ( (nat_rules_count=n.prolog()) > 0 )
+        {
             oscnf->generateCodeForProtocolHandlers(true);
 	    n.compile();
 	    n.epilog();
 	} else
             oscnf->generateCodeForProtocolHandlers(false);
 
+	PolicyCompiler_ipt c( objdb , fwobjectname , oscnf );
+
+	c.setDebugLevel( dl );
+	c.setDebugRule(  drp );
+	c.setVerbose( (bool)(verbose) );
+        c.setHaveDynamicInterfaces(have_dynamic_interfaces);
+
+	if ( (policy_rules_count=c.prolog()) > 0 )
+        {
+	    c.compile();
+	    c.epilog();
+	}
+
+        RoutingCompiler_ipt r( objdb , fwobjectname , oscnf );
+
+	r.setDebugLevel( dl );
+	r.setDebugRule(  drp );
+	r.setVerbose( verbose );
+
+	if ( (routing_rules_count=r.prolog()) > 0 )
+        {
+	    r.compile();
+	    r.epilog();
+	}
+
+        oscnf->printChecksForRunTimeMultiAddress();
+        oscnf->processFirewallOptions();
+        oscnf->configureInterfaces();
+        oscnf->printCommandsToAddVirtualAddressesForNAT();
+
 /*
  * now write generated scripts to files
  */
 
-
         char          *timestr;
         time_t         tm;
         struct tm     *stm;
@@ -475,6 +522,14 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         script << "export PATH" << endl;
         script << endl;
 
+/*
+ * print definitions for variables IPTABLES, IP, LOGGER. Some day we may
+ * add a choice of distro in the GUI. Right now paths are either default
+ * for a given distro, or custom strings entered by user in the GUI and stored
+ * in firewall options.
+ */
+        script << oscnf->printPathForAllTools(DISTRO);
+
         string prolog_place= fw->getOptionsObject()->getStr("prolog_place");
         if (prolog_place == "") prolog_place="top";
 
@@ -516,6 +571,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
             if (have_dynamic_interfaces)
             {
                 script << "(" << endl;
+
                 script << c.flushAndSetDefaultPolicy();
 
                 if (prolog_place == "after_flush")
@@ -526,14 +582,25 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
                 script << c.getCompiledScript();
                 script << c.commit();
-                script << n.flushAndSetDefaultPolicy();
-                script << n.getCompiledScript();
-                script << n.commit();
+
+                if (m.getCompiledScriptLength()>0)
+                {
+                    script << m.flushAndSetDefaultPolicy();
+                    script << m.getCompiledScript();
+                    script << m.commit();
+                }
+                if (n.getCompiledScriptLength()>0)
+                {
+                    script << n.flushAndSetDefaultPolicy();
+                    script << n.getCompiledScript();
+                    script << n.commit();
+                }
                 script << "#" << endl;
                 script << ") | $IPTABLES_RESTORE" << endl;
             } else
             {
                 script << "cat << EOF | $IPTABLES_RESTORE" << endl;
+
                 script << c.flushAndSetDefaultPolicy();
 
                 if (prolog_place == "after_flush")
@@ -544,19 +611,30 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
                 script << c.getCompiledScript();
                 script << c.commit();
-                script << n.flushAndSetDefaultPolicy();
-                script << n.getCompiledScript();
-                script << n.commit();
+
+                if (m.getCompiledScriptLength()>0)
+                {
+                    script << m.flushAndSetDefaultPolicy();
+                    script << m.getCompiledScript();
+                    script << m.commit();
+                }
+                if (n.getCompiledScriptLength()>0)
+                {
+                    script << n.flushAndSetDefaultPolicy();
+                    script << n.getCompiledScript();
+                    script << n.commit();
+                }
                 script << "#" << endl;
                 script << "EOF" << endl;
             }
         } else
         {
-/* the order here is historical: I always installed default policy and
- * added optional global rules before adding NAT and added normal
- * policy after NAT
- */
+
             script << c.flushAndSetDefaultPolicy();
+            if (m.getCompiledScriptLength()>0)
+                script << m.flushAndSetDefaultPolicy();
+            if (n.getCompiledScriptLength()>0)
+                script << n.flushAndSetDefaultPolicy();
 
             if (prolog_place == "after_flush")
             {
@@ -564,12 +642,22 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
                              fw->getOptionsObject()->getStr("prolog_script"));
             }
 
-            script << n.flushAndSetDefaultPolicy();
-            script << n.getCompiledScript();
-            script << n.commit();
+            if (n.getCompiledScriptLength()>0)
+            {
+                script << n.getCompiledScript();
+                script << n.commit();
+            }
+
+            if (m.getCompiledScriptLength()>0)
+            {
+                script << m.getCompiledScript();
+                script << m.commit();
+            }
+
             script << c.getCompiledScript();
             script << c.commit();
         }
+        script << r.getCompiledScript();
 
 
 
@@ -658,6 +746,8 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 #else
         chmod(fw_file_name.c_str(),S_IXUSR|S_IRUSR|S_IWUSR|S_IRGRP);
 #endif
+
+        cout << _(" Compiled successfully") << endl << flush;
         
         return 0;
 
