@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: pf.cpp,v 1.24 2005/05/29 03:13:02 vkurland Exp $
+  $Id: pf.cpp,v 1.33 2006/05/21 04:47:10 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -24,7 +24,7 @@
 */
 
 #include "config.h"
-
+ 
 #include <qsettings.h>
 
 #ifdef HAVE_LOCALE_H
@@ -55,6 +55,8 @@
 
 #include "PolicyCompiler_pf.h"
 #include "NATCompiler_pf.h"
+#include "TableFactory.h"
+#include "Preprocessor_pf.h"
 
 #include "OSConfigurator_openbsd.h"
 #include "OSConfigurator_freebsd.h"
@@ -66,6 +68,7 @@
 #include "fwbuilder/FWException.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Interface.h"
+#include "fwbuilder/Policy.h"
 
 #ifdef HAVE_GETOPT_H
   #include <getopt.h>
@@ -93,6 +96,11 @@ static int              drp            = -1;
 static int              drn            = -1;
 static int              verbose        = 0;
 
+static map<string,RuleSet*> branches;
+static map<string,string>   anchor_files;
+
+FWObjectDatabase       *objdb = NULL;
+
 class UpgradePredicate: public XMLTools::UpgradePredicate
 {
     public:
@@ -231,30 +239,30 @@ int main(int argc, char * const *argv)
         new Resources(respath+FS_SEPARATOR+"resources.xml");
 
 	/* create database */
-	new FWObjectDatabase();
+	objdb = new FWObjectDatabase();
 
 	/* load the data file */
 	UpgradePredicate upgrade_predicate; 
 
 	if (verbose) cout << _(" *** Loading data ...");
 
-        FWObjectDatabase::db->setReadOnly( false );
-        FWObjectDatabase::db->load( sysfname, &upgrade_predicate, librespath);
-        FWObjectDatabase::db->setFileName("");
+        objdb->setReadOnly( false );
+        objdb->load( sysfname, &upgrade_predicate, librespath);
+        objdb->setFileName("");
         FWObjectDatabase *ndb = new FWObjectDatabase();
         ndb->load(filename, &upgrade_predicate,  librespath);
-        FWObjectDatabase::db->merge(ndb, NULL);
+        objdb->merge(ndb, NULL);
         delete ndb;
-        FWObjectDatabase::db->setFileName(filename);
+        objdb->setFileName(filename);
 
-//	FWObjectDatabase::db->load(filename,  &upgrade_predicate, librespath);
+//	objdb->load(filename,  &upgrade_predicate, librespath);
 	if (verbose) cout << _(" done\n");
 
-        FWObject *slib = FWObjectDatabase::db->getById("syslib000");
+        FWObject *slib = objdb->getById("syslib000");
         if ( slib->isReadOnly()) slib->setReadOnly(false);
 
 	/* Review firewall and OS options and generate commands */
-	Firewall*  fw=FWObjectDatabase::db->findFirewallByName(fwobjectname);
+	Firewall*  fw=objdb->findFirewallByName(fwobjectname);
 
         /* some initial sanity checks */
         list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
@@ -270,7 +278,7 @@ int main(int argc, char * const *argv)
                 {
                     char errstr[256];
                     for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
-                        if ( FWObjectDatabase::db->findAllReferences(*j).size()!=0 )
+                        if ( objdb->findAllReferences(*j).size()!=0 )
                         {
                             sprintf(errstr,
 _("Dynamic interface %s has an IP address that is used in the firewall policy rule.\n"),
@@ -289,8 +297,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
             {
 
                 list<FWObject*> la=iface->getByType(IPv4::TYPENAME);
-                if ( !iface->isDyn() && !iface->isUnnumbered() &&
-                     la.empty() )
+                if ( iface->isRegular() && la.empty() )
                 {
                     char errstr[256];
                     sprintf(errstr,_("Missing IP address for interface %s\n"),
@@ -331,6 +338,9 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 //        if (fw->getStr("version")=="obsd_3.2")    pfctl_f_option="-f ";
         if (fw->getStr("version")=="obsd_lt_3.2") pfctl_f_option="-R ";
         
+        Preprocessor_pf* prep=new Preprocessor_pf(objdb , fwobjectname);
+        prep->compile();
+
 /*
  * Process firewall options, build OS network configuration script
  */
@@ -338,25 +348,48 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
 
 	if (family=="solaris")
-            oscnf=new OSConfigurator_solaris(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_solaris(objdb , fwobjectname);
 
 	if (family=="openbsd")
-            oscnf=new OSConfigurator_openbsd(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_openbsd(objdb , fwobjectname);
 
 	if (family=="freebsd")
-            oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+            oscnf=new OSConfigurator_freebsd(objdb , fwobjectname);
 
 	if (oscnf==NULL)
 	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
 
 	oscnf->prolog();
 
-/*
- * create compilers and run the whole thing 
- */
+        // find branching rules and store names of the branches and
+        // pointers to corresponding rule sets
+        // 
+        FWObject *policy = fw->getFirstByType(Policy::TYPENAME);
+        for (FWObject::iterator i=policy->begin(); i!=policy->end(); i++)
+        {
+            PolicyRule *rule = PolicyRule::cast(*i);
+            if (rule->getAction()==PolicyRule::Branch)
+            {
+                int parentRuleNum = rule->getPosition();
+                RuleSet *subset = rule->getBranch();
+                if (subset==NULL)
+                {
+                    throw FWException(
+                        _("Action 'Branch' but no branch policy in policy rule ")
+                        +rule->getLabel());
+                }
+                subset->setInt("parent_rule_num",parentRuleNum);
+                FWOptions *ropt = rule->getOptionsObject();
+                string branchName = ropt->getStr("branch_name");
+                branches[branchName] = subset;
+                subset->ref();
+                rule->remove(subset);
+            }
+        }
 
+        TableFactory *table_factory = new TableFactory();
 
-	NATCompiler_pf n( FWObjectDatabase::db , fwobjectname , oscnf );
+	NATCompiler_pf n( objdb, fwobjectname, oscnf, table_factory );
 
 	n.setDebugLevel( dl );
 	n.setDebugRule(  drn );
@@ -371,7 +404,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 	    n.epilog();
 	}
 
-	PolicyCompiler_pf c( FWObjectDatabase::db , fwobjectname , oscnf , &n );
+	PolicyCompiler_pf c( objdb, fwobjectname, oscnf, &n, table_factory );
 
 	c.setDebugLevel( dl );
 	c.setDebugRule(  drp );
@@ -382,6 +415,10 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         {
 	    have_pf=true;
 
+            cout << " Compiling policy rules for "
+                 << fwobjectname
+                 << " ..." <<  endl << flush;
+
 	    c.compile();
 	    c.epilog();
 	}
@@ -529,11 +566,69 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         }
         pf_file << endl;
 
+        pf_file << table_factory->PrintTables();
+
 	if (have_nat)   pf_file << n.getCompiledScript();
         if (have_pf)    pf_file << c.getCompiledScript();
         pf_file.close();
 
 
+        // run policy compiler for each branch we have found in the
+        // ruleset and store the result in a separate .conf file
+        //
+        map<string,RuleSet*>::iterator i;
+        for (i=branches.begin(); i!=branches.end(); ++i)
+        {
+            table_factory = new TableFactory();
+
+            string branchName = i->first;
+            RuleSet *subset = i->second;
+            PolicyCompiler_pf c( objdb , fwobjectname , oscnf , &n, table_factory );
+            c.setSourceRuleSet( subset );
+            c.setRuleSetName(branchName);
+
+            c.setDebugLevel( dl );
+            c.setDebugRule(  drp );
+            c.setVerbose( verbose );
+
+            if ( c.prolog() > 0 ) 
+            {
+                cout << " Compiling rules for anchor "
+                     << branchName
+                     << " ..." <<  endl << flush;
+
+                c.compile();
+                c.epilog();
+
+                string anchor_file_name;
+                if (fw_file_name.empty())
+                {
+                    anchor_file_name=string(fwobjectname) + "-" + branchName + ".conf";
+                } else
+                {
+                    string::size_type n = fw_file_name.rfind(".");
+                    anchor_file_name = fw_file_name;
+                    anchor_file_name.erase(n);
+                    anchor_file_name.append("-" + branchName + ".conf");
+                }
+                anchor_files[branchName] = anchor_file_name;
+
+                ofstream pf_file;
+                pf_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+
+#ifdef _WIN32
+                pf_file.open(anchor_file_name.c_str(), ios::out|ios::binary);
+#else
+                pf_file.open(anchor_file_name.c_str());
+#endif
+                pf_file << endl;
+                pf_file << table_factory->PrintTables();
+                pf_file << endl;
+                pf_file << c.getCompiledScript();
+                pf_file.close();
+            }
+        }
+
 
         char          *timestr;
         time_t         tm;
@@ -581,6 +676,10 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
         fw_file << MANIFEST_MARKER << "* " << fw_file_name << endl;
         fw_file << MANIFEST_MARKER << "  " << pf_file_name << endl;
+        for (map<string,string>::iterator i=anchor_files.begin();
+                                          i!=anchor_files.end(); ++i)
+            fw_file << MANIFEST_MARKER << "  " << i->second << endl;
+
         fw_file << "#" << endl;
         fw_file << "#" << endl;
 
@@ -619,6 +718,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
                 << "$PFCTL -F Sources" << endl
                 << "$PFCTL -F Tables" << endl;
 
+
 /*
  *  we add prolog and epilog to the activation shell script rather
  *  than to the .conf file. This is more flexible since user can
@@ -643,6 +743,14 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
         fw_file << "$PFCTL " << pfctl_dbg << pfctl_f_option
                 << "${FWDIR}/" << pf_file_name << endl;
+
+        for (map<string,string>::iterator i=anchor_files.begin();
+             i!=anchor_files.end(); ++i)
+            fw_file << "$PFCTL " << pfctl_dbg
+                    << "-a " << i->first << " "
+                    << pfctl_f_option
+                    << "${FWDIR}/" << i->second << endl;
+
 	fw_file << "$PFCTL -e" << endl;
         
         fw_file << endl;
@@ -665,6 +773,8 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 #else
         chmod(fw_file_name.c_str(),S_IXUSR|S_IRUSR|S_IWUSR|S_IRGRP);
 #endif
+
+        cout << _(" Compiled successfully") << endl << flush;
         
         return 0;
 
