@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipf.cpp,v 1.2 2005/03/29 08:38:33 vkurland Exp $
+  $Id: NATCompiler_ipf.cpp,v 1.9 2006/05/17 02:30:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -39,6 +39,7 @@
 #include "fwbuilder/Network.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/AddressTable.h"
 
 #include <iostream>
 
@@ -85,7 +86,7 @@ bool NATCompiler_ipf::VerifyRules::processNext()
 	throw FWException(_("Translated service should be 'Original' or should contain single object. Rule: ")+rule->getLabel());
 
     FWObject *o=tsrv->front();
-    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     if ( Group::cast(o)!=NULL)
 	throw FWException(_("Can not use group in translated service. Rule ")+rule->getLabel());
@@ -245,7 +246,9 @@ bool NATCompiler_ipf::AssignInterface::processNext()
         {
             Interface *iface=Interface::cast(*j);
             assert(iface);
-            if ( iface->isUnnumbered() || iface->isLoopback()) continue;
+            if ( iface->isUnnumbered() || 
+                 iface->isBridgePort() || 
+                 iface->isLoopback()) continue;
 
             NATRule *r= NATRule::cast(compiler->dbcopy->create(NATRule::TYPENAME,true) 
 );
@@ -281,7 +284,9 @@ bool NATCompiler_ipf::AssignInterface::processNext()
             Interface *iface=Interface::cast(*i);
             assert(iface);
 
-            if (iface->isLoopback() || iface->isUnnumbered()) continue;
+            if (iface->isLoopback() ||
+                iface->isUnnumbered() ||
+                iface->isBridgePort()) continue;
 
             NATRule *r = NATRule::cast(
                 compiler->dbcopy->create(NATRule::TYPENAME,false) );
@@ -321,7 +326,7 @@ bool NATCompiler_ipf::prepareForLB::processNext()
             for(list<FWObject*>::iterator i=tdst->begin(); i!=tdst->end(); ++i) 
             {
                 FWObject *o= *i;
-                if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+                if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
                 cl.push_back(o);
                 if (cl.size()==2)
@@ -394,18 +399,24 @@ bool NATCompiler_ipf::appProxy::processNext()
     bool raudio_proxy = compiler->getCachedFwOpt()->getBool("ipf_nat_raudio_proxy");
     bool h323_proxy   = compiler->getCachedFwOpt()->getBool("ipf_nat_h323_proxy");
     bool ipsec_proxy  = compiler->getCachedFwOpt()->getBool("ipf_nat_ipsec_proxy");
+    bool pptp_proxy  = compiler->getCachedFwOpt()->getBool("ipf_nat_pptp_proxy");
+    bool irc_proxy  = compiler->getCachedFwOpt()->getBool("ipf_nat_irc_proxy");
 
     char ipsec_proxy_str[64];
     char ftp_proxy_str[64];
     char rcmd_proxy_str[64];
     char raudio_proxy_str[64];
     char h323_proxy_str[64];
+    char pptp_proxy_str[64];
+    char irc_proxy_str[64];
 
     sprintf(ipsec_proxy_str,  "proxy port %d ipsec/udp ",  ISAKMP_PORT);
     sprintf(ftp_proxy_str,    "proxy port %d ftp/tcp ",    FTP_PORT);
     sprintf(rcmd_proxy_str,   "proxy port %d rcmd/tcp ",   RCMD_PORT);
     sprintf(raudio_proxy_str, "proxy port %d raudio/tcp ", RAUDIO_PORT);
     sprintf(h323_proxy_str,   "proxy port %d h323/tcp ",   H323_PORT);
+    sprintf(pptp_proxy_str,   "proxy port %d pptp/tcp ",   PPTP_PORT);
+    sprintf(irc_proxy_str,    "proxy port %d irc/tcp ",    IRC_PORT);
 
 
     if (rule->getRuleType()==NATRule::SNAT || 
@@ -439,6 +450,14 @@ bool NATCompiler_ipf::appProxy::processNext()
             if (h323_proxy &&
                 s->getInt("dst_range_start")==H323_PORT && s->getInt("dst_range_end")==H323_PORT )   
                 rule->setStr("nat_rule_proxy",h323_proxy_str);
+
+            if (pptp_proxy &&
+                s->getInt("dst_range_start")==PPTP_PORT && s->getInt("dst_range_end")==PPTP_PORT )   
+                rule->setStr("nat_rule_proxy",pptp_proxy_str);
+
+            if (irc_proxy &&
+                s->getInt("dst_range_start")==IRC_PORT && s->getInt("dst_range_end")==IRC_PORT )   
+                rule->setStr("nat_rule_proxy",irc_proxy_str);
         }
     }
     tmp_queue.push_back(rule);
@@ -465,6 +484,24 @@ bool NATCompiler_ipf::expandAnyService::processNext()
     return true;
 }
 
+bool NATCompiler_ipf::processMultiAddressObjectsInRE::processNext()
+{
+    NATRule *rule=getNext(); if (rule==NULL) return false;
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+        if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+            compiler->abort("Run-time AddressTable objects are not supported. Rule " + rule->getLabel());
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
 
 void NATCompiler_ipf::compile()
 {
@@ -476,7 +513,7 @@ void NATCompiler_ipf::compile()
 
 	Compiler::compile();
 
-        add( new Begin("Begin processing") );
+        add( new Begin());
         add( new printTotalNumberOfRules() );
 
         add( new recursiveGroupsInOSrc(      "check for recursive groups in OSRC"     ) );
@@ -500,6 +537,12 @@ void NATCompiler_ipf::compile()
         add( new eliminateDuplicatesInODST(  "eliminate duplicates in ODST"              ) );
         add( new eliminateDuplicatesInOSRV(  "eliminate duplicates in OSRV"              ) );
 
+        add( new swapMultiAddressObjectsInOSrc(" swap MultiAddress -> MultiAddressRunTime in OSrc") );
+        add( new swapMultiAddressObjectsInODst(" swap MultiAddress -> MultiAddressRunTime in ODst") );
+
+        add( new processMultiAddressObjectsInOSrc("process MultiAddress objects in OSrc") );
+        add( new processMultiAddressObjectsInODst("process MultiAddress objects in ODst") );
+
         add( new splitOnOSrv( "split rule on original service" ) );
 	add( new ExpandPortRange("expand port ranges") );
         add( new fillTranslatedSrv( "fill translated service" ) );
@@ -540,5 +583,4 @@ void NATCompiler_ipf::compile()
 
 void NATCompiler_ipf::epilog()
 {
-    cout << _(" NAT rules compiled successfully \n") << flush;
 }
