@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_pf.cpp,v 1.3.4.1 2005/09/05 05:31:23 vkurland Exp $
+  $Id: NATCompiler_pf.cpp,v 1.15 2006/05/21 04:47:10 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -41,6 +41,7 @@
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/AddressTable.h"
 
 #include <iostream>
 #include <iomanip>
@@ -56,7 +57,6 @@ string NATCompiler_pf::myPlatformName() { return "pf"; }
 
 int NATCompiler_pf::prolog()
 {
-
     int n=NATCompiler::prolog();
 
     if ( n>0 ) 
@@ -88,7 +88,7 @@ int NATCompiler_pf::prolog()
     }
 
 /* pseudo-host with ip address 127.0.0.1  We'll use it for redirection NAT rules */
-    FWObject    *grp;
+    //FWObject    *grp;
     loopback_address=IPv4::cast(dbcopy->create(IPv4::TYPENAME,false) );
     loopback_address->setAddress("127.0.0.1");
     loopback_address->setName("__loopback_address__");
@@ -96,6 +96,8 @@ int NATCompiler_pf::prolog()
     dbcopy->add(loopback_address,false);
     cacheObj(loopback_address);
 
+    if (tables) tables->init(dbcopy);
+
     return n;
 }
 
@@ -119,8 +121,8 @@ bool NATCompiler_pf::NATRuleType::processNext()
 
     RuleElementTDst *tdstre=rule->getTDst();
 
-    Address  *osrc=compiler->getFirstOSrc(rule);
-    Address  *odst=compiler->getFirstODst(rule);
+    //Address  *osrc=compiler->getFirstOSrc(rule);
+    //Address  *odst=compiler->getFirstODst(rule);
                                              
     Address  *tsrc=compiler->getFirstTSrc(rule);
     Address  *tdst=compiler->getFirstTDst(rule);
@@ -242,8 +244,8 @@ bool NATCompiler_pf::VerifyRules::processNext()
     RuleElementTDst  *tdst=rule->getTDst();  assert(tdst);
     RuleElementTSrv  *tsrv=rule->getTSrv();  assert(tsrv);
 
-    if (rule->getRuleType()==NATRule::LB)
-        throw FWException(_("Load balancing rules are not supported. Rule ")+rule->getLabel());
+//    if (rule->getRuleType()==NATRule::LB)
+//        throw FWException(_("Load balancing rules are not supported. Rule ")+rule->getLabel());
 
     if (rule->getRuleType()==NATRule::DNAT && odst->size()!=1)
 	throw FWException(_("There should be no more than one object in original destination in the rule ")+rule->getLabel());
@@ -277,21 +279,19 @@ bool NATCompiler_pf::VerifyRules::processNext()
 	throw FWException(_("Translated service should be 'Original' or should contain single object. Rule: ")+rule->getLabel());
 
     FWObject *o=tsrv->front();
-    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     if ( Group::cast(o)!=NULL)
 	throw FWException(_("Can not use group in translated service. Rule ")+rule->getLabel());
 
-
+#if 0
     if (rule->getRuleType()==NATRule::SNAT ) 
     {
-//        if ( tsrc->size()!=1)
-//            throw FWException(_("There should be no more than one object in translated source in the rule ")+rule->getLabel());
-
         Address* o1=compiler->getFirstTSrc(rule);
         if ( Network::cast(o1)!=NULL || AddressRange::cast(o1)!=NULL )
             throw FWException(_("Can not use network or address range object in translated source. Rule ")+rule->getLabel());
     }
+#endif
 
     if (rule->getRuleType()==NATRule::DNAT || rule->getRuleType()==NATRule::Redirect ) 
     {
@@ -335,7 +335,7 @@ bool NATCompiler_pf::splitOnOSrv::processNext()
 	for(list<FWObject*>::iterator i=osrv->begin(); i!=osrv->end(); ++i) {
 	    FWObject *o= *i;
 //	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-	    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    Service *s=Service::cast( o );
 	    assert(s);
 
@@ -386,7 +386,8 @@ bool NATCompiler_pf::addVirtualAddress::processNext()
     if ( ! a->isAny() && a->getId()!=compiler->getFwId() ) {
 
 	list<FWObject*> l2=compiler->fw->getByType(Interface::TYPENAME);
-	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) {
+	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
+        {
 	    Interface *iface=dynamic_cast<Interface*>(*i);
 	    assert(iface);
 
@@ -399,6 +400,50 @@ bool NATCompiler_pf::addVirtualAddress::processNext()
 }
 
 
+bool NATCompiler_pf::splitForTSrc::processNext()
+{
+    NATRule *rule=getNext(); if (rule==NULL) return false;
+    RuleElementTSrc  *tsrc=rule->getTSrc();  assert(tsrc);
+
+    map<string,list<FWObject*> > interfaceGroups;
+
+    for(list<FWObject*>::iterator i=tsrc->begin(); i!=tsrc->end(); ++i)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        Interface *iface = compiler->findInterfaceFor(Address::cast(o),compiler->fw);
+        if (iface!=NULL)
+            interfaceGroups[iface->getId()].push_back(o);
+    }
+
+    if (interfaceGroups.size()==1) tmp_queue.push_back(rule);
+    else
+    {
+        map<string,list<FWObject*> >::iterator i;
+        for (i=interfaceGroups.begin(); i!=interfaceGroups.end(); i++)
+        {
+            list<FWObject*> &objSubset = (*i).second;
+
+            RuleElementTSrc  *ntsrc = NULL;
+            NATRule *r = NATRule::cast(
+                compiler->dbcopy->create(NATRule::TYPENAME,false) );
+            r->duplicate(rule);
+            compiler->temp_ruleset->add(r);
+                
+            ntsrc=r->getTSrc();
+            ntsrc->clearChildren();
+            ntsrc->setAnyElement();
+            for (FWObject::iterator j=objSubset.begin(); j!=objSubset.end(); j++)
+            {
+                ntsrc->addRef(*j);
+            }
+            tmp_queue.push_back(r);
+        }
+    }
+    return true;
+}
+
+
 bool NATCompiler_pf::AssignInterface::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
@@ -412,7 +457,9 @@ bool NATCompiler_pf::AssignInterface::processNext()
             Interface *iface=Interface::cast(*i);
             assert(iface);
 
-            if (iface->isLoopback() || iface->isUnnumbered()) continue;
+            if (iface->isLoopback() ||
+                iface->isUnnumbered() ||
+                iface->isBridgePort()) continue;
 
             if (n) regular_interfaces+=",";
             regular_interfaces+= iface->getName();
@@ -426,7 +473,15 @@ bool NATCompiler_pf::AssignInterface::processNext()
     case NATRule::SNAT:
     {
 	Address *a=compiler->getFirstTSrc(rule);
-
+#if 0
+        Interface *iface = compiler->findInterfaceFor(a,compiler->fw);
+        if (iface!=NULL)
+        {
+            rule->setInterfaceId( iface->getId() );
+            tmp_queue.push_back(rule);
+            return true;
+        }
+#endif
         if ( (Interface::isA(a) || IPv4::isA(a)) && a->isChildOf(compiler->fw))
         {
             FWObject *p=a;
@@ -437,9 +492,9 @@ bool NATCompiler_pf::AssignInterface::processNext()
         }
 
 /* if we appear here, then TSrc is not an interface or address of
- * an interface 
+ * an interface. Generate NAT rule without "on iface" clause
  */
-        rule->setInterfaceStr( regular_interfaces );
+        rule->setInterfaceStr("");
     }
     break;
 
@@ -460,7 +515,7 @@ bool NATCompiler_pf::AssignInterface::processNext()
  * interface. If this is so, just do not specify interface for rdr
  * rule.
  */
-
+        rule->setInterfaceStr("");
     }
     break;
 
@@ -546,14 +601,17 @@ bool NATCompiler_pf::ReplaceFirewallObjectsTSrc::processNext()
 
             Interface *iface=compiler->findInterfaceFor(odst,compiler->fw);
 
-            if (!odst->isAny() && iface!=NULL) rel->addRef(iface);
+            if (!odst->isAny() && !rule->getODst()->getNeg() && iface!=NULL)
+                rel->addRef(iface);
             else  // else use all interfaces except loopback and unnumbered ones
             {
                 list<FWObject*> l2=compiler->fw->getByType(Interface::TYPENAME);
                 for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
                 {
                     Interface *iface=Interface::cast(*i);
-                    if (! iface->isLoopback() && ! iface->isUnnumbered() ) 
+                    if (! iface->isLoopback() &&
+                        ! iface->isUnnumbered() &&
+                        ! iface->isBridgePort()) 
                         rel->addRef( *i );
                 }
                 for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1)
@@ -631,6 +689,147 @@ bool NATCompiler_pf::ReplaceObjectsTDst::processNext()
     return true;
 }
 
+
+bool NATCompiler_pf::swapAddressTableObjectsInRE::processNext()
+{
+    NATCompiler_pf *pf_comp=dynamic_cast<NATCompiler_pf*>(compiler);
+    Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
+
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+
+    list<MultiAddress*> cl;
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        /* 
+         * All addressTable objects will be run-time here because we
+         * switch them in preprocessor. The difference is: if address
+         * table was originally run-time, at this point it will have
+         * no children, however if it was compile-time originally, it
+         * will have children objects.  That is how we distinguish
+         * them in this rule processor. Here we only deal with
+         * AddressTable objects that originally used to be
+         * compile-time because we need to create tables for them.
+         */
+        if (AddressTable::cast(o)!=NULL &&
+            AddressTable::cast(o)->isRunTime() &&
+            o->size() > 0)
+            cl.push_back(MultiAddress::cast(o));
+    }
+
+    if (!cl.empty())
+    {
+        for (list<MultiAddress*>::iterator i=cl.begin(); i!=cl.end(); i++)
+        {
+            MultiAddress *atbl = *i;
+
+            string mart_id = atbl->getId()+"_runtime";
+            MultiAddressRunTime *mart = 
+                MultiAddressRunTime::cast(compiler->dbcopy->findInIndex(mart_id));
+            if (mart==NULL)
+            {
+                mart = new MultiAddressRunTime(atbl);
+
+                // need to ensure stable ID for the runtime object, so
+                // that when the same object is replaced in different
+                // rulesets by different compiler passes, chosen
+                // runtime object has the same ID and is identified as
+                // the same by the compiler.
+
+                mart->setId( mart_id );
+                compiler->dbcopy->addToIndex(mart);
+                compiler->dbcopy->add(mart);
+
+// register this object as a table
+                string tblname = atbl->getName();
+                string tblID = tblname + "_addressTableObject";
+                pf_comp->tables->registerTable(tblname,tblID,atbl);
+            }
+
+            re->removeRef(atbl);
+            re->addRef(mart);
+        }
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
+bool NATCompiler_pf::processMultiAddressObjectsInRE::processNext()
+{
+    NATCompiler_pf *pf_comp=dynamic_cast<NATCompiler_pf*>(compiler);
+    NATRule *rule=getNext(); if (rule==NULL) return false;
+
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+    bool neg = re->getNeg();
+
+    list<FWObject*> cl;
+
+    try
+    {
+        for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+        {
+            FWObject *o= *i;
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+
+            MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+            if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+            {
+                if (re->size()>1 && neg)
+                {
+                    string err = "AddressTable object can not be used with negation in combination with other objects in the same rule element. Rule ";
+                    err += rule->getLabel();
+                    compiler->abort(err);
+                }
+                o->setBool("pf_table",true);
+                string tblname = o->getName();
+                string tblID = tblname + "_addressTableObject";
+                pf_comp->tables->registerTable(tblname,tblID,o);
+                cl.push_back(o);
+            }
+        }
+    } catch(FWException &ex)  // TableFactory::registerTable throws exception
+    {
+        string err;
+        err = "Can not process MultiAddress object in rule " +
+            rule->getLabel() + " : " + ex.toString();
+        compiler->abort( err );
+    }
+
+    if (!cl.empty())
+    {
+        RuleElement *nre;
+
+        for (FWObject::iterator i=cl.begin(); i!=cl.end(); i++)
+        {
+            NATRule *r= NATRule::cast(compiler->dbcopy->create(NATRule::TYPENAME,false) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            nre=RuleElement::cast( r->getFirstByType(re_type) );
+            nre->clearChildren();
+            nre->addRef( *i );
+            tmp_queue.push_back(r);
+        }
+
+        for (FWObject::iterator i=cl.begin(); i!=cl.end(); i++)
+            re->removeRef( *i );
+
+        if (!re->isAny())
+            tmp_queue.push_back(rule);
+
+        return true;
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
+
 void NATCompiler_pf::checkForDynamicInterfacesOfOtherObjects::findDynamicInterfaces(RuleElement *re,
                                                                                      Rule        *rule)
 {
@@ -640,7 +839,7 @@ void NATCompiler_pf::checkForDynamicInterfacesOfOtherObjects::findDynamicInterfa
     {
         FWObject *o   = *i1;
         FWObject *obj = o;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Interface  *ifs   =Interface::cast( obj );
 
         if (ifs!=NULL && ifs->isDyn() && ! ifs->isChildOf(compiler->fw))        
@@ -670,6 +869,29 @@ bool NATCompiler_pf::checkForDynamicInterfacesOfOtherObjects::processNext()
     return true;
 }
 
+bool NATCompiler_pf::createTables::processNext()
+{
+    NATCompiler_pf *pf_comp=dynamic_cast<NATCompiler_pf*>(compiler);
+    NATRule *rule=getNext(); if (rule==NULL) return false;
+
+    RuleElementOSrc *osrc=rule->getOSrc();
+    RuleElementODst *odst=rule->getODst();
+
+    if (osrc->size()!=1) pf_comp->tables->createTablesForRE(osrc,rule);
+    if (odst->size()!=1) pf_comp->tables->createTablesForRE(odst,rule);
+
+#if 0
+    RuleElementTSrc *tsrc=rule->getTSrc();
+    RuleElementTDst *tdst=rule->getTDst();
+
+    if (tsrc->size()!=1) pf_comp->tables->createTablesForRE(tsrc,rule);
+    if (tdst->size()!=1) pf_comp->tables->createTablesForRE(tdst,rule);
+#endif
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
 
 void NATCompiler_pf::compile()
 {
@@ -682,7 +904,7 @@ void NATCompiler_pf::compile()
 
 	Compiler::compile();
 
-        add( new Begin("Begin processing") );
+        add( new Begin());
         add( new printTotalNumberOfRules() );
 
         add( new recursiveGroupsInOSrc("check for recursive groups in OSRC") );
@@ -706,26 +928,31 @@ void NATCompiler_pf::compile()
         add( new eliminateDuplicatesInODST(  "eliminate duplicates in ODST") );
         add( new eliminateDuplicatesInOSRV(  "eliminate duplicates in OSRV") );
 
+        add( new swapMultiAddressObjectsInOSrc(" swap MultiAddress -> MultiAddressRunTime in OSrc") );
+        add( new swapMultiAddressObjectsInODst(" swap MultiAddress -> MultiAddressRunTime in ODst") );
+
+        add( new swapAddressTableObjectsInOSrc("AddressTable -> MultiAddressRunTime in OSrc") );
+        add( new swapAddressTableObjectsInODst("AddressTable -> MultiAddressRunTime in ODst") );
+
+        add( new processMultiAddressObjectsInOSrc("process MultiAddress objects in OSrc") );
+        add( new processMultiAddressObjectsInODst("process MultiAddress objects in ODst") );
+
         add( new splitOnOSrv( "split rule on original service" ) );
         add( new fillTranslatedSrv( "fill translated service" ) );
 
-        add( new doOSrcNegation( "process negation in OSrc" ) );
-        add( new doODstNegation( "process negation in ODst" ) );
-        add( new doOSrvNegation( "process negation in OSrv" ) );
+        //add( new doOSrcNegation( "process negation in OSrc" ) );
+        //add( new doODstNegation( "process negation in ODst" ) );
+        //add( new doOSrvNegation( "process negation in OSrv" ) );
 
         add( new NATRuleType( "determine NAT rule types" ) );
         add( new splitSDNATRule("split SDNAT rules"      ) );
         add( new NATRuleType( "determine NAT rule types" ) );
         add( new VerifyRules( "verify NAT rules" ) );
-        add( new splitODstForSNAT("split rule if objects in ODst belong to different subnets"    ) );
+        //add( new splitODstForSNAT("split rule if objects in ODst belong to different subnets"    ) );
         add( new ReplaceFirewallObjectsODst( "replace references to the firewall in ODst" ) );
         add( new ReplaceFirewallObjectsTSrc( "replace references to the firewall in TSrc" ) );
         add( new ReplaceObjectsTDst( "replace objects in TDst" ) );
 
-//        add( new doOSrcNegation( "process negation in OSrc" ) );
-//        add( new doODstNegation( "process negation in ODst" ) );
-//        add( new doOSrvNegation( "process negation in OSrv" ) );
-
 	if ( manage_virtual_addr ) {
             add( new addVirtualAddress( "add virtual addresses for NAT rules" ) );
 	}
@@ -734,9 +961,14 @@ void NATCompiler_pf::compile()
         add( new checkForUnnumbered("check for unnumbered interfaces" ) );
         add( new checkForDynamicInterfacesOfOtherObjects( "check for dynamic interfaces of other hosts and firewalls" ) );
         add( new ExpandAddressRanges( "expand address range objects" ) );
-        add( new ConvertToAtomic( "convert to atomic rules" ) );
+        //add( new ConvertToAtomicForTSrc( "convert to atomic rules" ) );
+        add( new splitForTSrc("split if addresses in TSrc belong to different networks" ) );
         add( new AssignInterface( "assign rules to interfaces" ) );
         add( new convertInterfaceIdToStr("prepare interface assignments") );
+
+        add( new createTables(      "create tables"    ) );
+//        add( new PrintTables(       "print tables"     ) );
+
         add( new PrintRule("generate pf code") );
         add( new simplePrintProgress() );
 
@@ -752,5 +984,4 @@ void NATCompiler_pf::compile()
 
 void NATCompiler_pf::epilog()
 {
-    cout << _(" NAT rules compiled successfully \n") << flush;
 }
