@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipf.cpp,v 1.2 2005/04/04 07:00:54 vkurland Exp $
+  $Id: PolicyCompiler_ipf.cpp,v 1.8 2006/05/17 02:30:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,6 +36,7 @@
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/AddressTable.h"
 
 #include <iostream>
 
@@ -51,7 +52,7 @@ int PolicyCompiler_ipf::prolog()
 {
     int n= PolicyCompiler_pf::prolog();
 
-    FWObject    *grp;
+//    FWObject    *grp;
     anytcp=TCPService::cast(dbcopy->create(TCPService::TYPENAME,false) );
     anytcp->setId(ANY_TCP_OBJ_ID);
 //    grp=dbcopy->getById( dbcopy->std.TCPServicesId , true );
@@ -92,7 +93,7 @@ bool PolicyCompiler_ipf::expandAnyService::processNext()
 	r->duplicate(rule);
 	RuleElementSrv *nsrv=r->getSrv();
 	nsrv->clearChildren();
-	nsrv->addRef(compiler->getCachedObj(ANY_ICMP_OBJ_ID));
+	nsrv->addRef(compiler->dbcopy->findInIndex(ANY_ICMP_OBJ_ID));
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
@@ -101,7 +102,7 @@ bool PolicyCompiler_ipf::expandAnyService::processNext()
 	r->duplicate(rule);
 	nsrv=r->getSrv();
 	nsrv->clearChildren();
-	nsrv->addRef(compiler->getCachedObj(ANY_TCP_OBJ_ID));
+	nsrv->addRef(compiler->dbcopy->findInIndex(ANY_TCP_OBJ_ID));
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
@@ -110,7 +111,7 @@ bool PolicyCompiler_ipf::expandAnyService::processNext()
 	r->duplicate(rule);
 	nsrv=r->getSrv();
 	nsrv->clearChildren();
-	nsrv->addRef(compiler->getCachedObj(ANY_UDP_OBJ_ID));
+	nsrv->addRef(compiler->dbcopy->findInIndex(ANY_UDP_OBJ_ID));
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
@@ -224,10 +225,10 @@ void PolicyCompiler_ipf::specialCaseWithDynInterface::dropDynamicInterface(RuleE
     {
         FWObject *o   = *i1;
         FWObject *obj = o;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Interface  *ifs   =Interface::cast( obj );
 
-        if (ifs!=NULL && (ifs->isDyn() || ifs->isUnnumbered())) continue;
+        if (ifs!=NULL && !ifs->isRegular()) continue;
         cl.push_back(obj);
     }
     if (!cl.empty()) 
@@ -255,7 +256,7 @@ PolicyCompiler_ipf::calculateSkip::calculateSkip(const std::string &n) : PolicyR
 
 bool PolicyCompiler_ipf::calculateSkip::processNext()
 {
-    PolicyRule *rule;
+//    PolicyRule *rule;
 
     slurp();
     if (tmp_queue.size()==0) return false;
@@ -365,6 +366,24 @@ bool PolicyCompiler_ipf::eliminateDuplicateRules::processNext()
     return true;
 }
 
+bool PolicyCompiler_ipf::processMultiAddressObjectsInRE::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+        if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+            compiler->abort("Run-time AddressTable objects are not supported. Rule " + rule->getLabel());
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
 
 void PolicyCompiler_ipf::compile()
 {
@@ -379,9 +398,12 @@ void PolicyCompiler_ipf::compile()
 
         if ( fw->getOptionsObject()->getBool ("check_shading") ) 
         {
-            add( new Begin                       (" Detecting rule shadowing"                ) );
+            add( new Begin                       ("Detecting rule shadowing"              ) );
             add( new printTotalNumberOfRules     (                                         ) );
 
+            add( new ItfNegation(            "process negation in Itf"  ) );
+            add( new InterfacePolicyRules("process interface policy rules and store interface ids") );
+
             add( new recursiveGroupsInSrc(       "check for recursive groups in SRC"     ) );
             add( new recursiveGroupsInDst(       "check for recursive groups in DST"     ) );
             add( new recursiveGroupsInSrv(       "check for recursive groups in SRV"     ) );
@@ -402,7 +424,7 @@ void PolicyCompiler_ipf::compile()
         }
 
 
-        add( new Begin(                       "Begin processing"                                ) );
+        add( new Begin());
         add( new printTotalNumberOfRules() );
 //        add( new MACFiltering(                "verify for MAC address filtering"                ) );
         add( new setQuickFlag(                "set 'quick' flag"                                ) );
@@ -418,6 +440,9 @@ void PolicyCompiler_ipf::compile()
         add( new emptyGroupsInDst(           "check for empty groups in DST"         ) );
         add( new emptyGroupsInSrv(           "check for empty groups in SRV"         ) );
 
+        add( new ItfNegation(            "process negation in Itf"  ) );
+        add( new InterfacePolicyRules("process interface policy rules and store interface ids") );
+
         add( new doSrcNegation(               "process negation in Src"                         ) );
         add( new doDstNegation(               "process negation in Dst"                         ) );
         add( new doSrvNegation(               "process negation in Srv"                         ) );
@@ -426,6 +451,12 @@ void PolicyCompiler_ipf::compile()
         add( new eliminateDuplicatesInDST(    "eliminate duplicates in DST"                     ) );
         add( new eliminateDuplicatesInSRV(    "eliminate duplicates in SRV"                     ) );
 
+        add( new swapMultiAddressObjectsInSrc(" swap MultiAddress -> MultiAddressRunTime in Src") );
+        add( new swapMultiAddressObjectsInDst(" swap MultiAddress -> MultiAddressRunTime in Dst") );
+
+        add( new processMultiAddressObjectsInSrc("process MultiAddress objects in Src") );
+        add( new processMultiAddressObjectsInDst("process MultiAddress objects in Dst") );
+
         add( new splitIfFirewallInSrc(        "split rule if firewall is in Src"                ) );
         add( new splitIfFirewallInDst(        "split rule if firewall is in Dst"                ) );
         add( new fillDirection(               "determine directions"                            ) );
@@ -477,7 +508,7 @@ void PolicyCompiler_ipf::compile()
 string PolicyCompiler_ipf::debugPrintRule(Rule *r)
 {
     PolicyRule *rule=PolicyRule::cast(r);
-    FWOptions  *ruleopt =rule->getOptionsObject();
+//    FWOptions  *ruleopt =rule->getOptionsObject();
 
     string  iface = rule->getInterfaceId();
     if (iface!="") {
@@ -496,5 +527,4 @@ string PolicyCompiler_ipf::debugPrintRule(Rule *r)
 
 void PolicyCompiler_ipf::epilog()
 {
-    cout << _(" Policy compiled successfully \n") << flush;
 }
