@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw.cpp,v 1.4 2005/04/04 07:00:54 vkurland Exp $
+  $Id: PolicyCompiler_ipfw.cpp,v 1.8 2006/05/17 02:30:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -82,7 +82,7 @@ void PolicyCompiler_ipfw::_expandAddr(Rule *rule,FWObject *s)
     if (re!=NULL && re->size()==1 )
     {
         FWObject *o=re->front();
-        if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
         if (o->getId()==fw->getId()) return;
     }
@@ -104,7 +104,7 @@ bool PolicyCompiler_ipfw::expandAnyService::processNext()
 	r->duplicate(rule);
 	RuleElementSrv *nsrv=r->getSrv();
 	nsrv->clearChildren();
-	nsrv->addRef(compiler->getCachedObj(ANY_ICMP_OBJ_ID));
+	nsrv->addRef(compiler->dbcopy->findInIndex(ANY_ICMP_OBJ_ID));
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
@@ -113,7 +113,7 @@ bool PolicyCompiler_ipfw::expandAnyService::processNext()
 	r->duplicate(rule);
 	nsrv=r->getSrv();
 	nsrv->clearChildren();
-	nsrv->addRef(compiler->getCachedObj(ANY_TCP_OBJ_ID));
+	nsrv->addRef(compiler->dbcopy->findInIndex(ANY_TCP_OBJ_ID));
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
@@ -122,7 +122,7 @@ bool PolicyCompiler_ipfw::expandAnyService::processNext()
 	r->duplicate(rule);
 	nsrv=r->getSrv();
 	nsrv->clearChildren();
-	nsrv->addRef(compiler->getCachedObj(ANY_UDP_OBJ_ID));
+	nsrv->addRef(compiler->dbcopy->findInIndex(ANY_UDP_OBJ_ID));
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
@@ -245,7 +245,7 @@ bool PolicyCompiler_ipfw::separatePortRanges::processNext()
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
     {	    
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast(o);
 	assert(s!=NULL);
 
@@ -301,8 +301,7 @@ bool PolicyCompiler_ipfw::sortTCPUDPServices::processNext()
     }
 
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *s1= Service::cast(o);
     if ( !UDPService::isA(s1) && !TCPService::isA(s1))
@@ -321,7 +320,7 @@ bool PolicyCompiler_ipfw::sortTCPUDPServices::processNext()
     {	    
 	FWObject *o= *i;
 	if (FWReference::cast(o)!=NULL)
-            o=compiler->getCachedObj(o->getStr("ref"));
+            o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast(o);
 	assert(s!=NULL);
 
@@ -363,10 +362,10 @@ void PolicyCompiler_ipfw::specialCaseWithDynInterface::dropDynamicInterface(Rule
     {
         FWObject *o   = *i1;
         FWObject *obj = o;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Interface  *ifs   =Interface::cast( obj );
 
-        if (ifs!=NULL && (ifs->isDyn() || ifs->isUnnumbered())) continue;
+        if (ifs!=NULL && !ifs->isRegular()) continue;
         cl.push_back(obj);
     }
     if (!cl.empty()) 
@@ -492,9 +491,12 @@ void PolicyCompiler_ipfw::compile()
 
         if ( fw->getOptionsObject()->getBool("check_shading")) 
         {
-            add( new Begin(" Detecting rule shadowing"));
+            add( new Begin("Detecting rule shadowing"));
             add( new printTotalNumberOfRules(                                         ));
 
+            add( new ItfNegation(         "process negation in Itf"  ) );
+            add( new InterfacePolicyRules("process interface policy rules and store interface ids") );
+
             add( new recursiveGroupsInSrc("check for recursive grps in SRC"));
             add( new recursiveGroupsInDst("check for recursive grps in DST"));
             add( new recursiveGroupsInSrv("check for recursive grps in SRV"));
@@ -517,7 +519,7 @@ void PolicyCompiler_ipfw::compile()
         }
 
 
-        add( new Begin("Begin processing"));
+        add( new Begin());
         add( new printTotalNumberOfRules());
 
         if (check_for_recursive_groups)
@@ -531,6 +533,9 @@ void PolicyCompiler_ipfw::compile()
         add( new emptyGroupsInDst("check for empty grps in DST"));
         add( new emptyGroupsInSrv("check for empty grps in SRV"));
 
+        add( new ItfNegation(         "process negation in Itf"  ) );
+        add( new InterfacePolicyRules("process interface policy rules and store interface ids") );
+
         add( new doSrcNegation("process negation in Src"));
         add( new doDstNegation("process negation in Dst"));
         add( new doSrvNegation("process negation in Srv"));
@@ -539,6 +544,10 @@ void PolicyCompiler_ipfw::compile()
         add( new eliminateDuplicatesInDST("eliminate duplicates in DST"));
         add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
 
+        add( new swapMultiAddressObjectsInSrc(" swap MultiAddress -> MultiAddressRunTime in Src") );
+        add( new swapMultiAddressObjectsInDst(" swap MultiAddress -> MultiAddressRunTime in Dst") );
+
+
         add( new splitIfFirewallInSrc("split rule if firewall is in Src"));
         add( new splitIfFirewallInDst("split rule if firewall is in Dst"));
         add( new fillDirection("determine directions"));
@@ -595,5 +604,4 @@ string PolicyCompiler_ipfw::debugPrintRule(Rule *r)
 
 void PolicyCompiler_ipfw::epilog()
 {
-    cout << _(" Policy compiled successfully \n") << flush;
 }
