@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf.cpp,v 1.8 2005/05/30 22:09:19 vkurland Exp $
+  $Id: PolicyCompiler_pf.cpp,v 1.22 2006/05/21 04:47:10 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -38,6 +38,7 @@
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Network.h"
+#include "fwbuilder/AddressTable.h"
 
 #include <algorithm>
 #include <functional>
@@ -79,10 +80,156 @@ int PolicyCompiler_pf::prolog()
         }
     }
 
+    if (tables)
+    {
+        tables->init(dbcopy);
+        tables->setRuleSetName(getRuleSetName());
+    }
 
-    return PolicyCompiler::prolog();
+    return  PolicyCompiler::prolog();
 }
 
+
+bool PolicyCompiler_pf::swapAddressTableObjectsInRE::processNext()
+{
+    PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
+    Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
+
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+
+    list<MultiAddress*> cl;
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        /* 
+         * All addressTable objects will be run-time here because we
+         * switch them in preprocessor. The difference is: if address
+         * table was originally run-time, at this point it will have
+         * no children, however if it was compile-time originally, it
+         * will have children objects.  That is how we distinguish
+         * them in this rule processor. Here we only deal with
+         * AddressTable objects that originally used to be
+         * compile-time because we need to create tables for them.
+         */
+        if (AddressTable::cast(o)!=NULL &&
+            AddressTable::cast(o)->isRunTime() &&
+            o->size() > 0)
+            cl.push_back(MultiAddress::cast(o));
+    }
+
+    if (!cl.empty())
+    {
+        for (list<MultiAddress*>::iterator i=cl.begin(); i!=cl.end(); i++)
+        {
+            MultiAddress *atbl = *i;
+
+            string mart_id = atbl->getId()+"_runtime";
+            MultiAddressRunTime *mart = 
+                MultiAddressRunTime::cast(compiler->dbcopy->findInIndex(mart_id));
+            if (mart==NULL)
+            {
+                mart = new MultiAddressRunTime(atbl);
+
+                // need to ensure stable ID for the runtime object, so
+                // that when the same object is replaced in different
+                // rulesets by different compiler passes, chosen
+                // runtime object has the same ID and is identified as
+                // the same by the compiler.
+
+                mart->setId( mart_id );
+                compiler->dbcopy->addToIndex(mart);
+                compiler->dbcopy->add(mart);
+
+// register this object as a table
+                string tblname = atbl->getName();
+                string tblID = tblname + "_addressTableObject";
+                pf_comp->tables->registerTable(tblname,tblID,atbl);
+            }
+
+            re->removeRef(atbl);
+            re->addRef(mart);
+        }
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
+bool PolicyCompiler_pf::processMultiAddressObjectsInRE::processNext()
+{
+    PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+    bool neg = re->getNeg();
+
+    list<FWObject*> maddr_runtime;
+
+    try
+    {
+        for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+        {
+            FWObject *o= *i;
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+
+            MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+            if (atrt!=NULL &&
+                atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+            {
+                if (re->size()>1 && neg)
+                {
+                    string err = "AddressTable object can not be used with negation in combination with other objects in the same rule element. Rule ";
+                    err += rule->getLabel();
+                    compiler->abort(err);
+                }
+                string tblname = o->getName();
+                string tblID = tblname + "_addressTableObject";
+                pf_comp->tables->registerTable(tblname,tblID,o);
+                o->setBool("pf_table",true);
+                maddr_runtime.push_back(o);
+            }
+        }
+    } catch(FWException &ex)  // TableFactory::registerTable throws exception
+    {
+        string err;
+        err = "Can not process MultiAddress object in rule " +
+            rule->getLabel() + " : " + ex.toString();
+        compiler->abort( err );
+    }
+
+    if (!maddr_runtime.empty())
+    {
+        RuleElement *nre;
+
+        for (FWObject::iterator i=maddr_runtime.begin(); i!=maddr_runtime.end(); i++)
+        {
+            PolicyRule *r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            nre=RuleElement::cast( r->getFirstByType(re_type) );
+            nre->clearChildren();
+            nre->addRef( *i );
+            tmp_queue.push_back(r);
+        }
+
+        for (FWObject::iterator i=maddr_runtime.begin(); i!=maddr_runtime.end(); i++)
+            re->removeRef( *i );
+
+        if (!re->isAny())
+            tmp_queue.push_back(rule);
+
+        return true;
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
 bool PolicyCompiler_pf::splitIfFirewallInSrc::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
@@ -102,7 +249,7 @@ bool PolicyCompiler_pf::splitIfFirewallInSrc::processNext()
 	FWObject *o   = *i1;
 	FWObject *obj = NULL;
 //	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-	if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
 	if (obj==NULL)  throw FWException(_("Broken Src object in rule: ")+rule->getLabel());
 
 	if (obj->getId()==compiler->getFwId()) {
@@ -142,12 +289,12 @@ bool PolicyCompiler_pf::splitIfFirewallInDst::processNext()
 
     FWObject       *fw_in_dst=NULL;
     vector<FWObject*> cl;
-    for (FWObject::iterator i1=dst->begin(); i1!=dst->end(); ++i1) {
-
+    for (FWObject::iterator i1=dst->begin(); i1!=dst->end(); ++i1)
+    {
 	FWObject *o   = *i1;
 	FWObject *obj = NULL;
 //	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-	if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
 	if (obj==NULL) throw FWException(_("Broken Dst in rule: ")+rule->getLabel());
 
 	if (obj->getId()==compiler->getFwId()) {
@@ -178,9 +325,24 @@ bool PolicyCompiler_pf::fillDirection::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     tmp_queue.push_back(rule);
 
-    if (rule->getDirectionAsString()=="") 
+/* after interface policies have been merged with global policy, rules
+ * with empty direction have disappeared.  In fact, xslt
+ * transformation 2.1.1->2.1.2 leaves empty direction in the old
+ * global policy rules, but the GUI promptly fixes that replacing it
+ * with "Both" whenever user opens the policy. So, we have to handle
+ * both the case of an empty direction and direction "Both". To
+ * preserve old semantics as accurately as possible, I check for a
+ * combination of empty interface and direction "Both", this is what
+ * old global rules become after user opens the combined policy in the
+ * GUI. In fact, it does not matter what direction is set in the rule
+ * as long as it misses interface - we need to determine direction
+ * again anyway.
+ */
+
+    if (rule->getDirectionAsString()=="" || rule->getInterfaceId()=="" )
     {
-	if ( compiler->getCachedFwOpt()->getBool("pass_all_out") )  rule->setDirection( PolicyRule::Inbound );
+	if ( compiler->getCachedFwOpt()->getBool("pass_all_out") )
+            rule->setDirection( PolicyRule::Inbound );
 	else 
         {
 	    rule->setDirection( PolicyRule::Both );
@@ -189,20 +351,30 @@ bool PolicyCompiler_pf::fillDirection::processNext()
 	    Address  *dst=compiler->getFirstDst(rule);
 	    string    fwid=compiler->getFwId();
 
+            if (src==NULL || dst==NULL)
+                compiler->abort("Broken src or dst in rule "+rule->getLabel());
+
             if (!src->isAny() && !dst->isAny() &&
                 compiler->complexMatch(compiler->fw, src) &&
                 compiler->complexMatch(compiler->fw, dst)) return true;
 
-            if (!src->isAny() && compiler->complexMatch(compiler->fw, src)) rule->setDirection( PolicyRule::Outbound );
-            if (!dst->isAny() && compiler->complexMatch(compiler->fw, dst)) rule->setDirection( PolicyRule::Inbound );
+            if (!src->isAny() && compiler->complexMatch(compiler->fw, src))
+                rule->setDirection( PolicyRule::Outbound );
+            if (!dst->isAny() && compiler->complexMatch(compiler->fw, dst))
+                rule->setDirection( PolicyRule::Inbound );
 	}
     }
     return true;
 }
 
-
+/*
+ * add default and auto-generated rules, except when we process rules
+ * in a branch (i.e. anchor, since this is PF)
+ */
 void PolicyCompiler_pf::addDefaultPolicyRule()
 {
+    if (!getRuleSetName().empty()) return;
+
     if ( getCachedFwOpt()->getBool("mgmt_ssh") &&
          !getCachedFwOpt()->getStr("mgmt_addr").empty() )
     {
@@ -210,6 +382,7 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
         TCPService *ssh=TCPService::cast(dbcopy->create(TCPService::TYPENAME,false) );
         ssh->setInt("dst_range_start",22);
         ssh->setInt("dst_range_end",22);
+        ssh->setName("mgmt_ssh");
         dbcopy->add(ssh,false);
         cacheObj(ssh); // to keep cache consistent
 
@@ -244,6 +417,7 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
         }
 
         Network *mgmt_workstation = Network::cast(dbcopy->create(Network::TYPENAME,false));
+        mgmt_workstation->setName("mgmt_addr");
         mgmt_workstation->setAddress( addr );
         mgmt_workstation->setNetmask( netmask );
 //        IPv4 *mgmt_workstation = IPv4::cast(dbcopy->create(IPv4::TYPENAME,false));
@@ -257,7 +431,7 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
 	r->setAction(PolicyRule::Accept);
 	r->setLogging(false);
 	r->setDirection(PolicyRule::Inbound);
-	r->setPosition(-1);
+	r->setPosition(9999);
         r->setComment("   backup ssh access rule ");
         r->setHidden(true);
         r->setFallback(false);
@@ -331,7 +505,7 @@ bool PolicyCompiler_pf::SpecialServices::processNext()
 
     for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast( o );
 	assert(s);
 
@@ -431,7 +605,10 @@ bool PolicyCompiler_pf::setQuickFlag::processNext()
     tmp_queue.push_back(rule);
 
     if ( rule->getAction()!=PolicyRule::Scrub &&
-         rule->getAction()!=PolicyRule::Accounting ) rule->setBool("quick",true);
+         rule->getAction()!=PolicyRule::Accounting &&
+         rule->getAction()!=PolicyRule::Tag &&
+         rule->getAction()!=PolicyRule::Branch
+    ) rule->setBool("quick",true);
 
     return true;
 }
@@ -530,7 +707,7 @@ bool PolicyCompiler_pf::addLoopbackForRedirect::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
 
-    RuleElementSrc *src=rule->getSrc();
+//    RuleElementSrc *src=rule->getSrc();
     RuleElementDst *dst=rule->getDst();
     RuleElementSrv *srv=rule->getSrv();
 
@@ -554,14 +731,14 @@ bool PolicyCompiler_pf::addLoopbackForRedirect::processNext()
     for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) 
     {
 	FWObject *o1= *i;
-	if (FWReference::cast(o1)!=NULL) o1=compiler->getCachedObj(o1->getStr("ref"));
+	if (FWReference::cast(o1)!=NULL) o1=FWReference::cast(o1)->getPointer();
 	Service *s=Service::cast( o1 );
 	assert(s);
 
         for (FWObject::iterator j=dst->begin(); j!=dst->end(); j++) 
         {
             FWObject *o2= *j;
-            if (FWReference::cast(o2)!=NULL) o2=compiler->getCachedObj(o2->getStr("ref"));
+            if (FWReference::cast(o2)!=NULL) o2=FWReference::cast(o2)->getPointer();
             Address *a=Address::cast( o2 );
             assert(a);
 
@@ -591,7 +768,7 @@ void PolicyCompiler_pf::checkForDynamicInterfacesOfOtherObjects::findDynamicInte
     {
         FWObject *o   = *i1;
         FWObject *obj = o;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Interface  *ifs   =Interface::cast( obj );
 
         if (ifs!=NULL    &&
@@ -639,7 +816,7 @@ bool PolicyCompiler_pf::splitIfInterfaceInRE::processNext()
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
     {
         FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
         Interface *interface_=Interface::cast(o);
         if (interface_!=NULL && interface_->isDyn())
@@ -687,7 +864,7 @@ bool PolicyCompiler_pf::separateSrcPort::processNext()
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
 	    
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast(o);
 	assert(s!=NULL);
 
@@ -719,96 +896,17 @@ bool PolicyCompiler_pf::separateSrcPort::processNext()
     return true;
 }
 
-
-FWObject* PolicyCompiler_pf::createTables::findTable(RuleElement *re)
-{
-    PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
-    list<FWObject*> relement;
-    for (FWObject::iterator i1=re->begin(); i1!=re->end(); ++i1) 
-    {
-        FWObject *o   = *i1;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
-        relement.push_back(o);
-    }
-
-    for (FWObject::iterator i=pf_comp->tables.begin(); i!=pf_comp->tables.end(); ++i)
-    {
-        FWObject *tbl=*i;
-        if (tbl->size()==0 || (tbl->size()!=re->size()) ) continue;
-
-        bool match=true;
-        for (FWObject::iterator i1=tbl->begin(); i1!=tbl->end(); ++i1) 
-        {
-            FWObject *o   = *i1;  // table holds pointers to objects rather than references
-            if ( find(relement.begin(), relement.end(), o)==relement.end() )
-            {
-                match=false;
-                break;
-            }
-        }
-        if (match) return tbl;
-    }
-    return NULL;
-}
-
-void PolicyCompiler_pf::createTables::createTablesForRE(RuleElement *re,
-                                                        Rule        *rule)
-{
-    PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
-    Interface      *rule_iface = compiler->getCachedFwInterface(rule->getInterfaceId());
-
-    /*
-     * Before we create a new table, we scan tables and try to find
-     * the one that already exists and contains the same objects.
-     */
-
-    FWObject *tblgrp=findTable(re);
-
-    if (tblgrp==NULL)
-    {
-        tblgrp=ObjectGroup::cast(compiler->dbcopy->create(ObjectGroup::TYPENAME,false));
-
-// TODO: can two rules yeild the same name for the group using this method?
-        string tname="";
-/*
-        if (rule_iface)
-        {
-            if ( ! rule_iface->getLabel().empty() ) tname=rule_iface->getLabel()+".";
-            else tname=rule_iface->getName()+".";
-        }
-*/
-        tname=tname+rule->getId();
-        if (RuleElementSrc::isA(re)) tname=tname+".1";
-        if (RuleElementDst::isA(re)) tname=tname+".2";
-
-        tblgrp->setName( tname );
-
-        compiler->dbcopy->add(tblgrp,false);
-        compiler->cacheObj(tblgrp);
-        tblgrp->setBool("pf_table",true);
-        pf_comp->tables.push_back( tblgrp );
-
-        for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
-        {
-            FWObject *o= *i;
-            if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
-            tblgrp->add( o );
-        }
-    }
-    re->clearChildren();
-    re->addRef(tblgrp);
-}
-
 bool PolicyCompiler_pf::createTables::processNext()
 {
+    PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
     RuleElementSrc *src=rule->getSrc();
     RuleElementDst *dst=rule->getDst();
 //    RuleElementSrv *srv=rule->getSrv();
 
-    if (src->size()!=1) createTablesForRE(src,rule);
-    if (dst->size()!=1) createTablesForRE(dst,rule);
+    if (src->size()!=1) pf_comp->tables->createTablesForRE(src,rule);
+    if (dst->size()!=1) pf_comp->tables->createTablesForRE(dst,rule);
 //    if (srv->size()!=1) createTablesForRE(srv,rule);
 
     tmp_queue.push_back(rule);
@@ -836,8 +934,6 @@ bool PolicyCompiler_pf::printScrubRule::processNext()
 
 void PolicyCompiler_pf::compile()
 {
-    cout << _(" Compiling policy for ") << fw->getName() << " ..." <<  endl << flush;
-
     try 
     {
 	Compiler::compile();
@@ -847,9 +943,12 @@ void PolicyCompiler_pf::compile()
 
         if ( fw->getOptionsObject()->getBool ("check_shading") ) 
         {
-            add( new Begin                       (" Detecting rule shadowing" ) );
+            add( new Begin                       ("Detecting rule shadowing" ) );
             add( new printTotalNumberOfRules     () );
 
+            add( new ItfNegation(            "process negation in Itf"  ) );
+            add( new InterfacePolicyRules("process interface policy rules and store interface ids") );
+
             add( new recursiveGroupsInSrc( "check for recursive groups in SRC") );
             add( new recursiveGroupsInDst( "check for recursive groups in DST") );
             add( new recursiveGroupsInSrv( "check for recursive groups in SRV") );
@@ -869,7 +968,7 @@ void PolicyCompiler_pf::compile()
             deleteRuleProcessors();
         }
 
-        add( new Begin(                 "Start processing rules"             ) );
+        add( new Begin() );
         add( new printTotalNumberOfRules() );
 
 //        add( new printScrubRule            (" Defragmentation"             ) );
@@ -886,12 +985,27 @@ void PolicyCompiler_pf::compile()
 
 //        add( new doSrcNegation(         "process negation in Src"          ) );
 //        add( new doDstNegation(         "process negation in Dst"          ) );
-////	add( new doSrvNegation(         "process negation in Srv"            ) );
+	add( new doSrvNegation(         "process negation in Srv"            ) );
+
+// ExpandGroups opens groups, as well as groups in groups etc.
         add( new ExpandGroups(          "expand groups"                      ) );
+
         add( new eliminateDuplicatesInSRC("eliminate duplicates in SRC"      ) );
         add( new eliminateDuplicatesInDST("eliminate duplicates in DST"      ) );
         add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV"      ) );
 
+        add( new swapAddressTableObjectsInSrc("AddressTable -> MultiAddressRunTime in Src") );
+        add( new swapAddressTableObjectsInDst("AddressTable -> MultiAddressRunTime in Dst") );
+
+        add( new swapMultiAddressObjectsInSrc("MultiAddress -> MultiAddressRunTime in Src") );
+        add( new swapMultiAddressObjectsInDst("MultiAddress -> MultiAddressRunTime in Dst") );
+
+        add( new processMultiAddressObjectsInSrc("process MultiAddress objects in Src") );
+        add( new processMultiAddressObjectsInDst("process MultiAddress objects in Dst") );
+
+        add( new ItfNegation(         "process negation in Itf"  ) );
+        add( new InterfacePolicyRules("process interface policy rules and store interface ids") );
+
 	add( new splitIfFirewallInSrc(  "split rule if firewall is in Src"   ) );
 	add( new splitIfFirewallInDst(  "split rule if firewall is in Dst"   ) );
 	add( new fillDirection(         "determine directions"               ) );
@@ -912,11 +1026,9 @@ void PolicyCompiler_pf::compile()
         add( new checkForZeroAddr(      "check for zero addresses"       ) );
         add( new convertInterfaceIdToStr("prepare interface assignments" ) );
 
-//        if (fw->getOptionsObject()->getBool("use_tables"))
-//        {
-            add( new createTables(      "create tables"    ) );
-            add( new PrintTables(       "print tables"     ) );
-//        }
+        add( new createTables(      "create tables"    ) );
+//        add( new PrintTables(       "print tables"     ) );
+
         add( new PrintRule(             "generate pf code" ) );
         add( new simplePrintProgress() );
 
@@ -931,5 +1043,4 @@ void PolicyCompiler_pf::compile()
 
 void PolicyCompiler_pf::epilog()
 {
-    cout << _(" Policy compiled successfully \n") << flush;
 }
