@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf_writers.cpp,v 1.9 2004/09/19 06:20:12 vkurland Exp $
+  $Id: PolicyCompiler_pf_writers.cpp,v 1.19 2006/06/17 21:53:54 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,12 +32,15 @@
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/CustomService.h"
+#include "fwbuilder/TagService.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/FWOptions.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/RuleElement.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
+#include "fwbuilder/DNSName.h"
+#include "fwbuilder/AddressTable.h"
 
 #include <iostream>
 #if __GNUC__ > 3 || \
@@ -72,8 +75,17 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
 
     switch (rule->getAction()) {
     case PolicyRule::Accept:  
-    case PolicyRule::Accounting:  compiler->output << "pass "; break;
-    case PolicyRule::Deny:        compiler->output << "block "; break;
+    case PolicyRule::Tag:
+    case PolicyRule::Classify:
+    case PolicyRule::Accounting:
+    case PolicyRule::Route:
+        compiler->output << "pass ";
+        break;
+
+    case PolicyRule::Deny:
+        compiler->output << "block ";
+        break;
+
     case PolicyRule::Reject: 
 	if (TCPService::isA(srv)) compiler->output << "block return-rst ";
 	else {
@@ -95,14 +107,75 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
 		code="return-icmp   ";
 
 	    compiler->output << "block " << code;
-
 	}
 	break;
-    case PolicyRule::Scrub:   compiler->output << "scrub   "; break;
-    default:		      compiler->output << rule->getActionAsString() << " ";
+    case PolicyRule::Scrub:
+        compiler->output << "scrub   ";
+        break;
+    case PolicyRule::Custom:
+        compiler->output << ruleopt->getStr("custom_str") << " ";
+        break;
+    case PolicyRule::Branch:
+        compiler->output << "anchor " << ruleopt->getStr("branch_name") << " ";
+        break;
+    default:
+        compiler->output << rule->getActionAsString() << " ";
     }
 }
 
+void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
+{
+    FWOptions *ruleopt =rule->getOptionsObject();
+
+    if (rule->getAction() == PolicyRule::Route)
+    {
+        if (ruleopt->getBool("pf_fastroute"))
+            compiler->output << "fastroute ";
+
+        string prefix = "pf";
+        if (compiler->myPlatformName()=="ipf")
+            prefix="ipf";
+
+        string ro = ruleopt->getStr(prefix+"_route_option");
+        string roif = ruleopt->getStr(prefix+"_route_opt_if");
+        string roaddr = ruleopt->getStr(prefix+"_route_opt_addr");
+
+        if (!ro.empty())
+        {
+           if (roif.empty())
+                compiler->abort("Interface specification is required for action Route in rule "+rule->getLabel());
+
+            if (ro == "route_through")       compiler->output << "route-to ";
+            else if (ro == "route_reply_through") compiler->output << "reply-to ";
+            else if (ro == "route_copy_through")  compiler->output << "dup-to ";
+            else
+                compiler->abort("Unknown option for rule action Route: '" + 
+                                ro + "' in rule "+rule->getLabel());
+
+            compiler->output << "( ";
+            compiler->output << roif << " ";
+            if (!roaddr.empty())  compiler->output << roaddr << " ";
+            compiler->output << ") ";
+        }
+    }
+}
+
+void PolicyCompiler_pf::PrintRule::_printQueue(PolicyRule *rule)
+{
+    FWOptions *ruleopt =rule->getOptionsObject();
+
+    if (rule->getAction() == PolicyRule::Classify)
+        compiler->output << "queue " << ruleopt->getStr("classify_str") << " ";
+}
+
+void PolicyCompiler_pf::PrintRule::_printTag(PolicyRule *rule)
+{
+    FWOptions *ruleopt =rule->getOptionsObject();
+
+    if (rule->getAction() == PolicyRule::Tag)
+        compiler->output << "tag " << ruleopt->getStr("tagvalue") << " ";
+}
+
 void PolicyCompiler_pf::PrintRule::_printDirection(PolicyRule *rule)
 {
     if (rule->getDirection()==PolicyRule::Outbound)  compiler->output << "out "; 
@@ -162,7 +235,7 @@ string PolicyCompiler_pf::PrintRule::_printLogPrefix(PolicyRule *rule,
             s.replace(n,2,"global");
     }
     if (rule && (n=s.find("%C"))!=string::npos ) {
-        s.replace(n,2,rule->getStr("ipt_chain"));
+        s.replace(n,2,"");  // there is no chain in PF and friends
     }
 
     return "\"" + s + "\" ";
@@ -179,7 +252,11 @@ void PolicyCompiler_pf::PrintRule::_printInterface(PolicyRule *rule)
 void PolicyCompiler_pf::PrintRule::_printProtocol(libfwbuilder::Service *srv)
 {
 
-    if ( ! srv->isAny() && ! CustomService::isA(srv) && srv->getProtocolName()!="ip") {
+    if (!srv->isAny() &&
+        !CustomService::isA(srv) &&
+        !TagService::isA(srv) &&
+        srv->getProtocolName()!="ip")
+    {
 	compiler->output << "proto ";
 	compiler->output << srv->getProtocolName();
 	compiler->output << " ";
@@ -240,8 +317,7 @@ void PolicyCompiler_pf::PrintRule::_printSrcService(RuleElementSrv  *rel)
  * find the object. I'd rather use a cached copy in the compiler
  */
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *srv= Service::cast(o);
 
@@ -255,8 +331,7 @@ void PolicyCompiler_pf::PrintRule::_printSrcService(RuleElementSrv  *rel)
 	string str;
 	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
 	    FWObject *o= *i;
-//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-	    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    Service *s=Service::cast( o );
 	    assert(s);
 	    if (UDPService::isA(srv) || TCPService::isA(srv)) {
@@ -286,8 +361,7 @@ string PolicyCompiler_pf::PrintRule::_printSrcService(Service *srv,bool neg)
 void PolicyCompiler_pf::PrintRule::_printDstService(RuleElementSrv  *rel)
 {
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *srv= Service::cast(o);
 
@@ -321,8 +395,7 @@ void PolicyCompiler_pf::PrintRule::_printDstService(RuleElementSrv  *rel)
 	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
         {
 	    FWObject *o= *i;
-//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-	    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    Service *s=Service::cast( o );
 	    assert(s);
 	    string str1= _printDstService(s , rel->getNeg() );
@@ -366,6 +439,11 @@ string PolicyCompiler_pf::PrintRule::_printDstService(Service *srv,bool neg)
 	str << CustomService::cast(srv)->getCodeForPlatform( compiler->myPlatformName() ) << " ";
     }
 
+    if (TagService::isA(srv)) 
+    {
+	str << "tagged " << TagService::cast(srv)->getCode() << " ";
+    }
+
     return str.str();
 }
 
@@ -393,6 +471,22 @@ string PolicyCompiler_pf::PrintRule::_printTCPFlags(libfwbuilder::TCPService *sr
 
 void PolicyCompiler_pf::PrintRule::_printAddr(Address  *o,bool neg)
 {
+    MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+    if (atrt!=NULL)
+    {
+        if (atrt->getSubstitutionTypeName()==DNSName::TYPENAME)
+        {
+            compiler->output <<  atrt->getSourceName() << " ";
+            return;
+        }
+        if (atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+        {
+            compiler->output << "<" << o->getName() << "> ";
+            return;
+        }
+        assert(atrt==NULL);
+    }
+
     IPAddress addr=o->getAddress();
     Netmask   mask=o->getNetmask();
 
@@ -435,7 +529,7 @@ void PolicyCompiler_pf::PrintRule::_printAddrList(FWObject  *grp,bool negflag)
     {
         if (i!=grp->begin())  compiler->output << ", ";
         FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         Address *s=Address::cast( o );
         assert(s);
         _printAddr(s , negflag);
@@ -446,13 +540,19 @@ void PolicyCompiler_pf::PrintRule::_printAddrList(FWObject  *grp,bool negflag)
 void PolicyCompiler_pf::PrintRule::_printSrcAddr(RuleElementSrc  *rel)
 {
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    FWReference *oref = FWReference::cast(o);
+    if (o && oref!=NULL) o=oref->getPointer();
 
     Address *src= Address::cast(o);
 
     _printNegation(rel);
 
+    if (o==NULL)
+    {
+        PolicyRule *rule = PolicyRule::cast(rel->getParent());
+        compiler->abort("Broken rule element " + rel->getTypeName() + " in rule '" + rule->getLabel() + "'  rel->front(): " + oref->getPointerId());
+    }
+
     if (rel->size()==1 && ! o->getBool("pf_table") )
     {
 	_printAddr( src , rel->getNeg() );
@@ -472,13 +572,19 @@ void PolicyCompiler_pf::PrintRule::_printSrcAddr(RuleElementSrc  *rel)
 void PolicyCompiler_pf::PrintRule::_printDstAddr(RuleElementDst  *rel)
 {
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    FWReference *oref = FWReference::cast(o);
+    if (o && oref!=NULL) o=oref->getPointer();
 
     Address *dst= Address::cast(o);
 
     _printNegation(rel);
 
+    if (o==NULL)
+    {
+        PolicyRule *rule = PolicyRule::cast(rel->getParent());
+        compiler->abort("Broken rule element " + rel->getTypeName() + " in rule '" + rule->getLabel() + "'  rel->front(): " + oref->getPointerId());
+    }
+
     if (rel->size()==1 && ! o->getBool("pf_table") )
     {
 	_printAddr( dst , rel->getNeg());
@@ -511,6 +617,9 @@ bool PolicyCompiler_pf::PrintRule::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     FWOptions  *ruleopt =rule->getOptionsObject();
 
+//    cerr << endl;
+//    cerr << "Rule " << rule->getPosition() << endl;
+
     tmp_queue.push_back(rule);
 
     string rl=rule->getLabel();
@@ -533,6 +642,7 @@ bool PolicyCompiler_pf::PrintRule::processNext()
         current_rule_label=rl;
     }
 
+//    cerr << "CP 1" << endl;
 
 
 
@@ -546,13 +656,19 @@ bool PolicyCompiler_pf::PrintRule::processNext()
     _printAction(rule);
     _printDirection(rule);
     _printLogging(rule);
+
     if ( rule->getBool("quick") ) compiler->output << " quick ";
+
     _printInterface(rule);
 
+    _printRouteOptions(rule);
+
     compiler->output << "inet ";
 
     _printProtocol(srv);
 
+//    cerr << "CP 2" << endl;
+
     compiler->output << " from ";
     _printSrcAddr(srcrel);
     _printSrcService(srvrel);
@@ -561,6 +677,9 @@ bool PolicyCompiler_pf::PrintRule::processNext()
     _printDstAddr(dstrel);
     _printDstService(srvrel);
 
+    _printTag(rule);
+
+//    cerr << "CP 3" << endl;
 /* 
  * Dealing with "keep state" and "modulate state" flags
  *
@@ -569,7 +688,11 @@ bool PolicyCompiler_pf::PrintRule::processNext()
  *    all services in a rule are of the same protocol, therefore we can simply
  *    check type of srv
  */
-    if ( ! ruleopt->getBool("stateless") && rule->getAction()==PolicyRule::Accept)
+    if ( ! ruleopt->getBool("stateless") && 
+         (rule->getAction()==PolicyRule::Accept ||
+          rule->getAction()==PolicyRule::Tag ||
+          rule->getAction()==PolicyRule::Route)
+    )
     {
 
         TCPService *tcpsrv=TCPService::cast(srv);
@@ -611,6 +734,7 @@ bool PolicyCompiler_pf::PrintRule::processNext()
 
     if (rule->getBool("allow_opts")) compiler->output << "allow-opts  ";
 
+    _printQueue(rule);
     _printLabel(rule);
 
     compiler->output << endl;
@@ -626,47 +750,7 @@ bool PolicyCompiler_pf::PrintTables::processNext()
     if (tmp_queue.size()==0) return false;
 
 /* print tables */
-
-    compiler->output << endl;
-    compiler->output << endl;
-    compiler->output << "# Tables: (" << pf_comp->tables.size() << ")" << endl;
-    for (FWObject::iterator i=pf_comp->tables.begin(); i!=pf_comp->tables.end(); i++)
-    {
-        FWObject *grp=*i;
-        compiler->output << "table <" << grp->getName() << "> ";
-
-        compiler->output << "{ ";
-        for (FWObject::iterator i=grp->begin(); i!=grp->end(); i++)
-        {
-            if (i!=grp->begin())  compiler->output << ", ";
-            FWObject *o= *i;
-            if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
-            if (Interface::cast(o))
-            {
-                compiler->output << o->getName();
-            } else
-            {
-                Address *A=Address::cast( o );
-                assert(A);
-
-                IPAddress addr=A->getAddress();
-                Netmask   mask=A->getNetmask();
-
-                if (IPv4::cast(A)!=NULL) {
-                    mask=Netmask("255.255.255.255");
-                }
-
-                compiler->output << addr.toString();
-                if (mask.toString()!="255.255.255.255") {
-                    compiler->output << "/" << mask.getLength();
-                }
-            }
-            compiler->output << " ";
-        }
-        compiler->output << "} ";
-        compiler->output << endl;
-    }
-    compiler->output << endl;
+    compiler->output << pf_comp->tables->PrintTables();
 
     return true;
 }
